/*
 * aaccess.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <vector>
#include <sax/FromXMLParserHelper.h>

#include <factory/XmlDataFactory.hpp>
#include <exception/CommonException.h>

#include "AutomatonAccess.h"
#include "GrammarAccess.h"
#include "RegExpAccess.h"
#include "StringAccess.h"
#include "ExceptionAccess.h"
#include "TreeAccess.h"
#include "PairSetAccess.h"

#include "settings/OperationSettings.h"

#include <container/ObjectsSet.h>
#include <container/ObjectsPair.h>

int main ( int argc, char * argv[] ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Access internals of automata, grammars, regexps, ...", ' ', "0.01" );

		// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------
		std::vector < std::string > automatonSettings {
			AutomatonSettings::stringsVector ( )
		};

		TCLAP::ValuesConstraint < std::string > automatonPrintingOptionsVals ( automatonSettings );

		TCLAP::ValueArg < std::string > automatonInput ( "", "automaton", "Access components of automata", false, "", & automatonPrintingOptionsVals );

		// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------

		std::vector < std::string > grammarSettings {
			GrammarSettings::stringsVector ( )
		};

		TCLAP::ValuesConstraint < std::string > grammarPrintingOptionsVals ( grammarSettings );

		TCLAP::ValueArg < std::string > grammarInput ( "", "grammar", "Access components of grammars", false, "", & grammarPrintingOptionsVals );

		// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------

		std::vector < std::string > regexpSettings {
			RegExpSettings::stringsVector ( )
		};

		TCLAP::ValuesConstraint < std::string > regexpPrintingOptionsVals ( regexpSettings );

		TCLAP::ValueArg < std::string > regexpInput ( "", "regexp", "Access components of regexps", false, "", & regexpPrintingOptionsVals );

		// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------

		std::vector < std::string > exceptionSettings {
			ExceptionSettings::stringsVector ( )
		};

		TCLAP::ValuesConstraint < std::string > exceptionPrintingOptionsVals ( exceptionSettings );

		TCLAP::ValueArg < std::string > exceptionInput ( "", "exception", "Access components of exceptions", false, "", & exceptionPrintingOptionsVals );

		// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------

		std::vector < std::string > stringSettings {
			StringSettings::stringsVector ( )
		};

		TCLAP::ValuesConstraint < std::string > stringPrintingOptionsVals ( stringSettings );

		TCLAP::ValueArg < std::string > stringInput ( "", "string", "Access components of strings", false, "", & stringPrintingOptionsVals );

		// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------

		std::vector < std::string > treeSettings {
			TreeSettings::stringsVector ( )
		};

		TCLAP::ValuesConstraint < std::string > treePrintingOptionsVals ( treeSettings );

		TCLAP::ValueArg < std::string > treeInput ( "", "tree", "Access components of trees", false, "", & treePrintingOptionsVals );

		// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------

		std::vector < std::string > pairSetSettings {
			PairSetSettings::stringsVector ( )
		};

		TCLAP::ValuesConstraint < std::string > pairSetPrintingOptionsVals ( pairSetSettings );

		TCLAP::ValueArg < std::string > pairSetInput ( "", "pairSet", "Access components of set of pairs", false, "", & pairSetPrintingOptionsVals );

		// ----------------------------------------------------------------------------------------------------------------------------------------------------------------------

		std::vector < TCLAP::Arg * > xorlist;
		xorlist.push_back ( & automatonInput );
		xorlist.push_back ( & grammarInput );
		xorlist.push_back ( & regexpInput );
		xorlist.push_back ( & exceptionInput );
		xorlist.push_back ( & stringInput );
		xorlist.push_back ( & treeInput );
		xorlist.push_back ( & pairSetInput );
		cmd.xorAdd ( xorlist );

		std::vector < std::string > operationSettings {
			OperationSettings::stringsVector ( )
		};
		TCLAP::ValuesConstraint < std::string > allowedVals ( operationSettings );

		TCLAP::ValueArg < std::string > operation ( "o", "operation", "Operation type", false, "", & allowedVals );
		cmd.add ( operation );

		TCLAP::ValueArg < std::string > file ( "i", "input", "Read from file", false, "-", "file" );
		cmd.add ( file );

		TCLAP::ValueArg < std::string > argument ( "a", "argument", "Operation argument", false, "", "file" );
		cmd.add ( argument );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if ( verbose.isSet ( ) )
			common::GlobalData::verbose = true;

		if ( measure.isSet ( ) )
			common::GlobalData::measure = true;

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		std::deque < sax::Token > tokens = sax::FromXMLParserHelper::parseInput ( file );

		std::deque < sax::Token > argumentTokens;
		if ( argument.isSet ( ) )
			argumentTokens = sax::FromXMLParserHelper::parseInput( argument );

		if ( alib::XmlDataFactory::first < automaton::Automaton > ( tokens ) && automatonInput.isSet ( ) ) {
			automaton::Automaton automaton = alib::XmlDataFactory::fromTokens ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Accesss print", measurements::Type::MAIN );

			AutomatonAccess::access ( automaton, AutomatonSettings::fromString ( automatonInput.getValue ( ) ), OperationSettings::fromString ( operation.getValue ( ) ), argumentTokens );
		} else if ( alib::XmlDataFactory::first < grammar::Grammar > ( tokens ) && grammarInput.isSet ( ) ) {
			grammar::Grammar grammar = alib::XmlDataFactory::fromTokens ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Accesss print", measurements::Type::MAIN );

			GrammarAccess::access ( grammar, GrammarSettings::fromString ( grammarInput.getValue ( ) ), OperationSettings::fromString ( operation.getValue ( ) ), argumentTokens );
		} else if ( alib::XmlDataFactory::first < regexp::RegExp > ( tokens ) && regexpInput.isSet ( ) ) {
			regexp::RegExp regexp = alib::XmlDataFactory::fromTokens ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Accesss print", measurements::Type::MAIN );

			RegExpAccess::access ( regexp, RegExpSettings::fromString ( regexpInput.getValue ( ) ), OperationSettings::fromString ( operation.getValue ( ) ), argumentTokens );
		} else if ( alib::XmlDataFactory::first < exception::CommonException > ( tokens ) && exceptionInput.isSet ( ) ) {
			exception::CommonException exception = alib::XmlDataFactory::fromTokens ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Accesss print", measurements::Type::MAIN );

			ExceptionAccess::access ( exception, ExceptionSettings::fromString ( exceptionInput.getValue ( ) ), OperationSettings::fromString ( operation.getValue ( ) ) );
		} else if ( alib::XmlDataFactory::first < string::String > ( tokens ) && stringInput.isSet ( ) ) {
			string::String string = alib::XmlDataFactory::fromTokens ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Accesss print", measurements::Type::MAIN );

			StringAccess::access ( string, StringSettings::fromString ( stringInput.getValue ( ) ), OperationSettings::fromString ( operation.getValue ( ) ), argumentTokens );
		} else if ( alib::XmlDataFactory::first < tree::Tree > ( tokens ) && treeInput.isSet ( ) ) {
			tree::Tree tree = alib::XmlDataFactory::fromTokens ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Accesss print", measurements::Type::MAIN );

			TreeAccess::access ( tree, TreeSettings::fromString ( treeInput.getValue ( ) ), OperationSettings::fromString ( operation.getValue ( ) ), argumentTokens );
		} else if ( alib::XmlDataFactory::first < std::set < std::pair < alib::Object, alib::Object > > > ( tokens ) && pairSetInput.isSet ( ) ) {
			std::set < std::pair < alib::Object, alib::Object > > pairSet = alib::XmlDataFactory::fromTokens ( std::move ( tokens ) );

			measurements::end ( );
			measurements::start ( "Accesss print", measurements::Type::MAIN );

			PairSetAccess::access ( pairSet, PairSetSettings::fromString ( pairSetInput.getValue ( ) ), OperationSettings::fromString ( operation.getValue ( ) ) );
		} else {
			throw exception::CommonException ( "Input not recognized." );
		}

		measurements::end ( );
		measurements::end ( );

		if ( measure.getValue ( ) ) ext::cmeasure << measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::CommonException & exception ) {
		alib::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cout << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
