/*
 * GrammarCompare.cpp
 *
 *  Created on: Apr 1, 2013
 *      Author: honza
 */

#include "GrammarCompare.h"

#include <set>
#include <map>
#include <list>
#include <utility>
#include <vector>
#include <typeinfo>
#include <iostream>
#include <algorithm>

#include "grammar/Regular/LeftLG.h"
#include "grammar/Regular/LeftRG.h"
#include "grammar/Regular/RightLG.h"
#include "grammar/Regular/RightRG.h"
#include "grammar/ContextFree/LG.h"
#include "grammar/ContextFree/CFG.h"
#include "grammar/ContextFree/EpsilonFreeCFG.h"
#include "grammar/ContextFree/CNF.h"
#include "grammar/ContextFree/GNF.h"
#include "grammar/ContextSensitive/CSG.h"
#include "grammar/ContextSensitive/NonContractingGrammar.h"
#include "grammar/Unrestricted/ContextPreservingUnrestrictedGrammar.h"
#include "grammar/Unrestricted/UnrestrictedGrammar.h"
#include <registration/AlgoRegistration.hpp>

bool GrammarCompare::testCompare(const grammar::LeftLG < > & a, const grammar::LeftLG < > & b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarCompare::testCompare(const grammar::LeftRG < > & a, const grammar::LeftRG < > & b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarCompare::testCompare(const grammar::RightLG < > & a, const grammar::RightLG < > & b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarCompare::testCompare(const grammar::RightRG < > & a, const grammar::RightRG < > & b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarCompare::testCompare(const grammar::LG < > & a, const grammar::LG < > & b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarCompare::testCompare(const grammar::CFG < > & a, const grammar::CFG < > & b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarCompare::testCompare(const grammar::EpsilonFreeCFG < > & a, const grammar::EpsilonFreeCFG < > & b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarCompare::testCompare(const grammar::CNF < > & a, const grammar::CNF < > & b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarCompare::testCompare(const grammar::GNF < > & a, const grammar::GNF < > & b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarCompare::testCompare(const grammar::CSG < > & a, const grammar::CSG < > & b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarCompare::testCompare(const grammar::NonContractingGrammar < > & a, const grammar::NonContractingGrammar < > & b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarCompare::testCompare(const grammar::ContextPreservingUnrestrictedGrammar < > & a, const grammar::ContextPreservingUnrestrictedGrammar < > & b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

bool GrammarCompare::testCompare(const grammar::UnrestrictedGrammar < > & a, const grammar::UnrestrictedGrammar < > & b) {
	return  	a.getNonterminalAlphabet() == b.getNonterminalAlphabet() &&
			a.getRules()               == b.getRules()               &&
			a.getInitialSymbol()       == b.getInitialSymbol()       &&
			a.getTerminalAlphabet()    == b.getTerminalAlphabet()    ;
}

template <class T> void GrammarCompare::setCompare(const std::set<T> &a, const std::set<T> &b) {
	std::set<T> aMinusB;
	std::set_difference(a.begin(), a.end(), b.begin(), b.end(), std::inserter(aMinusB, aMinusB.begin()));

	std::set<T> bMinusA;
	std::set_difference(b.begin(), b.end(), a.begin(), a.end(), std::inserter(bMinusA, bMinusA.begin()));

	for(typename std::set<T>::const_iterator iter = aMinusB.begin(); iter != aMinusB.end(); iter++) {
		std::cout << "< " << *iter << std::endl;
	}
	std::cout << "---" << std::endl;
	for(typename std::set<T>::const_iterator iter = bMinusA.begin(); iter != bMinusA.end(); iter++) {
		std::cout << "> " << *iter << std::endl;
	}
}

template <class T> void GrammarCompare::listCompare(const std::list<T> &a, const std::list<T> &b) {
	std::list<T> aMinusB;
	std::set_difference(a.begin(), a.end(), b.begin(), b.end(), std::inserter(aMinusB, aMinusB.begin()));

	std::list<T> bMinusA;
	std::set_difference(b.begin(), b.end(), a.begin(), a.end(), std::inserter(bMinusA, bMinusA.begin()));

	for(typename std::list<T>::const_iterator iter = aMinusB.begin(); iter != aMinusB.end(); iter++) {
		std::cout << "< " << *iter << std::endl;
	}
	std::cout << "---" << std::endl;
	for(typename std::list<T>::const_iterator iter = bMinusA.begin(); iter != bMinusA.end(); iter++) {
		std::cout << "> " << *iter << std::endl;
	}
}

template <class T, class R> void GrammarCompare::mapCompare(const std::map<T, R> &a, const std::map<T, R> &b) {
	std::map<T, R> aMinusB;
	std::set_difference(a.begin(), a.end(), b.begin(), b.end(), std::inserter(aMinusB, aMinusB.begin()));

	std::map<T, R> bMinusA;
	std::set_difference(b.begin(), b.end(), a.begin(), a.end(), std::inserter(bMinusA, bMinusA.begin()));

	for(typename std::map<T, R>::const_iterator iter = aMinusB.begin(); iter != aMinusB.end(); iter++) {
		std::cout << "< " << iter->first << ", " << iter->second << std::endl;
	}
	std::cout << "---" << std::endl;
	for(typename std::map<T, R>::const_iterator iter = bMinusA.begin(); iter != bMinusA.end(); iter++) {
		std::cout << "> " << iter->first << ", " << iter->second << std::endl;
	}
}

void GrammarCompare::printCompare(const grammar::LeftLG < > & a, const grammar::LeftLG < > & b) {
	std::cout << "GrammarCompareer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarCompare::mapCompare(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarCompare::printCompare(const grammar::LeftRG < > & a, const grammar::LeftRG < > & b) {
	std::cout << "GrammarCompareer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarCompare::mapCompare(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarCompare::printCompare(const grammar::RightLG < > & a, const grammar::RightLG < > & b) {
	std::cout << "GrammarCompareer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarCompare::mapCompare(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarCompare::printCompare(const grammar::RightRG < > & a, const grammar::RightRG < > & b) {
	std::cout << "GrammarCompareer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarCompare::mapCompare(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarCompare::printCompare(const grammar::LG < > & a, const grammar::LG < > & b) {
	std::cout << "GrammarCompareer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarCompare::mapCompare(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarCompare::printCompare(const grammar::CFG < > & a, const grammar::CFG < > & b) {
	std::cout << "GrammarCompareer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarCompare::mapCompare(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarCompare::printCompare(const grammar::EpsilonFreeCFG < > & a, const grammar::EpsilonFreeCFG < > & b) {
	std::cout << "GrammarCompareer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarCompare::mapCompare(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarCompare::printCompare(const grammar::CNF < > & a, const grammar::CNF < > & b) {
	std::cout << "GrammarCompareer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarCompare::mapCompare(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarCompare::printCompare(const grammar::GNF < > & a, const grammar::GNF < > & b) {
	std::cout << "GrammarCompareer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarCompare::mapCompare(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarCompare::printCompare(const grammar::CSG < > & a, const grammar::CSG < > & b) {
	std::cout << "GrammarCompareer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarCompare::mapCompare(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarCompare::printCompare(const grammar::NonContractingGrammar < > & a, const grammar::NonContractingGrammar < > & b) {
	std::cout << "GrammarCompareer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarCompare::mapCompare(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarCompare::printCompare(const grammar::ContextPreservingUnrestrictedGrammar < > & a, const grammar::ContextPreservingUnrestrictedGrammar < > & b) {
	std::cout << "GrammarCompareer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarCompare::mapCompare(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

void GrammarCompare::printCompare(const grammar::UnrestrictedGrammar < > & a, const grammar::UnrestrictedGrammar < > & b) {
	std::cout << "GrammarCompareer" << std::endl;

	if(a.getNonterminalAlphabet() != b.getNonterminalAlphabet()) {
		std::cout << "Nonterminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getNonterminalAlphabet(), b.getNonterminalAlphabet());
	}

	if(a.getRules() != b.getRules()){
		std::cout << "Rules" << std::endl;

		GrammarCompare::mapCompare(a.getRules(), b.getRules());
	}

	if(a.getInitialSymbol() != b.getInitialSymbol()){
		std::cout << "Initial symbol" << std::endl;

		std::cout << "< " << a.getInitialSymbol() << std::endl;
		std::cout << "---" << std::endl;
		std::cout << "> " << b.getInitialSymbol() << std::endl;
	}

	if(a.getTerminalAlphabet() != b.getTerminalAlphabet()) {
		std::cout << "Terminal alphabet" << std::endl;

		GrammarCompare::setCompare(a.getTerminalAlphabet(), b.getTerminalAlphabet());
	}
}

auto GrammarCompareLeftLG = registration::OverloadRegister < GrammarCompare, int, grammar::LeftLG < >, grammar::LeftLG < > > ( GrammarCompare::compare );
auto GrammarCompareLeftRG = registration::OverloadRegister < GrammarCompare, int, grammar::LeftRG < >, grammar::LeftRG < > > ( GrammarCompare::compare );
auto GrammarCompareRightLG = registration::OverloadRegister < GrammarCompare, int, grammar::RightLG < >, grammar::RightLG < > > ( GrammarCompare::compare );
auto GrammarCompareRightRG = registration::OverloadRegister < GrammarCompare, int, grammar::RightRG < >, grammar::RightRG < > > ( GrammarCompare::compare );

auto GrammarCompareLG = registration::OverloadRegister < GrammarCompare, int, grammar::LG < >, grammar::LG < > > ( GrammarCompare::compare );
auto GrammarCompareCFG = registration::OverloadRegister < GrammarCompare, int, grammar::CFG < >, grammar::CFG < > > ( GrammarCompare::compare );
auto GrammarCompareEpsilonFreeCFG = registration::OverloadRegister < GrammarCompare, int, grammar::EpsilonFreeCFG < >, grammar::EpsilonFreeCFG < > > ( GrammarCompare::compare );
auto GrammarCompareCNF = registration::OverloadRegister < GrammarCompare, int, grammar::CNF < >, grammar::CNF < > > ( GrammarCompare::compare );
auto GrammarCompareGNF = registration::OverloadRegister < GrammarCompare, int, grammar::GNF < >, grammar::GNF < > > ( GrammarCompare::compare );
auto GrammarCompareCSG = registration::OverloadRegister < GrammarCompare, int, grammar::CSG < >, grammar::CSG < > > ( GrammarCompare::compare );

auto GrammarCompareNonContractingGrammar = registration::OverloadRegister < GrammarCompare, int, grammar::NonContractingGrammar < >, grammar::NonContractingGrammar < > > ( GrammarCompare::compare );
auto GrammarCompareContextPreservingUnrestrictedGrammar = registration::OverloadRegister < GrammarCompare, int, grammar::ContextPreservingUnrestrictedGrammar < >, grammar::ContextPreservingUnrestrictedGrammar < > > ( GrammarCompare::compare );

auto GrammarCompareUnrestrictedGrammar = registration::OverloadRegister < GrammarCompare, int, grammar::UnrestrictedGrammar < >, grammar::UnrestrictedGrammar < > > ( GrammarCompare::compare );

int GrammarCompare::compare(const grammar::Grammar& a, const grammar::Grammar& b) {
	return dispatch(a.getData(), b.getData());
}

