/*
 * StringCompare.cpp
 *
 *  Created on: Apr 4, 2017
 *      Author: Tomas Pecka
 */

#include "StringCompare.h"

#include "string/CyclicString.h"
#include "string/LinearString.h"
#include <registration/AlgoRegistration.hpp>

bool StringCompare::testCompare ( const string::LinearString < > & a, const string::LinearString < > & b ) {
	return a.getContent ( ) == b.getContent ( ) && a.getAlphabet ( ) == b.getAlphabet ( );
}

bool StringCompare::testCompare ( const string::CyclicString < > &, const string::CyclicString < > & ) {
	throw "NYI";
}

template < class T >
void StringCompare::setCompare ( const std::set < T > a, const std::set < T > b ) {
	std::set < T > aMinusB;
	std::set_difference ( a.begin ( ), a.end ( ), b.begin ( ), b.end ( ), std::inserter ( aMinusB, aMinusB.begin ( ) ) );

	std::set < T > bMinusA;
	std::set_difference ( b.begin ( ), b.end ( ), a.begin ( ), a.end ( ), std::inserter ( bMinusA, bMinusA.begin ( ) ) );

	for ( typename std::set < T >::const_iterator iter = aMinusB.begin ( ); iter != aMinusB.end ( ); iter++ )
		std::cout << "< " << * iter << std::endl;

	std::cout << "---" << std::endl;

	for ( typename std::set < T >::const_iterator iter = bMinusA.begin ( ); iter != bMinusA.end ( ); iter++ )
		std::cout << "> " << * iter << std::endl;
}

template < class T >
void StringCompare::vectorCompare ( const std::vector < T > a, const std::vector < T > b ) {
	std::vector < T > aMinusB;
	std::set_difference ( a.begin ( ), a.end ( ), b.begin ( ), b.end ( ), std::inserter ( aMinusB, aMinusB.begin ( ) ) );

	std::vector < T > bMinusA;
	std::set_difference ( b.begin ( ), b.end ( ), a.begin ( ), a.end ( ), std::inserter ( bMinusA, bMinusA.begin ( ) ) );

	for ( typename std::vector < T >::const_iterator iter = aMinusB.begin ( ); iter != aMinusB.end ( ); iter++ )
		std::cout << "< " << * iter << std::endl;

	std::cout << "---" << std::endl;

	for ( typename std::vector < T >::const_iterator iter = bMinusA.begin ( ); iter != bMinusA.end ( ); iter++ )
		std::cout << "> " << * iter << std::endl;
}

void StringCompare::printCompare ( const string::CyclicString < > &, const string::CyclicString < > & ) {
	throw "NYI";
}

void StringCompare::printCompare ( const string::LinearString < > & a, const string::LinearString < > & b ) {
	std::cout << "StringCompare" << std::endl;

	if ( a.getAlphabet ( ) != b.getAlphabet ( ) ) {
		std::cout << "Alphabet" << std::endl;

		StringCompare::setCompare ( a.getAlphabet ( ), b.getAlphabet ( ) );
	}

	if ( a.getContent ( ) != b.getContent ( ) ) {
		std::cout << "Content" << std::endl;

		StringCompare::vectorCompare ( a.getContent ( ), b.getContent ( ) );
	}
}

auto StringCompareLinear = registration::OverloadRegister < StringCompare, int, string::LinearString < >, string::LinearString < > > ( StringCompare::compare );
auto StringCompareCyclic = registration::OverloadRegister < StringCompare, int, string::CyclicString < >, string::CyclicString < > > ( StringCompare::compare );

int StringCompare::compare ( const string::String & a, const string::String & b ) {
	return dispatch ( a.getData ( ), b.getData ( ) );
}
