/*
 * agenerate.cpp
 *
 *  Created on: 16. 2. 2015
 *	  Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <vector>
#include <sax/FromXMLParserHelper.h>

#include <factory/XmlDataFactory.hpp>
#include <exception/CommonException.h>
#include <string/LinearString.h>
#include <grammar/Grammar.h>
#include <grammar/generate/GenerateUpToLength.h>
#include <grammar/generate/CockeYoungerKasami.h>
#include <container/ObjectsSet.h>
#include <primitive/Bool.h>

int main ( int argc, char * argv[] ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "String generate from grammar", ' ', "0.01" );

		std::vector < std::string > allowed;
		allowed.push_back ( "upTo" );
		allowed.push_back ( "CYK" );
		TCLAP::ValuesConstraint < std::string > allowedVals ( allowed );

		TCLAP::ValueArg < std::string > type ( "t", "type", "Run type", false, "accept", & allowedVals );
		cmd.add ( type );

		TCLAP::ValueArg < unsigned > upto ( "", "upto", "Generate all strings up to length", false, 5, "unsigned" );
		cmd.add ( upto );

		TCLAP::ValueArg < std::string > stringInput ( "s", "string", "String from file", false, "-", "file" );
		cmd.add ( stringInput );

		TCLAP::ValueArg < std::string > grammarInput ( "g", "grammar", "Grammar to use", false, "-", "file" );
		cmd.add ( grammarInput );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if ( verbose.isSet ( ) )
			common::GlobalData::verbose = true;

		if ( measure.isSet ( ) )
			common::GlobalData::measure = true;

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		if ( ( type.getValue ( ) == "upTo" ) && upto.isSet ( ) ) {
			grammar::Grammar grammar = alib::XmlDataFactory::fromTokens ( sax::FromXMLParserHelper::parseInput(grammarInput) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < string::LinearString < > > res = grammar::generate::GenerateUpToLength::generate ( grammar, upto.getValue ( ) );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( type.getValue ( ) == "CYK" ) {
			grammar::Grammar grammar = alib::XmlDataFactory::fromTokens ( sax::FromXMLParserHelper::parseInput(grammarInput) );
			string::LinearString < > string = alib::XmlDataFactory::fromTokens ( sax::FromXMLParserHelper::parseInput(stringInput) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			bool res = grammar::generate::CockeYoungerKasami::generate ( grammar, string );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else {
			throw exception::CommonException ( "Invalid run type" );
		}

		measurements::end ( );
		measurements::end ( );

		if ( measure.getValue ( ) ) ext::cmeasure << measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::CommonException & exception ) {
		alib::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cout << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
