/*
 * aconvert.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Martin Zak
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <factory/XmlDataFactory.hpp>
#include <exception/CommonException.h>

#include <introspection/DataTypes.hpp>
#include <introspection/Algorithms.hpp>
#include <introspection/Casts.hpp>

int main ( int argc, char * argv[] ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Introspection binary", ' ', "0.01" );

		TCLAP::SwitchArg dataTypeGroups ( "", "datatype_groups", "Data type Groups", false );
		cmd.add ( dataTypeGroups );

		TCLAP::ValueArg < std::string > dataTypesInGroup ( "d", "datatypes_in_group", "Data type Groups", false, "", "string" );
		cmd.add ( dataTypesInGroup );

		TCLAP::SwitchArg algorithms ( "", "algorithms", "Algorithms", false );
		cmd.add ( algorithms );

		TCLAP::ValueArg < std::string > algorithmInterface ( "", "algorithm_interface", "Algorithm Interface", false, "", "string" );
		cmd.add ( algorithmInterface );

		TCLAP::ValueArg < std::string > algorithmCallbacks ( "a", "algorithm_callbacks", "Algorithm Callbacks", false, "", "string" );
		cmd.add ( algorithmCallbacks );

		TCLAP::ValueArg < std::string > castFrom ( "", "cast_from", "Casting handlers from type", false, "", "string" );
		cmd.add ( castFrom );

		TCLAP::ValueArg < std::string > castTo ( "", "cast_to", "Casting handlers to type", false, "", "string" );
		cmd.add ( castTo );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if ( verbose.isSet ( ) )
			common::GlobalData::verbose = true;

		if ( measure.isSet ( ) )
			common::GlobalData::measure = true;

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Output write", measurements::Type::AUXILIARY );

		if ( dataTypeGroups.getValue ( ) ) {
			for ( const std::string & group : introspection::DataTypes::getGroups ( ) )
				std::cout << group << std::endl;

		} else if ( dataTypesInGroup.isSet ( ) ) {
			auto groupIter = introspection::DataTypes::getTypesInGroup ( ).find ( dataTypesInGroup.getValue ( ) );
			if ( groupIter == introspection::DataTypes::getTypesInGroup ( ).end ( ) )
				std::cout << "No such group as " << dataTypesInGroup.getValue ( ) << std::endl;
			else
				for ( const std::string & group : groupIter->second )
					std::cout << group << std::endl;

		} else if ( algorithms.getValue ( ) ) {
			for ( const std::string & algorithm : introspection::Algorithms::getAlgorithms ( ) )
				std::cout << algorithm << std::endl;

		} else if ( algorithmInterface.isSet ( ) ) {
			auto algorithmInterfaceIter = introspection::Algorithms::getAlgorithmInterfaces ( ).find ( algorithmInterface.getValue ( ) );
			if ( algorithmInterfaceIter == introspection::Algorithms::getAlgorithmInterfaces ( ).end ( ) )
				std::cout << "No such algorithm as " << algorithmInterface.getValue ( ) << std::endl;
			else
				std::cout << * algorithmInterfaceIter << std::endl;

		} else if ( algorithmCallbacks.isSet ( ) ) {
			auto algorithmCallbacksIter = introspection::Algorithms::getAlgorithmCallbacks ( ).find ( algorithmCallbacks.getValue ( ) );
			if ( algorithmCallbacksIter == introspection::Algorithms::getAlgorithmCallbacks ( ).end ( ) )
				std::cout << "No such algorithm as " << algorithmCallbacks.getValue ( ) << std::endl;
			else
				for ( const auto & callback : algorithmCallbacksIter->second )
					std::cout << callback << std::endl;

		} else if ( castFrom.isSet ( ) ) {
			auto fromIter = introspection::Casts::getCastsFrom ( ).find ( castFrom.getValue ( ) );
			if ( fromIter == introspection::Casts::getCastsFrom ( ).end ( ) || fromIter->second.empty ( ) )
				std::cout << "No cast from " << castFrom.getValue ( ) << " registered" << std::endl;
			else
				for ( const std::string & castTarget : fromIter->second )
					std::cout << castTarget << std::endl;

		} else if ( castTo.isSet ( ) ) {
			auto toIter = introspection::Casts::getCastsTo ( ).find ( castTo.getValue ( ) );
			if ( toIter == introspection::Casts::getCastsTo ( ).end ( ) || toIter->second.empty ( ) )
				std::cout << "No cast to " << castTo.getValue ( ) << " registered" << std::endl;
			else
				for ( const std::string & castSource : toIter->second )
					std::cout << castSource << std::endl;

		}

		measurements::end ( );
		measurements::end ( );

		if ( measure.getValue ( ) ) ext::cmeasure << measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::CommonException & exception ) {
		alib::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cerr << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
