/*
 * NonlinearFullAndLinearIndexPatterns.h
 *
 *  Created on: 2. 1. 2017
 *      Author: Jan Travnicek
 */

#ifndef NONLINEAR_FULL_AND_LINEAR_INDEX_PATTERNS_H_
#define NONLINEAR_FULL_AND_LINEAR_INDEX_PATTERNS_H_

#include <indexes/arbology/NonlinearFullAndLinearIndex.h>
#include <tree/RankedTreeWrapper.h>
#include <tree/ranked/PrefixRankedNonlinearPattern.h>
#include <tree/ranked/PrefixRankedBarNonlinearPattern.h>
#include <core/multipleDispatch.hpp>
#include <global/GlobalData.h>

#include <stringology/query/PositionHeapFactors.h>

namespace arbology {

namespace query {

/**
 * Query full and linear index for given tree.
 *
 */

class NonlinearFullAndLinearIndexPatterns : public alib::SingleDispatchFirstStaticParam < NonlinearFullAndLinearIndexPatterns, std::set < unsigned >, const indexes::arbology::NonlinearFullAndLinearIndex < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > &, const tree::RankedTreeBase & > {
	template < class SymbolType, class RankType >
	static std::vector < std::pair < unsigned, unsigned > > FindOccurrences ( const indexes::stringology::PositionHeap < common::ranked_symbol < SymbolType, RankType > > & stringIndex, const std::vector < common::ranked_symbol < SymbolType, RankType > > & string ) {
		std::vector < std::pair < unsigned, unsigned > > res;
		for ( unsigned occurrence : stringology::query::PositionHeapFactors::query ( stringIndex, string::LinearString < common::ranked_symbol < SymbolType, RankType > > ( string ) ) ) {
			res.push_back ( std::make_pair ( occurrence, occurrence + string.size ( ) ) );
		}
		return res;
	}

	static std::vector < std::pair < unsigned, unsigned > > MergeOccurrences ( const std::vector < std::pair < unsigned, unsigned > > & prevOcc, const std::vector < std::pair < unsigned, unsigned > > & subOcc, std::vector < unsigned > & rev ) {
		std::vector < std::pair < unsigned, unsigned > > res;

		for ( const std::pair < unsigned, unsigned > & occurrence : prevOcc ) {
			rev [ occurrence.second ] = occurrence.first;
		}

		for ( const std::pair < unsigned, unsigned > & subOccurrence : subOcc ) {
			if ( rev [ subOccurrence.first ] != ( unsigned ) -1 )
				res.push_back ( std::make_pair ( rev [ subOccurrence.first ], subOccurrence.second ) );
		}

		for ( const std::pair < unsigned, unsigned > & occurrence : prevOcc ) {
			rev [ occurrence.second ] = ( unsigned ) -1;
		}

		return res;
	}
public:
	/**
	 * Query a suffix trie
	 * @param suffix trie to query
	 * @param tree tree to query by
	 * @return occurences of factors
	 */
	static std::set < unsigned > query ( const indexes::arbology::NonlinearFullAndLinearIndex < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > & fullAndLinearIndex, const tree::RankedTreeWrapper & pattern );

	template < class SymbolType, class RankType >
	static std::set < unsigned > query ( const indexes::arbology::NonlinearFullAndLinearIndex < common::ranked_symbol < SymbolType, RankType > > & compressedBitParallelTreeIndex, const tree::PrefixRankedNonlinearPattern < SymbolType, RankType > & pattern );

	template < class SymbolType, class RankType >
	static std::set < unsigned > query ( const indexes::arbology::NonlinearFullAndLinearIndex < common::ranked_symbol < SymbolType, RankType > > & compressedBitParallelTreeIndex, const tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType > & pattern );
};

template < class SymbolType, class RankType >
std::set < unsigned > NonlinearFullAndLinearIndexPatterns::query ( const indexes::arbology::NonlinearFullAndLinearIndex < common::ranked_symbol < SymbolType, RankType > > & fullAndLinearIndex, const tree::PrefixRankedNonlinearPattern < SymbolType, RankType > & pattern ) {
	std::map < std::pair < unsigned, common::ranked_symbol < SymbolType, RankType > >, unsigned > nonlinearVariablesMap;
	std::vector < unsigned > rev ( fullAndLinearIndex.getString ( ).size ( ), ( unsigned ) -1 );

	std::vector < std::vector < common::ranked_symbol < SymbolType, RankType > > > treePatternParts;
	treePatternParts.push_back ( std::vector < common::ranked_symbol < SymbolType, RankType > > ( ) );
	for ( const common::ranked_symbol < SymbolType, RankType > & symbol : pattern.getContent ( ) ) {
		if ( pattern.getSubtreeWildcard ( ) == symbol || pattern.getNonlinearVariables ( ).count ( symbol ) ) {
			treePatternParts.push_back ( std::vector < common::ranked_symbol < SymbolType, RankType > > ( ) );
			treePatternParts.back ( ).push_back ( symbol );
			treePatternParts.push_back ( std::vector < common::ranked_symbol < SymbolType, RankType > > ( ) );
		} else {
			treePatternParts.back ( ).push_back ( symbol );
		}
	}

	std::vector < std::pair < unsigned, unsigned > > prevOcc = FindOccurrences ( fullAndLinearIndex.getStringIndex ( ) , treePatternParts [ 0 ] );

	for ( unsigned i = 1; i < treePatternParts.size ( ); ++ i ) {
		for ( std::pair < unsigned, unsigned > & occurrence : prevOcc ) {
			if ( pattern.getNonlinearVariables ( ).count ( treePatternParts [ i ].back ( ) ) ) {
				auto variableSettingIter = nonlinearVariablesMap.find ( std::make_pair ( occurrence.first, treePatternParts [ i ].back ( ) ) );
				if ( variableSettingIter == nonlinearVariablesMap.end ( ) )
					nonlinearVariablesMap.insert ( std::make_pair ( std::make_pair ( occurrence.first, treePatternParts [ i ].back ( ) ), fullAndLinearIndex.getRepeats ( ) [ occurrence.second ] ) );
				else if ( variableSettingIter->second != fullAndLinearIndex.getRepeats ( ) [ occurrence.second ] )
					occurrence.first = ( unsigned ) -1;
			}
			occurrence.second = fullAndLinearIndex.getJumps ( ) [ occurrence.second ];
		}

		++ i;

		if ( ! treePatternParts [ i ].empty ( ) )
			prevOcc = MergeOccurrences ( prevOcc, FindOccurrences ( fullAndLinearIndex.getStringIndex ( ), treePatternParts [ i ] ), rev );
	}

	std::set < unsigned > res;
	for ( const std::pair < unsigned, unsigned > & occurrence : prevOcc ) {
		if ( occurrence.first != ( unsigned ) -1 )
			res.insert ( occurrence.first );
	}

	return res;
}

template < class SymbolType, class RankType >
std::set < unsigned > NonlinearFullAndLinearIndexPatterns::query ( const indexes::arbology::NonlinearFullAndLinearIndex < common::ranked_symbol < SymbolType, RankType > > & fullAndLinearIndex, const tree::PrefixRankedBarNonlinearPattern < SymbolType, RankType > & pattern ) {
	std::map < std::pair < unsigned, common::ranked_symbol < SymbolType, RankType > >, unsigned > nonlinearVariablesMap;
	std::vector < unsigned > rev ( fullAndLinearIndex.getString ( ).size ( ), ( unsigned ) -1 );

	std::vector < std::vector < common::ranked_symbol < SymbolType, RankType > > > treePatternParts;
	treePatternParts.push_back ( std::vector < common::ranked_symbol < SymbolType, RankType > > ( ) );
	for ( const common::ranked_symbol < SymbolType, RankType > & symbol : pattern.getContent ( ) ) {
		if ( symbol == pattern.getVariablesBar ( ) )
			continue;

		if ( pattern.getSubtreeWildcard ( ) == symbol || pattern.getNonlinearVariables ( ).count ( symbol ) ) {
			treePatternParts.push_back ( std::vector < common::ranked_symbol < SymbolType, RankType > > ( ) );
			treePatternParts.back ( ).push_back ( symbol );
			treePatternParts.push_back ( std::vector < common::ranked_symbol < SymbolType, RankType > > ( ) );
		} else {
			treePatternParts.back ( ).push_back ( symbol );
		}
	}

	std::vector < std::pair < unsigned, unsigned > > prevOcc = FindOccurrences ( fullAndLinearIndex.getStringIndex ( ) , treePatternParts [ 0 ] );

	for ( unsigned i = 1; i < treePatternParts.size ( ); ++ i ) {
		for ( std::pair < unsigned, unsigned > & occurrence : prevOcc ) {
			if ( pattern.getNonlinearVariables ( ).count ( treePatternParts [ i ].back ( ) ) ) {
				auto variableSettingIter = nonlinearVariablesMap.find ( std::make_pair ( occurrence.first, treePatternParts [ i ].back ( ) ) );
				if ( variableSettingIter == nonlinearVariablesMap.end ( ) )
					nonlinearVariablesMap.insert ( std::make_pair ( std::make_pair ( occurrence.first, treePatternParts [ i ].back ( ) ), fullAndLinearIndex.getRepeats ( ) [ occurrence.second ] ) );
				else if ( variableSettingIter->second != fullAndLinearIndex.getRepeats ( ) [ occurrence.second ] )
					occurrence.first = ( unsigned ) -1;
			}
			occurrence.second = fullAndLinearIndex.getJumps ( ) [ occurrence.second ];
		}

		++ i;

		if ( ! treePatternParts [ i ].empty ( ) )
			prevOcc = MergeOccurrences ( prevOcc, FindOccurrences ( fullAndLinearIndex.getStringIndex ( ), treePatternParts [ i ] ), rev );
	}

	std::set < unsigned > res;
	for ( const std::pair < unsigned, unsigned > & occurrence : prevOcc ) {
		if ( occurrence.first != ( unsigned ) -1 )
			res.insert ( occurrence.first );
	}

	return res;
}

} /* namespace query */

} /* namespace arbology */

#endif /* NONLINEAR_FULL_AND_LINEAR_INDEX_PATTERNS_H_ */
