/*
 * BeginToEndIndex.h
 *
 *  Created on: 5. 11. 2014
 *      Author: Jan Travnicek
 */

#ifndef _ARBOLOGY_BEGIN_TO_END_INDEX_H_
#define _ARBOLOGY_BEGIN_TO_END_INDEX_H_

#include <set>
#include <core/multipleDispatch.hpp>
#include <tree/TreeFeatures.h>

#include <tree/properties/SubtreeJumpTable.h>

#include <tree/Tree.h>
#include <tree/ranked/PrefixRankedBarTree.h>
#include <tree/ranked/PrefixRankedTree.h>

namespace arbology {

namespace transform {

/**
 */
class BeginToEndIndex : public alib::SingleDispatch < BeginToEndIndex, std::set < unsigned >, const tree::TreeBase &, const std::set < unsigned > & > {
public:
	/**
	 * Search for pattern in linear string.
	 * @return set set of occurences
	 */
	static std::set < unsigned > transform ( const tree::Tree & subject, const std::set < unsigned > & indexes );

	template < class SymbolType, class RankType >
	static std::set < unsigned > transform ( const tree::PrefixRankedBarTree < SymbolType, RankType > & subject, const std::set < unsigned > & indexes );
	template < class SymbolType, class RankType >
	static std::set < unsigned > transform ( const tree::PrefixRankedTree < SymbolType, RankType > & subject, const std::set < unsigned > & indexes );

};

template < class SymbolType, class RankType >
std::set < unsigned > BeginToEndIndex::transform ( const tree::PrefixRankedBarTree < SymbolType, RankType > & subject, const std::set < unsigned > & indexes ) {
	std::vector < int > subjectSubtreeJumpTable = tree::properties::SubtreeJumpTable::compute ( subject );
	std::set < unsigned > res;

	for ( unsigned index : indexes )
		res.insert ( subjectSubtreeJumpTable[index] );

	return res;
}

template < class SymbolType, class RankType >
std::set < unsigned > BeginToEndIndex::transform ( const tree::PrefixRankedTree < SymbolType, RankType > & subject, const std::set < unsigned > & indexes ) {
	std::vector < int > subjectSubtreeJumpTable = tree::properties::SubtreeJumpTable::compute ( subject );
	std::set < unsigned > res;

	for ( unsigned index : indexes )
		res.insert ( subjectSubtreeJumpTable[index] );

	return res;
}

} /* namespace transform */

} /* namespace arbology */

#endif /* _ARBOLOGY_BEGIN_TO_END_INDEX_H_ */
