/*
 * ToRegExp.h
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#ifndef _AUTOMATON_TO_REGEXP_H__
#define _AUTOMATON_TO_REGEXP_H__

#include <core/multipleDispatch.hpp>

#include <automaton/FSM/DFA.h>
#include <automaton/FSM/NFA.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <automaton/FSM/ExtendedNFA.h>
#include <regexp/unbounded/UnboundedRegExp.h>

#include <automaton/Automaton.h>
#include <regexp/RegExp.h>

#include "ToRegExpStateElimination.h"

namespace automaton {

namespace convert {

class ToRegExp : public alib::SingleDispatch<ToRegExp, regexp::RegExp, const automaton::AutomatonBase &> {
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static regexp::RegExp convert(const automaton::Automaton& automaton);

	template < class SymbolType, class EpsilonType, class StateType >
	static regexp::RegExp convert(const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & automaton);
	template < class SymbolType, class StateType >
	static regexp::RegExp convert(const automaton::MultiInitialStateNFA < SymbolType, StateType > & automaton);
	template < class SymbolType, class StateType >
	static regexp::RegExp convert(const automaton::NFA < SymbolType, StateType > & automaton);
	template < class SymbolType, class StateType >
	static regexp::RegExp convert(const automaton::DFA < SymbolType, StateType > & automaton);
	template < class SymbolType, class StateType >
	static regexp::RegExp convert(const automaton::ExtendedNFA < SymbolType, StateType > & automaton);
	template < class SymbolType, class StateType >
	static regexp::RegExp convert(const automaton::CompactNFA < SymbolType, StateType > & automaton);
};

template < class SymbolType, class EpsilonType, class StateType >
regexp::RegExp ToRegExp::convert(const automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > & automaton) {
	return regexp::RegExp(ToRegExpStateElimination::convert(automaton));
}

template < class SymbolType, class StateType >
regexp::RegExp ToRegExp::convert(const automaton::MultiInitialStateNFA < SymbolType, StateType > & automaton) {
	return regexp::RegExp(ToRegExpStateElimination::convert(automaton));
}

template < class SymbolType, class StateType >
regexp::RegExp ToRegExp::convert(const automaton::NFA < SymbolType, StateType > & automaton) {
	return regexp::RegExp(ToRegExpStateElimination::convert(automaton));
}

template < class SymbolType, class StateType >
regexp::RegExp ToRegExp::convert(const automaton::DFA < SymbolType, StateType > & automaton) {
	return regexp::RegExp(ToRegExpStateElimination::convert(automaton));
}

template < class SymbolType, class StateType >
regexp::RegExp ToRegExp::convert(const automaton::ExtendedNFA < SymbolType, StateType > & automaton) {
	return regexp::RegExp(ToRegExpStateElimination::convert(automaton));
}

template < class SymbolType, class StateType >
regexp::RegExp ToRegExp::convert(const automaton::CompactNFA < SymbolType, StateType > & automaton) {
	return regexp::RegExp(ToRegExpStateElimination::convert(automaton));
}

} /* namespace convert */

} /* namespace automaton */

#endif /* _AUTOMATON_TO_REGEXP_H__ */
