/*
 * Translate.h
 *
 *  Created on: 12. 5. 2016
 *      Author: Jakub Doupal
 */

#ifndef _AUTOMATON_TRANSLATE_H__
#define _AUTOMATON_TRANSLATE_H__

#include <core/multipleDispatch.hpp>
#include <automaton/Automaton.h>
#include <automaton/AutomatonFeatures.h>
#include <object/Object.h>
#include <string/LinearString.h>

#include "Run.h"
#include <automaton/PDA/NPDTA.h>

#include <deque>
#include <set>

namespace automaton {

namespace run {

class Translate : public alib::DoubleDispatch < Translate, std::set < string::LinearString < > >, const automaton::AutomatonBase &, const alib::ObjectBase & > {
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static std::set < string::LinearString < > > translate ( const automaton::Automaton & automaton, const alib::Object & object );
	static std::set < string::LinearString < > > translate ( const automaton::Automaton & automaton, const string::LinearString < > & string );

	template < class InputSymbolType, class OutputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
	static std::set < string::LinearString < OutputSymbolType > > translate ( const automaton::NPDTA < InputSymbolType, OutputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string );

};

template < class InputSymbolType, class OutputSymbolType, class EpsilonType, class PushdownStoreSymbolType, class StateType >
std::set < string::LinearString < OutputSymbolType > > Translate::translate ( const automaton::NPDTA < InputSymbolType, OutputSymbolType, EpsilonType, PushdownStoreSymbolType, StateType > & automaton, const string::LinearString < InputSymbolType > & string ) {
	std::tuple < bool, std::set < StateType >, std::set < std::vector < OutputSymbolType > > > res = Run::calculateStates ( automaton, string );
	std::set < string::LinearString < OutputSymbolType > > strings;

	for ( const auto & rawString : std::get < 2 > ( res ) )
		strings . insert ( string::LinearString < OutputSymbolType > ( rawString ) );

	return strings;
}

} /* namespace run */

} /* namespace automaton */

#endif /* _AUTOMATON_TRANSLATE_H__ */
