/*
 * AutomatonIterationEpsilonTransition.h
 *
 *  Created on: 29. 11. 2014
 *	  Author: Tomas Pecka
 */

#ifndef AUTOMATON_ITERATION_EPSILON_TRANSITION_H_
#define AUTOMATON_ITERATION_EPSILON_TRANSITION_H_

#include <core/multipleDispatch.hpp>
#include <automaton/Automaton.h>
#include <automaton/FSM/EpsilonNFA.h>

namespace automaton {

namespace transform {

/**
 * Iterates language given by automaton
 *  - For finite automaton A1, we create automaton L accepting L(A1)*
 */
class AutomatonIterationEpsilonTransition : public alib::SingleDispatch<AutomatonIterationEpsilonTransition, automaton::Automaton, const automaton::AutomatonBase &> {
public:
	static automaton::Automaton iteration(const automaton::Automaton& automaton);

	template < class T, class SymbolType = typename automaton::SymbolTypeOfAutomaton < T >, class EpsilonType = DefaultEpsilonType, class StateType = typename automaton::StateTypeOfAutomaton < T > >
	static automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > iteration(const T& automaton);
};

template < class T, class SymbolType, class EpsilonType, class StateType >
automaton::EpsilonNFA < SymbolType, EpsilonType, StateType > AutomatonIterationEpsilonTransition::iteration(const T& automaton) {
	automaton::EpsilonNFA < > res(automaton);

	for(const auto&q : automaton.getFinalStates())
		res.addTransition(q, automaton.getInitialState());

	StateType newInitialState = common::createUnique(automaton.getInitialState(), automaton.getStates());
	res.addState(newInitialState);
	res.setInitialState(newInitialState);
	res.addFinalState(newInitialState);

	res.addTransition(newInitialState, automaton.getInitialState());
	return res;
}

} /* namespace transform */

} /* namespace automaton */

#endif /* AUTOMATON_ITERATION_EPSILON_H_ */
