/*
 * ToGrammarRightRGGlushkov.cpp
 *
 *  Created on: 11. 1. 2014
 *	  Author: Tomas Pecka
 */

#include "ToGrammarRightRGGlushkov.h"

#include <algorithm>

#include <alphabet/InitialSymbol.h>

#include "../glushkov/GlushkovFollow.h"
#include "../glushkov/GlushkovIndexate.h"
#include "../glushkov/GlushkovFirst.h"
#include "../glushkov/GlushkovLast.h"

#include "../properties/RegExpEpsilon.h"
#include <exception/CommonException.h>
#include <registration/AlgoRegistration.hpp>

namespace regexp {

namespace convert {

grammar::RightRG < > ToGrammarRightRGGlushkov::convert ( const regexp::RegExp & regexp ) {
	return dispatch ( regexp.getData ( ) );
}

grammar::RightRG < > ToGrammarRightRGGlushkov::convert ( const regexp::UnboundedRegExp < > & regexp ) {
	DefaultSymbolType S = alphabet::InitialSymbol::instance < DefaultSymbolType > ( );
	grammar::RightRG < > grammar ( S );

	 // step 1
	grammar.setTerminalAlphabet ( regexp.getAlphabet ( ) );

	regexp::UnboundedRegExp < std::pair < DefaultSymbolType, int > > indexedRegExp = regexp::GlushkovIndexate::index ( regexp );

	 // steps 2, 3, 4
	const std::set < regexp::UnboundedRegExpSymbol < std::pair < DefaultSymbolType, int > > > first = regexp::GlushkovFirst::first ( indexedRegExp );
	const std::set < regexp::UnboundedRegExpSymbol < std::pair < DefaultSymbolType, int > > > last = regexp::GlushkovLast::last ( indexedRegExp );

	// \e in q0 check is in step 7

	 // step 5
	for ( const std::pair < DefaultSymbolType, int > & symbol : indexedRegExp.getAlphabet ( ) )
		grammar.addNonterminalSymbol ( DefaultSymbolType ( container::ObjectsPair < DefaultSymbolType, int > ( symbol ) ) );

	 // step 6
	for ( const regexp::UnboundedRegExpSymbol < std::pair < DefaultSymbolType, int > > & symbol : first )
		grammar.addRule ( S, std::make_pair ( symbol.getSymbol ( ).first, DefaultSymbolType ( container::ObjectsPair < DefaultSymbolType, int > ( symbol.getSymbol ( ) ) ) ) );

	for ( const std::pair < DefaultSymbolType, int > & x : indexedRegExp.getAlphabet ( ) )
		for ( const regexp::UnboundedRegExpSymbol < std::pair < DefaultSymbolType, int > > & f : regexp::GlushkovFollow::follow ( indexedRegExp, UnboundedRegExpSymbol < std::pair < DefaultSymbolType, int > > ( x ) ) ) {
			const std::pair < DefaultSymbolType, int > & a = x;
			const std::pair < DefaultSymbolType, int > & b = f.getSymbol ( );

			grammar.addRule ( DefaultSymbolType ( container::ObjectsPair < DefaultSymbolType, int > ( a ) ), std::make_pair ( b.first, DefaultSymbolType ( container::ObjectsPair < DefaultSymbolType, int > ( b ) ) ) );
		}

	// step 7

	/*
	 * for all rules where ns.m_nonTerminal is on rightSide:
	 *  add Rule: leftSide -> symbol.getSymbol( )
	 *  unless it already exists
	 */
	for ( const auto & rule : grammar.getRawRules ( ) ) {
		for ( const auto & rhs : rule.second )
			for ( const regexp::UnboundedRegExpSymbol < std::pair < DefaultSymbolType, int > > & symbol : last ) {

				const std::pair < DefaultSymbolType, int > & a = symbol.getSymbol ( );

				if ( std::find ( rhs.begin ( ), rhs.end ( ), DefaultSymbolType ( container::ObjectsPair < DefaultSymbolType, int > ( a ) ) ) != rhs.end ( ) )
					grammar.addRule ( rule.first, rhs.at ( 0 ) );
			}

	}

	if ( regexp::properties::RegExpEpsilon::languageContainsEpsilon ( regexp ) )
		grammar.setGeneratesEpsilon ( true );

	return grammar;
}

auto ToGrammarRightRGGlushkovUnboundedRegExp = registration::OverloadRegister < ToGrammarRightRGGlushkov, grammar::RightRG < >, regexp::UnboundedRegExp < > > ( ToGrammarRightRGGlushkov::convert );

grammar::RightRG < > ToGrammarRightRGGlushkov::convert ( const regexp::FormalRegExp < > & /* regexp */ ) {
	throw exception::CommonException ( "Glushkov: Converting FormalRegExp NYI" ); // TODO
}

auto ToGrammarRightRGGlushkovFormalRegExp = registration::OverloadRegister < ToGrammarRightRGGlushkov, grammar::RightRG < >, regexp::FormalRegExp < > > ( ToGrammarRightRGGlushkov::convert );

} /* namespace convert */

} /* namespace regexp */
