/*
 * GlushkovFollow.h
 *
 *  Created on: 14. 4. 2016
 *	  Author: Tomas Pecka
 */

#ifndef RTE_GLUSHKOV_FOLLOW_H_
#define RTE_GLUSHKOV_FOLLOW_H_

#include <vector>
#include <map>
#include <set>

#include <rte/formal/FormalRTE.h>
#include <rte/formal/FormalRTEElements.h>

#include <alphabet/RankedSymbol.h>

#include "GlushkovFirst.h"
#include "GlushkovPos.h"
#include <iterator>
#include <vector>

namespace rte {

class GlushkovFollow {
	template < class SymbolType, class RankType >
	static std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > replaceConstants ( const std::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK, const std::vector < common::ranked_symbol < SymbolType, RankType > > & follow, const std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & subMap2 );
	template < class SymbolType, class RankType >
	static void preprocessSubMap ( const std::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK, std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & subMap );
	template < class T >
	static std::vector < std::vector < T > > cartesian ( const std::vector < std::vector < T > > & input );
	template < class T >
	static void cartesian_rec ( const std::vector < std::vector < T > > & input, std::vector < std::vector < T > > & ret, std::vector < T > & current, size_t depth );
public:
	/**
	 * @param re rte to probe
	 * @param symbol FormalRTESymbol for which we need the follow(), i.e., Follow(RTE, symbol)
	 * @return all symbols that can follow specific symbol in word
	 */
	template < class SymbolType, class RankType >
	static std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > follow ( const rte::FormalRTE < SymbolType, RankType > & re, const common::ranked_symbol < SymbolType, RankType > & symbol );

	template < class SymbolType, class RankType >
	class Formal {
	public:
		static std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > visit ( const rte::FormalRTEElement < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbol, const std::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK, std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & subM );
		static std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > visit ( const rte::FormalRTEAlternation < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbol, const std::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK, std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & subM );
		static std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > visit ( const rte::FormalRTESubstitution < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbol, const std::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK, std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & subM );
		static std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > visit ( const rte::FormalRTEIteration < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbol, const std::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK, std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & subM );
		static std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > visit ( const rte::FormalRTESymbolAlphabet < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbol, const std::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK, std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & subM );
		static std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > visit ( const rte::FormalRTESymbolSubst < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbol, const std::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK, std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & subM );
		static std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > visit ( const rte::FormalRTEEmpty < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbol, const std::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK, std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & subM );
	};
};

template < class SymbolType, class RankType >
std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollow::follow ( const rte::FormalRTE < SymbolType, RankType > & rte, const common::ranked_symbol < SymbolType, RankType > & symbol ) {
	std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > subMap;

	 /* Init substitution map, ie \forall a \in K: sub[a] = \emptyset */
	for ( const common::ranked_symbol < SymbolType, RankType > & ssymb : rte.getSubstitutionAlphabet ( ) )
		subMap.insert ( std::make_pair ( ssymb, std::set < common::ranked_symbol < SymbolType, RankType > > { } ) );

	 /* recursively compute follow */
	return rte.getRTE ( ).getStructure ( ).template accept < std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > >, GlushkovFollow::Formal < SymbolType, RankType > > ( symbol, rte.getSubstitutionAlphabet ( ), subMap );
}

// -----------------------------------------------------------------------------

template < class T >
void GlushkovFollow::cartesian_rec ( const std::vector < std::vector < T > > & input, std::vector < std::vector < T > > & ret, std::vector < T > & current, size_t depth ) {
	if ( depth == input.size ( ) )
		ret.push_back ( current );
	else
		for ( size_t i = 0; i < input[depth].size ( ); i++ ) {
			current[depth] = input[depth][i];
			cartesian_rec ( input, ret, current, depth + 1 );
		}

}

template < class T >
std::vector < std::vector < T > > GlushkovFollow::cartesian ( const std::vector < std::vector < T > > & input ) {
	std::vector < std::vector < T > > ret;
	std::vector < T > current ( input.size ( ), T ( 0, 0 ) );

	cartesian_rec ( input, ret, current, 0 );

	return ret;
}

template < class SymbolType, class RankType >
void GlushkovFollow::preprocessSubMap ( const std::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK, std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & subMap ) {
	for ( bool change = true; change; change = false )
		for ( std::pair < const common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & kv : subMap ) {
			std::set < common::ranked_symbol < SymbolType, RankType > > & substSet = kv.second;

			for ( auto eIter = substSet.begin ( ); eIter != substSet.end ( ); ) {
				if ( alphabetK.count ( * eIter ) == 0 )
					++ eIter;
				else {
					auto it = subMap.find ( * eIter );
					size_t oldSize = substSet.size ( );
					substSet.insert ( it->second.begin ( ), it->second.end ( ) );
					change = ( oldSize != substSet.size ( ) ); // something was added
					eIter = substSet.erase ( eIter );
				}
			}
		}
}

template < class SymbolType, class RankType >
std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollow::replaceConstants ( const std::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK, const std::vector < common::ranked_symbol < SymbolType, RankType > > & follow, const std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & subMap2 ) {
	std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > subMap ( subMap2 );
	preprocessSubMap ( alphabetK, subMap );

	std::vector < std::vector < common::ranked_symbol < SymbolType, RankType > > > input;

	for ( const common::ranked_symbol < SymbolType, RankType > & e : follow ) {
		if ( alphabetK.count ( e ) > 0 )
			input.push_back ( std::vector < common::ranked_symbol < SymbolType, RankType > > ( subMap.at ( e ).begin ( ), subMap.at ( e ).end ( ) ) );
		else
			input.push_back ( std::vector < common::ranked_symbol < SymbolType, RankType > > { e } );
	}

	 /* now do the cartesian product on "input" */
	std::vector < std::vector < common::ranked_symbol < SymbolType, RankType > > > followSet = cartesian ( input );
	return std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > ( followSet.begin ( ), followSet.end ( ) );
}

// -----------------------------------------------------------------------------

template < class SymbolType, class RankType >
std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollow::Formal < SymbolType, RankType >::visit ( const rte::FormalRTEAlternation < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbolF, const std::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK, std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & subMap ) {
	std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > ret, tmp;

	tmp = node.getLeftElement ( ).template accept < std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > >, GlushkovFollow::Formal < SymbolType, RankType > > ( symbolF, alphabetK, subMap );
	ret.insert ( tmp.begin ( ), tmp.end ( ) );

	tmp = node.getRightElement ( ).template accept < std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > >, GlushkovFollow::Formal < SymbolType, RankType > > ( symbolF, alphabetK, subMap );
	ret.insert ( tmp.begin ( ), tmp.end ( ) );

	return ret;
}

template < class SymbolType, class RankType >
std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollow::Formal < SymbolType, RankType >::visit ( const rte::FormalRTESubstitution < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbolF, const std::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK, std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & subMap ) {

	std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > subMap2 ( subMap );
	auto itMap = subMap2.find ( node.getSubstitutionSymbol ( ).getSymbol ( ) );

	itMap->second.clear ( );

	for ( const auto & s : node.getRightElement ( ).template accept < std::set < common::ranked_symbol < SymbolType, RankType > >, GlushkovFirst::Formal < SymbolType, RankType > > ( ) )
		itMap->second.insert ( s );

	/*
	 * E sub F
	 *   1. if symbolF in F subtree, then Follow(F, symbolF);
	 *   2. if symbolF in E subtree, then Follow(E, symbolF);
	 */

	if ( node.getLeftElement ( ).template accept < bool, GlushkovPos::Formal < SymbolType, RankType > > ( symbolF ) )
		return node.getLeftElement ( ).template accept < std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > >, GlushkovFollow::Formal < SymbolType, RankType > > ( symbolF, alphabetK, subMap2 );
	else
		return node.getRightElement ( ).template accept < std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > >, GlushkovFollow::Formal < SymbolType, RankType > > ( symbolF, alphabetK, subMap );
}

template < class SymbolType, class RankType >
std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollow::Formal < SymbolType, RankType >::visit ( const rte::FormalRTEIteration < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbolF, const std::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK, std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & subMap ) {

	std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > ret;

	for ( const auto & s : node.getElement ( ).template accept < std::set < common::ranked_symbol < SymbolType, RankType > >, GlushkovFirst::Formal < SymbolType, RankType > > ( ) )
		subMap[node.getSubstitutionSymbol ( ).getSymbol ( )].insert ( s );

	return node.getElement ( ).template accept < std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > >, GlushkovFollow::Formal < SymbolType, RankType > > ( symbolF, alphabetK, subMap );
}

template < class SymbolType, class RankType >
std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollow::Formal < SymbolType, RankType >::visit ( const rte::FormalRTESymbolAlphabet < SymbolType, RankType > & node, const common::ranked_symbol < SymbolType, RankType > & symbolF, const std::set < common::ranked_symbol < SymbolType, RankType > > & alphabetK, std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & subMap ) {

	std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > ret, tmp;

	if ( symbolF == node.getSymbol ( ) ) {
		std::vector < common::ranked_symbol < SymbolType, RankType > > children;

		for ( const std::smart_ptr < const rte::FormalRTESymbol < SymbolType, RankType > > & c : node.getElements ( ) )
			children.push_back ( c->getSymbol ( ) );

		return replaceConstants ( alphabetK, children, subMap );
	}

	for ( const auto & c : node.getElements ( ) ) {
		tmp = c->template accept < std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > >, GlushkovFollow::Formal < SymbolType, RankType > > ( symbolF, alphabetK, subMap );
		ret.insert ( tmp.begin ( ), tmp.end ( ) );
	}

	return ret;
}

template < class SymbolType, class RankType >
std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollow::Formal < SymbolType, RankType >::visit ( const rte::FormalRTESymbolSubst < SymbolType, RankType > & /* node */, const common::ranked_symbol < SymbolType, RankType > & /* symbolF */, const std::set < common::ranked_symbol < SymbolType, RankType > > & /* alphabetK */, std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & /* subMap */ ) {
	return std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > ( );
}

template < class SymbolType, class RankType >
std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > GlushkovFollow::Formal < SymbolType, RankType >::visit ( const rte::FormalRTEEmpty < SymbolType, RankType > & /* node */, const common::ranked_symbol < SymbolType, RankType > & /* symbolF */, const std::set < common::ranked_symbol < SymbolType, RankType > > & /* alphabetK */, std::map < common::ranked_symbol < SymbolType, RankType >, std::set < common::ranked_symbol < SymbolType, RankType > > > & /* subMap */ ) {
	return std::set < std::vector < common::ranked_symbol < SymbolType, RankType > > > ( );
}

} /* namespace rte */

#endif /* RTE_GLUSHKOV_FOLLOW_H_ */
