/*
 * BoyerMooreHorspool.h
 *
 *  Created on: 5. 11. 2014
 *      Author: Radomir Polach, Tomas Pecka
 */

#ifndef _STRINGOLOGY_BOYER_MOORE_HORSPOOL_H_
#define _STRINGOLOGY_BOYER_MOORE_HORSPOOL_H_

#include <set>
#include <map>
#include <measure>

#include <core/multipleDispatch.hpp>

#include <string/String.h>
#include <string/StringFeatures.h>
#include <string/LinearString.h>

#include <string/properties/BadCharacterShiftTable.h>

namespace stringology {

namespace exact {

/**
 * Implementation of BMH for MI(E+\eps)-EVY course 2014
 * To get rid of zeros in BCS table we ignore last haystack character
 */
class BoyerMooreHorspool : public alib::DoubleDispatch < BoyerMooreHorspool, std::set < unsigned >, const string::StringBase &, const string::StringBase & > {
public:
	/**
	 * Search for pattern in linear string.
	 * @return set set of occurences
	 */
	static std::set < unsigned > match ( const string::String & subject, const string::String & pattern );

	template < class SymbolType >
	static std::set < unsigned > match ( const string::LinearString < SymbolType > & subject, const string::LinearString < SymbolType > & pattern );

};

template < class SymbolType >
std::set<unsigned> BoyerMooreHorspool::match(const string::LinearString < SymbolType >& string, const string::LinearString < SymbolType >& pattern) {
	std::set<unsigned> occ;

	measurements::start ( "Preprocess", measurements::Type::PREPROCESS );
	std::map<SymbolType, size_t> bcs = string::properties::BadCharacterShiftTable::bcs(pattern); //NOTE: the subjects alphabet must be a subset or equal to the pattern
	measurements::end ( );

	measurements::start ( "Algorithm", measurements::Type::ALGORITHM );
	size_t haystack_offset = 0;
	while(haystack_offset + pattern.getContent().size() <= string.getContent().size()) {
		size_t i = pattern.getContent().size();
		while(i > 0 && string.getContent()[haystack_offset + i - 1] == pattern.getContent()[i - 1]) {
			i--;
		}

		// Yay, there is match!!!
		if(i == 0) occ.insert(haystack_offset);
		haystack_offset += bcs[string.getContent()[haystack_offset + pattern.getContent().size() - 1]];
		//std::cout << haystack_offset << std::endl;
	}
	measurements::end ( );

	return occ;
}

} /* namespace exact */

} /* namespace stringology */

#endif /* _STRINGOLOGY_BOYER_MOORE_HORSPOOL_H_ */
