/*
 * ExactMultiNondeterministicSubsequenceAutomaton.h
 *
 *  Created on: 7. 4. 2015
 *      Author: Jan Travnicek
 */

#ifndef _EXACT_MULTI_NONDETERMINISTIC_SUBSEQUENCE_AUTOMATON__H_
#define _EXACT_MULTI_NONDETERMINISTIC_SUBSEQUENCE_AUTOMATON__H_

#include <automaton/Automaton.h>
#include <automaton/FSM/EpsilonNFA.h>
#include <string/LinearString.h>
#include <string/String.h>
#include <container/ObjectsPair.h>
#include <primitive/Unsigned.h>

namespace stringology {

namespace indexing {

class ExactMultiNondeterministicSubsequenceAutomaton {
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static automaton::Automaton construct ( const std::set < string::String > & text );

	template < class SymbolType >
	static automaton::EpsilonNFA < SymbolType, DefaultEpsilonType, std::pair < unsigned, unsigned > > construct ( const std::set < string::LinearString < SymbolType > > & text);
};

template < class SymbolType >
automaton::EpsilonNFA < SymbolType, DefaultEpsilonType, std::pair < unsigned, unsigned > > ExactMultiNondeterministicSubsequenceAutomaton::construct ( const std::set < string::LinearString < SymbolType > > & texts) {
	automaton::EpsilonNFA < SymbolType, DefaultEpsilonType, std::pair < unsigned, unsigned > > res ( std::make_pair ( 0u, 0u ) );
	res.addFinalState ( std::make_pair ( 0u, 0u ) );

	unsigned j = 1;
	for ( const string::LinearString < SymbolType > & text : texts ) {
		res.addInputSymbols ( text.getAlphabet ( ) );

		res.addState ( std::make_pair ( 0u, j ) );
		res.addFinalState ( std::make_pair ( 0u, j ) );
		res.addTransition ( std::make_pair ( 0u, 0u ), std::make_pair ( 0u, j ) );

		unsigned i = 1;
		for ( const SymbolType & symbol : text.getContent ( ) ) {
			res.addState ( std::make_pair ( i, j ) );
			res.addFinalState ( std::make_pair ( i, j ) );

			res.addTransition ( std::make_pair ( i - 1, j ), symbol, std::make_pair ( i, j ) );
			res.addTransition ( std::make_pair ( i - 1, j ), std::make_pair ( i, j ) );
			i++;
		}
		j++;
	}

	return res;
}

} /* namespace indexing */

} /* namespace stringology */

#endif /* _EXACT_MULTI_NONDETERMINISTIC_SUBSEQUENCE_AUTOMATON__H_ */
