/*
 * BNDMMatcherConstruction.h
 *
 *  Created on: 6. 2. 2017
 *      Author: Jan Travnicek
 */

#ifndef BNDM_MATCHER_CONSTRUCTION_H_
#define BNDM_MATCHER_CONSTRUCTION_H_

#include <indexes/stringology/BNDMMatcher.h>
#include <string/String.h>
#include <string/LinearString.h>
#include <core/multipleDispatch.hpp>
#include <exception/CommonException.h>

namespace stringology {

namespace matching {

/**
 * Constructs a bit parallel index for given string.
 *
 */

class BNDMMatcherConstruction : public alib::SingleDispatch < BNDMMatcherConstruction, indexes::stringology::BNDMMatcher < >, const string::StringBase & > {
public:
	/**
	 * Creates suffix trie
	 * @param string string to construct suffix trie for
	 * @return automaton
	 */
	static indexes::stringology::BNDMMatcher < > construct ( const string::String & string );

	template < class SymbolType, size_t BitmaskBitCount = 64 >
	static indexes::stringology::BNDMMatcher < SymbolType, BitmaskBitCount > construct ( const string::LinearString < SymbolType > & string );

};

template < class SymbolType, size_t BitmaskBitCount >
indexes::stringology::BNDMMatcher < SymbolType, BitmaskBitCount > BNDMMatcherConstruction::construct ( const string::LinearString < SymbolType > & w ) {
	size_t bitmaskLength = std::min ( w.getContent ( ).size ( ), BitmaskBitCount );

	std::map < SymbolType, std::bitset < BitmaskBitCount > > res;
	for ( const SymbolType & symbol : w.getAlphabet ( ) )
		res [ symbol ] = std::bitset < BitmaskBitCount > ( 0 );

	for ( unsigned i = 0; i < bitmaskLength; ++i )
		res [ w.getContent ( ) [ i ] ] [ bitmaskLength - i - 1 ] = true;

	return indexes::stringology::BNDMMatcher < SymbolType, BitmaskBitCount > ( w.getAlphabet ( ), res, w.getContent ( ) );
}

} /* namespace matching */

} /* namespace stringology */

#endif /* BNDM_MATCHER_CONSTRUCTION_H_ */
