/*
 * Author: Radovan Cerveny
 */

#include "OracleMatcherConstruction.h"
#include <stringology/indexing/ExactSuffixAutomaton.h>
#include <registration/AlgoRegistration.hpp>

namespace stringology {

namespace matching {

automaton::Automaton OracleMatcherConstruction::construct ( const string::String & pattern ) {
	return dispatch ( pattern.getData ( ) );
}

automaton::DFA < > OracleMatcherConstruction::construct ( const string::LinearString < > & pattern ) {
	auto patternData = pattern.getContent ( );
	reverse ( patternData.begin ( ), patternData.end ( ) );
	string::LinearString < > reversedPattern ( pattern.getAlphabet ( ), std::move ( patternData ) );

	return stringology::indexing::ExactSuffixAutomaton::construct ( reversedPattern );
}

auto OracleMatcherConstructionLinearString = registration::OverloadRegister < OracleMatcherConstruction, automaton::DFA < >, string::LinearString < > > ( OracleMatcherConstruction::construct );

} /* namespace matching */

} /* namespace stringology */
