/*
 * Author: Radovan Cerveny
 */

#ifndef STRINGOLOGY_BACKWARD_ORACLE_MATCHING_H__
#define STRINGOLOGY_BACKWARD_ORACLE_MATCHING_H__

#include <string/String.h>
#include <string/StringFeatures.h>
#include <automaton/Automaton.h>
#include <automaton/AutomatonFeatures.h>
#include <core/multipleDispatch.hpp>

#include <set>

namespace stringology {

namespace query {

/**
 * Implementation of Backward Oracle Matching.
 */
class BackwardOracleMatching : public alib::DoubleDispatch <BackwardOracleMatching, std::set < unsigned >, const string::StringBase &, const automaton::AutomatonBase & > {
public:
	/**
	 * Search for pattern in linear string.
	 * @return set set of occurences
	 */
	static std::set < unsigned > match ( const string::String & subject, const automaton::Automaton & factorOracle );
	static std::set < unsigned > match ( const string::LinearString < > & subject, const automaton::DFA < > & factorOracle );

};

} /* namespace query */

} /* namespace stringology */

#endif /* STRINGOLOGY_BACKWARD_ORACLE_MATCHING_H__ */
