/*
 * NormalizeTreeLabels.h
 *
 *  Created on: 10. 5. 2017
 *      Author: Aleksandr Shatrovskii
 */

#ifndef _ARBOLOGY_NORMALIZE_TREE_LABELS_H_
#define _ARBOLOGY_NORMALIZE_TREE_LABELS_H_

#include <tree/TreeFeatures.h>
#include <alphabet/SymbolFeatures.h>
#include <core/multipleDispatch.hpp>

#include <map>
#include <vector>
#include <tree>
#include <alphabet/RankedSymbol.h>
#include <primitive/Unsigned.h>

#include "properties/SubtreeJumpTable.h"

#include <tree/ranked/RankedTree.h>
#include <global/GlobalData.h>

namespace tree {

/**
 * Simple computation of subtree repeats
 */
class NormalizeTreeLabels : public alib::SingleDispatch < NormalizeTreeLabels, tree::Tree, const tree::TreeBase & > {
	template < class SymbolType, class RankType >
	static ext::tree < common::ranked_symbol < unsigned, RankType > > normalize ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & tree, std::map < common::ranked_symbol < SymbolType, RankType >, unsigned > & mapping, unsigned & counter );

public:
	/**
	 * Compute a same shaped tree with nodes containing unique subtree ids.
	 * @return Tree of repeats
	 */
	static tree::Tree normalize ( const tree::Tree & pattern );

	/**
	 * Compute a same shaped tree with nodes containing unique subtree ids.
	 * @return Tree of repeats
	 */
	template < class SymbolType, class RankType >
	static tree::RankedTree < unsigned, RankType > normalize ( const tree::RankedTree < SymbolType, RankType > & tree );

};

template < class SymbolType, class RankType >
ext::tree < common::ranked_symbol < unsigned, RankType > > NormalizeTreeLabels::normalize ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & tree, std::map < common::ranked_symbol < SymbolType, RankType >, unsigned > & mapping, unsigned & counter ) {

	std::vector < ext::tree < common::ranked_symbol < unsigned, RankType > > > children;

	for ( const ext::tree < common::ranked_symbol < SymbolType, RankType > > & child : tree )
		children.push_back ( normalize < SymbolType, RankType > ( child, mapping, counter ) );

	auto newLabelIt = mapping.find ( tree.getData ( ) );
	unsigned newLabel;

	if ( newLabelIt == mapping.end ( ) ) {
		mapping.insert ( std::make_pair ( tree.getData ( ), counter ) );
		newLabel = counter;
		counter += 1;
	} else {
		newLabel = newLabelIt->second;
	}

	return ext::tree < common::ranked_symbol < unsigned, RankType > > ( common::ranked_symbol < unsigned, RankType > ( newLabel, tree.getData ( ).getRank ( ) ), std::move ( children ) );

}

template < class SymbolType, class RankType >
tree::RankedTree < unsigned, RankType > NormalizeTreeLabels::normalize ( const tree::RankedTree < SymbolType, RankType > & tree ) {

	std::map < common::ranked_symbol < SymbolType, RankType >, unsigned > mapping;
	unsigned counter = 1;

	return tree::RankedTree < unsigned, RankType > ( normalize ( tree.getContent ( ), mapping, counter ) );
}

} /* namespace tree */

#endif /* _ARBOLOGY_NORMALIZE_TREE_LABELS_H_ */
