#include "RegExpDerivationTest.h"

#include "regexp/transform/RegExpDerivation.h"
#include <string/String.h>

#include <factory/StringDataFactory.hpp>
#include "factory/XmlDataFactory.hpp"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( RegExpDerivationTest, "regexp" );
CPPUNIT_TEST_SUITE_REGISTRATION( RegExpDerivationTest );

void RegExpDerivationTest::setUp() {
}

void RegExpDerivationTest::tearDown() {
}

void RegExpDerivationTest::testRegExpDerivation() {
    // https://edux.fit.cvut.cz/oppa/BI-AAG/cviceni/aag-cv04.pdf

    // Exc. 13
    this->ExecSingleTest("0 1 0 + 1 0 1 + 0*1 + 1*0", "0", "#E 1 0+#0 0 1+#E 0* 1+#0+#0 1* 0+#E");
    this->ExecSingleTest("0 1 0 + 1 0 1 + 0*1 + 1*0", "0 0", "#0 1 0+#0 0+#0 0 1+(#0 0*+#E 0*) 1+#0+#0+#0 1* 0+#0");

    // Exc. 14
    this->ExecSingleTest("(0 1*0 + 1 0*)*", "1", "(#0 1* 0+#E 0*) (0 1* 0+1 0*)*");
    this->ExecSingleTest("(0 1*0 + 1 0*)*", "1 0", "(#0 1* 0+#0 0*+#E 0*) (0 1* 0+1 0*)*+(#E 1* 0+#0 0*) (0 1* 0+1 0*)*");

    // Exc. 15
    this->ExecSingleTest("1 0*1*0", "1 0 0", "#0 0* 1* 0+(#0 0*+#E 0*) 1* 0+#0 1* 0+#E+#0 1* 0+#0");
}

void RegExpDerivationTest::ExecSingleTest(std::string regexp_str, std::string string_str, std::string result) {
    regexp::RegExp regexp0 = alib::StringDataFactory::fromString (regexp_str);
    std::deque<sax::Token> tokens = alib::XmlDataFactory::toTokens(regexp0);
    regexp::RegExp regexp = alib::XmlDataFactory::fromTokens (std::move(tokens));

    string::String string0 = alib::StringDataFactory::fromString ("\"" + string_str + "\"");
    std::deque<sax::Token> tokens2 = alib::XmlDataFactory::toTokens(string0);
    string::String string1 = alib::XmlDataFactory::fromTokens (std::move(tokens2));

    string::LinearString < > string = static_cast<const string::LinearString < >&>(string1.getData());

    std::cout << alib::StringDataFactory::toString<regexp::RegExp>(regexp::RegExpDerivation::derivation(regexp, string)) << " == " << std::endl << result << std::endl << std::endl;

    CPPUNIT_ASSERT(alib::StringDataFactory::toString<regexp::RegExp>(regexp::RegExpDerivation::derivation(regexp, string)) == result);
}
