#include "DAWGMatcherTest.h"

#include "string/LinearString.h"
#include "stringology/matching/DAWGMatcherConstruction.h"
#include "stringology/query/BackwardDAWGMatching.h"
#include "stringology/exact/ExactFactorMatch.h"

#include "string/generate/RandomStringFactory.h"
#include "string/generate/RandomSubstringFactory.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( DAWGMatcherTest, "stringology" );
CPPUNIT_TEST_SUITE_REGISTRATION ( DAWGMatcherTest );

void DAWGMatcherTest::setUp ( ) {
}

void DAWGMatcherTest::tearDown ( ) {
}

void DAWGMatcherTest::testBackwardDAWGMatching ( ) {
	std::vector<std::string> subjects;
	std::vector<std::string> patterns;
	std::vector<std::set<unsigned>> expectedOccs;

	subjects.push_back("a"); patterns.push_back("a"); expectedOccs.push_back({0});
	subjects.push_back("a"); patterns.push_back("b"); expectedOccs.push_back({});
	subjects.push_back("alfalfalfa"); patterns.push_back("alfalfalfa"); expectedOccs.push_back({0});
	subjects.push_back("alfalfalfa"); patterns.push_back("blfalfalfa"); expectedOccs.push_back({});
	subjects.push_back("alfalfalfa"); patterns.push_back("alfalfalfb"); expectedOccs.push_back({});
	subjects.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfa"); patterns.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfa"); expectedOccs.push_back({0});
	subjects.push_back("alfalfalfaalfalfalfaabfalfalfaalfalfalfaalfalfalfaalfalfalfaalfa"); patterns.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfa"); expectedOccs.push_back({});
	subjects.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfaa"); patterns.push_back("alfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfalfalfaalfaa"); expectedOccs.push_back({0});
	subjects.push_back("atggccttgcc"); patterns.push_back("gcc"); expectedOccs.push_back({3,8});
	subjects.push_back("aaaaaaaaaa"); patterns.push_back("a"); expectedOccs.push_back({0,1,2,3,4,5,6,7,8,9});
	subjects.push_back("aaaaaaaaaa"); patterns.push_back("aa"); expectedOccs.push_back({0,1,2,3,4,5,6,7,8});


	for(size_t i = 0; i < subjects.size(); ++i) {
		string::String subject = string::stringFrom ( subjects[i] );
		string::String pattern = string::stringFrom ( patterns[i] );
		automaton::Automaton suffixAutomaton = stringology::matching::DAWGMatcherConstruction::construct ( pattern );
		std::set < unsigned > res = stringology::query::BackwardDAWGMatching::match ( subject, suffixAutomaton );
		std::cout << subjects[i] << ' ' << patterns[i] << ' ' << res << std::endl;
		CPPUNIT_ASSERT ( res == expectedOccs[i] );
	}

	auto longSubject = string::generate::RandomStringFactory::generateLinearString (64 * 64, 512, false, true);
	auto longPattern = string::generate::RandomSubstringFactory::generateSubstring(64 * 5, longSubject );
	automaton::DFA < > suffixAutomaton = stringology::matching::DAWGMatcherConstruction::construct ( longPattern );
	std::set < unsigned > res = stringology::query::BackwardDAWGMatching::match ( longSubject, suffixAutomaton );
	std::set < unsigned > resRef = stringology::exact::ExactFactorMatch::match ( longSubject, longPattern );
	std::cout << "long: " << res << std::endl;
	CPPUNIT_ASSERT ( res == resRef);
}

