/*
 * LRParser.cpp
 *
 *  Created on: 30. 4. 2016
 *	  Author: Martin Kocicka
 */

#include "LRParser.h"
#include <alphabet/EndSymbol.h>

#include <stack>
#include <common/createUnique.hpp>

namespace grammar {

namespace parsing {

DefaultSymbolType LRParser::getEndOfInputSymbol ( grammar::CFG < > originalGrammar ) {
	return common::createUnique ( DefaultSymbolType ( alphabet::EndSymbol ( ) ), originalGrammar.getTerminalAlphabet ( ), originalGrammar.getNonterminalAlphabet ( ) );
}

grammar::CFG < > LRParser::getAugmentedGrammar ( grammar::CFG < > originalGrammar ) {
	DefaultSymbolType initialSymbol = common::createUnique ( originalGrammar.getInitialSymbol(), originalGrammar.getTerminalAlphabet ( ), originalGrammar.getNonterminalAlphabet ( ) );

	originalGrammar.addNonterminalSymbol ( initialSymbol );
	originalGrammar.addRule ( initialSymbol, { originalGrammar.getInitialSymbol ( ) } );
	originalGrammar.setInitialSymbol ( initialSymbol );

	return originalGrammar;
}

bool LRParser::parse ( LRActionTable actionTable, LRGotoTable gotoTable, DefaultStateType initialState, std::vector < DefaultSymbolType > input ) {
	std::stack < DefaultStateType > states;
	states.push ( initialState );

	unsigned currentPosition = 0;
	while ( true ) {
		LRActionTable::iterator actionIterator = actionTable.find ( { states.top ( ), input[currentPosition] } );
		if ( actionIterator == actionTable.end ( ) )
			return false;

		switch ( actionIterator->second.first ) {
			case LRAction::Shift:
				states.push ( actionIterator->second.second.get < DefaultStateType > ( ) );

				++currentPosition;
				break;

			case LRAction::Reduce: {
				std::pair < DefaultSymbolType, std::vector < DefaultSymbolType > > reduceBy = actionIterator->second.second.get < std::pair < DefaultSymbolType, std::vector < DefaultSymbolType > > > ( );
				for ( unsigned i = 0; i < reduceBy.second.size ( ); ++i ) {
					states.pop ( );
				}

				LRGotoTable::iterator nextStateIterator = gotoTable.find ( { states.top ( ), reduceBy.first } );
				if (nextStateIterator == gotoTable.end()) {
					return false;
				}

				states.push ( nextStateIterator->second );

				break;
			}

			case LRAction::Accept:
				return true;
				break;
		}
	}

	return false;
}

} /* namespace parsing */

} /* namespace grammar */
