/*
 * FloydWarshall.h
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#ifndef GRAPH_FLOYD_WARSHALL_FORD_H_
#define GRAPH_FLOYD_WARSHALL_FORD_H_

#include <core/multipleDispatch.hpp>
#include <unordered_map>

#include <graph/GraphClasses.h>

namespace graph {

namespace shortestpath {

// Floyd-Warshall only works on graphs without negative-weight cycle
// However it will detect the negative-weight cycle and throw exception
//
// note: negative-weight undirected edge = negative-weight cycle
// note: std::numeric_limits<int>::max() is used as infinity

class FloydWarshall : public alib::SingleDispatch<FloydWarshall, std::unordered_map<Node, std::unordered_map<Node, int>>, const graph::GraphBase & > {
public:
	typedef std::unordered_map<Node, std::unordered_map<Node, int>> Result;

	static Result floydwarshall(const Graph &graph);

	static Result floydwarshall(const DirectedGraph &graph);
	static Result floydwarshall(const UndirectedGraph &graph);
};

} // namespace shortestpath

} // namespace graph

#endif // GRAPH_FLOYD_WARSHALL_FORD_H_
