#include "CornerSubstitution.h"

#include "grammar/ContextFree/CFG.h"
#include "grammar/parsing/CornerSubstitution.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( CornerSubstitution, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION ( CornerSubstitution );

void CornerSubstitution::setUp ( ) {
}

void CornerSubstitution::tearDown ( ) {
}

void CornerSubstitution::testCornerSubstitution ( ) {
	DefaultSymbolType A = DefaultSymbolType ( "A" );
	DefaultSymbolType B = DefaultSymbolType ( "B" );
	DefaultSymbolType C = DefaultSymbolType ( "C" );

	DefaultSymbolType a = DefaultSymbolType ( 'a' );
	DefaultSymbolType b = DefaultSymbolType ( 'b' );
	DefaultSymbolType c = DefaultSymbolType ( 'c' );
	DefaultSymbolType d = DefaultSymbolType ( 'd' );

	grammar::CFG < > grammar ( A );

	grammar.setTerminalAlphabet ( { a, b, c, d } );
	grammar.setNonterminalAlphabet ( { A, B, C } );
	grammar.setInitialSymbol ( A );

	grammar.addRule ( A, std::vector < DefaultSymbolType > { a, B } );
	grammar.addRule ( A, std::vector < DefaultSymbolType > { C, B } );
	grammar.addRule ( B, std::vector < DefaultSymbolType > { c, B } );
	grammar.addRule ( B, std::vector < DefaultSymbolType > { d } );
	grammar.addRule ( C, std::vector < DefaultSymbolType > { a, C } );
	grammar.addRule ( C, std::vector < DefaultSymbolType > { b, B } );

	grammar::CFG < > res = grammar;
	grammar::parsing::CornerSubstitution::cornerSubstitution ( res, a, A );

	grammar::CFG < > comp ( A );

	comp.setTerminalAlphabet ( { a, b, c, d } );
	comp.setNonterminalAlphabet ( { A, B, C } );
	comp.setInitialSymbol ( A );

	comp.addRule ( A, std::vector < DefaultSymbolType > { a, B } );
	comp.addRule ( A, std::vector < DefaultSymbolType > { a, C, B } );
	comp.addRule ( A, std::vector < DefaultSymbolType > { b, B, B } );
	comp.addRule ( B, std::vector < DefaultSymbolType > { c, B } );
	comp.addRule ( B, std::vector < DefaultSymbolType > { d } );
	comp.addRule ( C, std::vector < DefaultSymbolType > { a, C } );
	comp.addRule ( C, std::vector < DefaultSymbolType > { b, B } );

//	std::cout << res << std::endl << comp << std::endl;

	CPPUNIT_ASSERT ( res == comp );
}

void CornerSubstitution::testCornerSubstitution2 ( ) {
	DefaultSymbolType A = DefaultSymbolType ( "A" );
	DefaultSymbolType B = DefaultSymbolType ( "B" );
	DefaultSymbolType C = DefaultSymbolType ( "C" );

	DefaultSymbolType a = DefaultSymbolType ( 'a' );
	DefaultSymbolType b = DefaultSymbolType ( 'b' );
	DefaultSymbolType c = DefaultSymbolType ( 'c' );
	DefaultSymbolType d = DefaultSymbolType ( 'd' );

	grammar::CFG < > grammar ( A );

	grammar.setTerminalAlphabet ( { a, b, c, d } );
	grammar.setNonterminalAlphabet ( { A, B, C } );
	grammar.setInitialSymbol ( A );

	grammar.addRule ( A, std::vector < DefaultSymbolType > { a, B } );
	grammar.addRule ( A, std::vector < DefaultSymbolType > { C, B } );
	grammar.addRule ( B, std::vector < DefaultSymbolType > { c, B } );
	grammar.addRule ( B, std::vector < DefaultSymbolType > { a } );
	grammar.addRule ( C, std::vector < DefaultSymbolType > { a, C } );
	grammar.addRule ( C, std::vector < DefaultSymbolType > { B, b } );

	grammar::CFG < > res = grammar;
	grammar::parsing::CornerSubstitution::cornerSubstitution ( res, a, A );
	grammar::parsing::CornerSubstitution::cornerSubstitution ( res, a, A );

	grammar::CFG < > comp ( A );

	comp.setTerminalAlphabet ( { a, b, c, d } );
	comp.setNonterminalAlphabet ( { A, B, C } );
	comp.setInitialSymbol ( A );

	comp.addRule ( A, std::vector < DefaultSymbolType > { a, B } );
	comp.addRule ( A, std::vector < DefaultSymbolType > { a, C, B } );
	comp.addRule ( A, std::vector < DefaultSymbolType > { a, b, B } );
	comp.addRule ( A, std::vector < DefaultSymbolType > { c, B, b, B } );
	comp.addRule ( B, std::vector < DefaultSymbolType > { c, B } );
	comp.addRule ( B, std::vector < DefaultSymbolType > { a } );
	comp.addRule ( C, std::vector < DefaultSymbolType > { a, C } );
	comp.addRule ( C, std::vector < DefaultSymbolType > { B, b } );

//	std::cout << res << std::endl << comp << std::endl;

	CPPUNIT_ASSERT ( res == comp );
}
