#include "LL1ParseTable.h"

#include "grammar/parsing/LL1ParseTable.h"
#include "string/Epsilon.h"
#include "grammar/ContextFree/CFG.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( LL1ParseTable, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION ( LL1ParseTable );

void LL1ParseTable::setUp ( ) {
}

void LL1ParseTable::tearDown ( ) {
}

void LL1ParseTable::testLL1Table ( ) {
	{
		DefaultSymbolType nE  = DefaultSymbolType ( 'E' );
		DefaultSymbolType nEp = DefaultSymbolType ( "E'" );
		DefaultSymbolType nT  = DefaultSymbolType ( 'T' );
		DefaultSymbolType nTp = DefaultSymbolType ( "T'" );
		DefaultSymbolType nF  = DefaultSymbolType ( 'F' );

		DefaultSymbolType tP = DefaultSymbolType ( '+' );
		DefaultSymbolType tS = DefaultSymbolType ( '*' );
		DefaultSymbolType tL = DefaultSymbolType ( '(' );
		DefaultSymbolType tR = DefaultSymbolType ( ')' );
		DefaultSymbolType tA = DefaultSymbolType ( 'a' );

		grammar::CFG < > grammar ( nE );
		grammar.setTerminalAlphabet ( std::set < DefaultSymbolType > { tP, tS, tL, tR, tA } );
		grammar.setNonterminalAlphabet ( std::set < DefaultSymbolType > { nE, nEp, nT, nTp, nF } );

		std::vector < DefaultSymbolType > rhsE1 ( { nT, nEp } );
		std::vector < DefaultSymbolType > rhsEp1 ( { tP, nT, nEp } );
		std::vector < DefaultSymbolType > rhsEp2 ( { } );
		std::vector < DefaultSymbolType > rhsT1 ( { nF, nTp } );
		std::vector < DefaultSymbolType > rhsTp1 ( { tS, nF, nTp } );
		std::vector < DefaultSymbolType > rhsTp2 ( { } );
		std::vector < DefaultSymbolType > rhsF1 ( { tA } );
		std::vector < DefaultSymbolType > rhsF2 ( { tL, nE, tR } );

		grammar.addRule ( nE, rhsE1 );
		grammar.addRule ( nEp, rhsEp1 );
		grammar.addRule ( nEp, rhsEp2 );
		grammar.addRule ( nT, rhsT1 );
		grammar.addRule ( nTp, rhsTp1 );
		grammar.addRule ( nTp, rhsTp2 );
		grammar.addRule ( nF, rhsF1 );
		grammar.addRule ( nF, rhsF2 );

		 // --------------------------------------------------
		std::map < std::pair < ext::variant < DefaultSymbolType, string::Epsilon < > >, DefaultSymbolType >, std::set < std::vector < DefaultSymbolType > > > parseTable;

		parseTable[std::make_pair ( tA, nE )].insert ( rhsE1 );
		parseTable[std::make_pair ( tL, nE )].insert ( rhsE1 );

		parseTable[std::make_pair ( tP, nEp )].insert ( rhsEp1 );
		parseTable[std::make_pair ( tR, nEp )].insert ( rhsEp2 );
		parseTable[std::make_pair ( string::Epsilon < >::EPSILON, nEp )].insert ( rhsEp2 );

		parseTable[std::make_pair ( tA, nT )].insert ( rhsT1 );
		parseTable[std::make_pair ( tL, nT )].insert ( rhsT1 );

		parseTable[std::make_pair ( tS, nTp )].insert ( rhsTp1 );
		parseTable[std::make_pair ( tP, nTp )].insert ( rhsTp2 );
		parseTable[std::make_pair ( tR, nTp )].insert ( rhsTp2 );
		parseTable[std::make_pair ( string::Epsilon < >::EPSILON, nTp )].insert ( rhsTp2 );

		parseTable[std::make_pair ( tA, nF )].insert ( rhsF1 );
		parseTable[std::make_pair ( tL, nF )].insert ( rhsF2 );

		// --------------------------------------------------

		std::map < std::pair < ext::variant < DefaultSymbolType, string::Epsilon < > >, DefaultSymbolType >, std::set < std::vector < DefaultSymbolType > > > parseTableAlgo = grammar::parsing::LL1ParseTable::parseTable ( grammar );

		std::cout << parseTable << std::endl;
		std::cout << parseTableAlgo << std::endl;
		CPPUNIT_ASSERT ( parseTable == parseTableAlgo );
	}
}
