#include "SLR1ParseTable.h"

#include "grammar/parsing/LRParser.h"
#include "grammar/parsing/SLR1ParseTable.h"

#include <label/LR0ItemsLabel.h>

#include <variant>
#include <vector>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( SLR1ParseTable, "grammar" );
CPPUNIT_TEST_SUITE_REGISTRATION ( SLR1ParseTable );

void SLR1ParseTable::setUp ( ) {
}

void SLR1ParseTable::tearDown ( ) {
}

grammar::CFG < > SLR1ParseTable::getExpressionGrammar() {
	grammar::CFG < > expressionGrammar(
		{ E, T, F },
		{ plus, times, leftParenthesis, rightParenthesis, identifier },
		E
	);
	
	expressionGrammar.addRule ( { E }, { E, plus, T } );
	expressionGrammar.addRule ( { E }, { T } );
	expressionGrammar.addRule ( { T }, { T, times, F } );
	expressionGrammar.addRule ( { T }, { F } );
	expressionGrammar.addRule ( { F }, { leftParenthesis, E, rightParenthesis } );
	expressionGrammar.addRule ( { F }, { identifier } );
	
	return expressionGrammar;
}

void SLR1ParseTable::testActionTable ( ) {
	std::vector < grammar::parsing::LR0Items > items(12);
	
	grammar::CFG < > augmentedExpressionGrammar = grammar::parsing::LRParser::getAugmentedGrammar ( getExpressionGrammar ( ) );
	
	items[0] = {
		{
			augmentedExpressionGrammar.getInitialSymbol(),
			{
				{ 0 , { E } }
			}
		},
		{
			E,
			{
				{ 0 , { E, plus, T } },
				{ 0 , { T } }
			}
		},
		{
			T,
			{
				{ 0 , { T, times, F } },
				{ 0 , { F } }
			}
		},
		{
			F,
			{
				{ 0 , { leftParenthesis, E, rightParenthesis } },
				{ 0 , { identifier } }
			}
		}
	};
	
	items[1] = {
		{
			augmentedExpressionGrammar.getInitialSymbol(),
			{
				{ 1 , { E } }
			}
		},
		{
			E,
			{
				{ 1 , { E, plus, T } }
			}
		}
	};

	items[2] = {
		{
			E,
			{
				{ 1 , { T } }
			}
		},
		{
			T,
			{
				{ 1 , { T, times, F } },
			}
		}
	};

	items[3] = {
		{
			T,
			{
				{ 1 , { F } }
			}
		}
	};

	items[4] = {
		{
			E,
			{
				{ 0 , { E, plus, T } },
				{ 0 , { T } }
			}
		},
		{
			T,
			{
				{ 0 , { T, times, F } },
				{ 0 , { F } }
			}
		},
		{
			F,
			{
				{ 0 , { leftParenthesis, E, rightParenthesis } },
				{ 1 , { leftParenthesis, E, rightParenthesis } },
				{ 0 , { identifier } }
			}
		}
	};

	items[5] = {
		{
			F,
			{
				{ 1 , { identifier } }
			}
		}
	};
	
	items[6] = {
		{
			E,
			{
				{ 2 , { E, plus, T } },
			}
		},
		{
			T,
			{
				{ 0 , { T, times, F } },
				{ 0 , { F } }
			}
		},
		{
			F,
			{
				{ 0 , { leftParenthesis, E, rightParenthesis } },
				{ 0 , { identifier } }
			}
		}
	};
	
	items[7] = {
		{
			T,
			{
				{ 2 , { T, times, F } }
			}
		},
		{
			F,
			{
				{ 0 , { leftParenthesis, E, rightParenthesis } },
				{ 0 , { identifier } }
			}
		}
	};
	
	items[8] = {
		{
			E,
			{
				{ 1 , { E, plus, T } }
			}
		},
		{
			F,
			{
				{ 2 , { leftParenthesis, E, rightParenthesis } },
			}
		}
	};
	
	items[9] = {
		{
			E,
			{
				{ 3 , { E, plus, T } }
			}
		},
		{
			T,
			{
				{ 1 , { T, times, F } }
			}
		}
	};
	
	items[10] = {
		{
			T,
			{
				{ 3 , { T, times, F } }
			}
		}
	};
	
	items[11] = {
		{
			F,
			{
				{ 3 , { leftParenthesis, E, rightParenthesis } }
			}
		}
	};
	
	std::vector < DefaultStateType > states;
	for (const grammar::parsing::LR0Items & currentItems : items) {
		states.emplace_back( DefaultStateType ( label::LR0ItemsLabel ( currentItems ) ) );
	}
	
	DefaultSymbolType endOfInput = grammar::parsing::LRParser::getEndOfInputSymbol ( getExpressionGrammar ( ) );
	grammar::parsing::LRActionTable correctActionTable {
		{ { states[0], identifier }, { grammar::parsing::LRAction::Shift, states[5] } },
		{ { states[0], leftParenthesis }, { grammar::parsing::LRAction::Shift, states[4] } },
		{ { states[1], plus }, { grammar::parsing::LRAction::Shift, states[6] } },
		{ { states[1], endOfInput }, { grammar::parsing::LRAction::Accept, states[0] } },
		{ { states[2], times }, { grammar::parsing::LRAction::Shift, states[7] } },
		{ { states[4], identifier }, { grammar::parsing::LRAction::Shift, states[5] } },
		{ { states[4], leftParenthesis }, { grammar::parsing::LRAction::Shift, states[4] } },
		{ { states[6], identifier }, { grammar::parsing::LRAction::Shift, states[5] } },
		{ { states[6], leftParenthesis }, { grammar::parsing::LRAction::Shift, states[4] } },
		{ { states[7], identifier }, { grammar::parsing::LRAction::Shift, states[5] } },
		{ { states[7], leftParenthesis }, { grammar::parsing::LRAction::Shift, states[4] } },
		{ { states[8], plus }, { grammar::parsing::LRAction::Shift, states[6] } },
		{ { states[8], rightParenthesis }, { grammar::parsing::LRAction::Shift, states[11] } },
		{ { states[9], times }, { grammar::parsing::LRAction::Shift, states[7] } }
	};
	
	std::pair < DefaultSymbolType, std::vector < DefaultSymbolType > > reduceBy = { E, { T } };
	correctActionTable.insert ( { { states[2], plus },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	correctActionTable.insert ( { { states[2], rightParenthesis },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	correctActionTable.insert ( { { states[2], endOfInput },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	
	reduceBy = { T, { F } };
	correctActionTable.insert ( { { states[3], plus },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	correctActionTable.insert ( { { states[3], times },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	correctActionTable.insert ( { { states[3], rightParenthesis },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	correctActionTable.insert ( { { states[3], endOfInput },  { grammar::parsing::LRAction::Reduce, reduceBy } } );

	reduceBy = { F, { identifier } };
	correctActionTable.insert ( { { states[5], plus },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	correctActionTable.insert ( { { states[5], times },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	correctActionTable.insert ( { { states[5], rightParenthesis },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	correctActionTable.insert ( { { states[5], endOfInput },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	
	reduceBy = { E, { E, plus, T } };
	correctActionTable.insert ( { { states[9], plus },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	correctActionTable.insert ( { { states[9], rightParenthesis },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	correctActionTable.insert ( { { states[9], endOfInput },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	
	reduceBy = { T, { T, times, F } };
	correctActionTable.insert ( { { states[10], plus },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	correctActionTable.insert ( { { states[10], times },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	correctActionTable.insert ( { { states[10], rightParenthesis },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	correctActionTable.insert ( { { states[10], endOfInput },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	
	reduceBy = { F, { leftParenthesis, E, rightParenthesis } };
	correctActionTable.insert ( { { states[11], plus },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	correctActionTable.insert ( { { states[11], times },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	correctActionTable.insert ( { { states[11], rightParenthesis },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	correctActionTable.insert ( { { states[11], endOfInput },  { grammar::parsing::LRAction::Reduce, reduceBy } } );
	
	grammar::parsing::LRActionTable actionTable = grammar::parsing::SLR1ParseTable::getActionTable ( getExpressionGrammar ( ) );
	
	bool correct = true;
	for ( const grammar::parsing::LRActionTable::value_type & actionTablePair : correctActionTable ) {
		grammar::parsing::LRActionTable::iterator value = actionTable.find ( actionTablePair.first );
		if ( value == actionTable.end ( ) ) {
			correct = false;
			break;
		}

		if ( actionTablePair.second.first != value->second.first ) {
			correct = false;
			break;
		}
		
		bool correctContent = true;
		switch ( actionTablePair.second.first ) {
			case grammar::parsing::LRAction::Shift:
				if ( actionTablePair.second.second.get < DefaultStateType > ( ) != value->second.second.get < DefaultStateType > ( ) ) {
					correctContent = false;
				}
				break;

			case grammar::parsing::LRAction::Reduce:
				if ( actionTablePair.second.second.get < std::pair < DefaultSymbolType, std::vector < DefaultSymbolType > > > ( ) != value->second.second.get < std::pair < DefaultSymbolType, std::vector < DefaultSymbolType > > > ( ) ) {
					correctContent = false;
				}
				break;

			case grammar::parsing::LRAction::Accept:
				break;
		}
		
		if ( !correctContent ) {
			correct = false;
			break;
		}
	}
	
	if (actionTable.size() != correctActionTable.size()) {
		correct = false;
	}
	
	CPPUNIT_ASSERT ( correct );
}

void SLR1ParseTable::testGotoTable ( ) {
	std::vector < grammar::parsing::LR0Items > items(12);
	
	grammar::CFG < > augmentedExpressionGrammar = grammar::parsing::LRParser::getAugmentedGrammar ( getExpressionGrammar ( ) );
	
	items[0] = {
		{
			augmentedExpressionGrammar.getInitialSymbol(),
			{
				{ 0 , { E } }
			}
		},
		{
			E,
			{
				{ 0 , { E, plus, T } },
				{ 0 , { T } }
			}
		},
		{
			T,
			{
				{ 0 , { T, times, F } },
				{ 0 , { F } }
			}
		},
		{
			F,
			{
				{ 0 , { leftParenthesis, E, rightParenthesis } },
				{ 0 , { identifier } }
			}
		}
	};
	
	items[1] = {
		{
			augmentedExpressionGrammar.getInitialSymbol(),
			{
				{ 1 , { E } }
			}
		},
		{
			E,
			{
				{ 1 , { E, plus, T } }
			}
		}
	};

	items[2] = {
		{
			E,
			{
				{ 1 , { T } }
			}
		},
		{
			T,
			{
				{ 1 , { T, times, F } },
			}
		}
	};

	items[3] = {
		{
			T,
			{
				{ 1 , { F } }
			}
		}
	};

	items[4] = {
		{
			E,
			{
				{ 0 , { E, plus, T } },
				{ 0 , { T } }
			}
		},
		{
			T,
			{
				{ 0 , { T, times, F } },
				{ 0 , { F } }
			}
		},
		{
			F,
			{
				{ 0 , { leftParenthesis, E, rightParenthesis } },
				{ 1 , { leftParenthesis, E, rightParenthesis } },
				{ 0 , { identifier } }
			}
		}
	};

	items[5] = {
		{
			F,
			{
				{ 1 , { identifier } }
			}
		}
	};
	
	items[6] = {
		{
			E,
			{
				{ 2 , { E, plus, T } },
			}
		},
		{
			T,
			{
				{ 0 , { T, times, F } },
				{ 0 , { F } }
			}
		},
		{
			F,
			{
				{ 0 , { leftParenthesis, E, rightParenthesis } },
				{ 0 , { identifier } }
			}
		}
	};
	
	items[7] = {
		{
			T,
			{
				{ 2 , { T, times, F } }
			}
		},
		{
			F,
			{
				{ 0 , { leftParenthesis, E, rightParenthesis } },
				{ 0 , { identifier } }
			}
		}
	};
	
	items[8] = {
		{
			E,
			{
				{ 1 , { E, plus, T } }
			}
		},
		{
			F,
			{
				{ 2 , { leftParenthesis, E, rightParenthesis } },
			}
		}
	};
	
	items[9] = {
		{
			E,
			{
				{ 3 , { E, plus, T } }
			}
		},
		{
			T,
			{
				{ 1 , { T, times, F } }
			}
		}
	};
	
	items[10] = {
		{
			T,
			{
				{ 3 , { T, times, F } }
			}
		}
	};
	
	items[11] = {
		{
			F,
			{
				{ 3 , { leftParenthesis, E, rightParenthesis } }
			}
		}
	};
	
	std::vector < DefaultStateType > states;
	for (const grammar::parsing::LR0Items & currentItems : items) {
		states.emplace_back( DefaultStateType ( label::LR0ItemsLabel ( currentItems ) ) );
	}
	
	grammar::parsing::LRGotoTable gotoTable {
		{ { states[0], E }, states[1] },
		{ { states[0], T }, states[2] },
		{ { states[0], F }, states[3] },
		{ { states[4], E }, states[8] },
		{ { states[4], T }, states[2] },
		{ { states[4], F }, states[3] },
		{ { states[6], T }, states[9] },
		{ { states[6], F }, states[3] },
		{ { states[7], F }, states[10] }
	};
	
	CPPUNIT_ASSERT ( grammar::parsing::SLR1ParseTable::getGotoTable ( getExpressionGrammar ( ) ) == gotoTable );
}

void SLR1ParseTable::testUnambiguousNotSLR1Grammar ( ) {
	DefaultSymbolType S = DefaultSymbolType ( 'S' );
	DefaultSymbolType L = DefaultSymbolType ( 'L' );
	DefaultSymbolType R = DefaultSymbolType ( 'R' );
	
	DefaultSymbolType assignment = DefaultSymbolType ( '=' );
	
	grammar::CFG < > unambiguousNotSLR1Grammar(
		{ S, L, R },
		{ assignment, times, identifier },
		S
	);
	
	unambiguousNotSLR1Grammar.addRule ( { S }, { L, assignment, R } );
	unambiguousNotSLR1Grammar.addRule ( { S }, { R } );
	unambiguousNotSLR1Grammar.addRule ( { L }, { times, R } );
	unambiguousNotSLR1Grammar.addRule ( { L }, { identifier } );
	unambiguousNotSLR1Grammar.addRule ( { R }, { L } );
	
	CPPUNIT_ASSERT_THROW ( grammar::parsing::SLR1ParseTable::getActionTable ( unambiguousNotSLR1Grammar ), exception::CommonException );
}
