#ifndef _CLI_HELP_COMMAND_H_
#define _CLI_HELP_COMMAND_H_

#include <command/Command.h>
#include <environment/Environment.h>

namespace cli {

class HelpCommand : public Command {
	std::string m_command;

public:
	HelpCommand ( std::string command ) : m_command ( std::move ( command ) ) {
	}

	virtual Command::Result run ( Environment & ) const override {
		if ( m_command == "execute" ) {
			std::cout << "Execute command executes statements and either prints the result or writes the result to a file" << std::endl;
			std::cout << "statement_list:" << std::endl;
			std::cout << "statement ( | statement )* - at least one statement followed by pipe separated sequence of other statements" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "param:" << std::endl;
			std::cout << "-                   - value from previous result (PreviousResultParam)" << std::endl;
			std::cout << "<( statement_list ) - statement list serving as param (StatementParam)" << std::endl;
			std::cout << "( type ) param      - casted parameter (CastParam)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "identifier   - immediate string param - value is std::string (ImmediateValueParam)" << std::endl;
			std::cout << "number       - immediate number param - value is int (ImmediateValueParam)" << std::endl;
			std::cout << ":identifier  - immediate value from environment - value is std::string (BindedValueParam)" << std::endl;
			std::cout << "$identifier  - value from variable ad (VariableValueParam)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "<identifier  - value from xml file named cd (ImmediateFileParam)" << std::endl;
			std::cout << "<:identifier - value from xml file with name from value in environment (BindedFileParam)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "statement:" << std::endl;
			std::cout << "name ( param )* output_spec - executes a statement with params (SingleStatement)" << std::endl;
			std::cout << "( type ) statement          - the result of statement is casted (CastedStatement)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << ":identifier  - immediate value from environment (BindedValueStatement)" << std::endl;
			std::cout << "$identifier  - value from variable (VariableValueStatement)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "<identifier  - value from xml file (ImmediateFileStatement)" << std::endl;
			std::cout << "<:identifier - value from xml file with name from environment (BindedFileStatement)" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "output_spec:" << std::endl;
			std::cout << ">identifier  - value to xml file (ResultImmediateFileStatement)" << std::endl;
			std::cout << ">:identifier - value to xml file with name from environment (ResultBindedFileStatement)" << std::endl;
			std::cout << ">$identifier - value to variable s (ResultVariableStatement)" << std::endl;
			std::cout << ">            - value is discarded" << std::endl;
			std::cout << "             - nothing specified prints to stdout (ResultPrintStatement)" << std::endl;
		} else if ( m_command == "" ) {
			std::cout << "Simple help for the query language" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "command quit: quits the processor." << std::endl;
			std::cout << "command help: shows this help." << std::endl;
			std::cout << "command introspect: prints available algorithms and algorithm overloads." << std::endl;
			std::cout << "command execute: executes statements" << std::endl;
			std::cout << "" << std::endl;
			std::cout << "for details use help of individual command" << std::endl;
		} else {
			std::cout << "The command " << m_command << " either does not exist or does not have a help entry." << std::endl;
		}
		return Command::Result::OK;
	}
};

} /* namespace cli */

#endif /* _CLI_HELP_COMMAND_H_ */
