/*
 * ValueOperationAbstraction.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _VALUE_OPERATION_ABSTRACTION_HPP_
#define _VALUE_OPERATION_ABSTRACTION_HPP_

#include <abstraction/OperationAbstraction.hpp>
#include <tuple>
#include <memory>
#include <typeindex>
#include <abstraction/Registry.h>

namespace abstraction {

template < class ReturnType >
class ValueOperationAbstraction : public OperationAbstraction {
protected:
	std::unique_ptr < ReturnType > m_data;

public:
	typedef ReturnType return_type;

	template < typename Callable, typename ... Ts, size_t ... Indexes >
	inline void run_helper ( Callable callback, const std::tuple < Ts ... > & inputs, std::index_sequence < Indexes ... > ) {
		if ( m_data == nullptr )
			m_data = std::unique_ptr < ReturnType > ( callback ( std::get < Indexes > ( inputs )->getData ( ) ... ) );
	}

	template < typename Callable, typename T >
	inline void run_helper ( Callable callback, const T & input ) const {
		if ( m_data == nullptr )
			m_data = std::unique_ptr < ReturnType > ( callback ( input->getData ( ) ) );
	}

	bool isReady ( ) const {
		return m_data != nullptr;
	}

	ReturnType & getData ( ) const {
		return * m_data;
	}

	virtual std::shared_ptr < OperationAbstraction > getXmlFileWriterFromResult ( const std::string & filename ) const override {
		std::string param = ext::to_string < ReturnType > ( );
		return Registry::getXmlFileWriterAbstraction ( param, filename );
	}

	virtual std::shared_ptr < OperationAbstraction > getValuePrinterFromResult ( ) const override {
		std::string param = ext::to_string < ReturnType > ( );
		return Registry::getValuePrinterAbstraction ( param );
}

	virtual std::shared_ptr < OperationAbstraction > getCastFromResult ( const std::string & target ) const override {
		std::string param = ext::to_string < ReturnType > ( );
		return Registry::getCastAbstraction ( target, param );
	}

	virtual std::shared_ptr < OperationAbstraction > getNormalizeResult ( ) const override {
		std::string param = ext::to_string < ReturnType > ( );
		return Registry::getNormalizeAbstraction ( param );
	}

	virtual std::shared_ptr < OperationAbstraction > getDowncastResult ( ) const override {
		std::string base = ext::to_string < ReturnType > ( );
		std::string concrete = ext::to_string ( runtime_type ( ) );
		return Registry::getDowncastAbstraction ( concrete, base );
	}

	virtual std::type_index type ( ) const override {
		return std::type_index ( typeid ( ReturnType ) );
	}

	virtual std::type_index runtime_type ( ) const override {
		if ( isReady ( ) )
			return std::type_index ( typeid ( getData ( ) ) );
		else
			throw exception::CommonException ( "Runtime type unknown before evaluation." );
	}

	virtual bool cached ( ) const override {
		return isReady ( );
	}
};

} /* namespace abstraction */

#endif /* _VALUE_OPERATION_ABSTRACTION_HPP_ */
