/*
 * VoidOperationAbstraction.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _VOID_OPERATION_ABSTRACTION_HPP_
#define _VOID_OPERATION_ABSTRACTION_HPP_

#include <abstraction/OperationAbstraction.hpp>
#include <tuple>

namespace abstraction {

class VoidOperationAbstraction : public OperationAbstraction {
public:
	template < typename Callable, typename ... Ts, size_t ... Indexes >
	inline void run_helper ( Callable callback, const std::tuple < Ts ... > & inputs, std::index_sequence < Indexes ... > ) const {
		callback ( std::get < Indexes > ( inputs )->getData ( ) ... );
	}

	template < typename Callable, typename T >
	inline void run_helper ( Callable callback, const T & input ) const {
		callback ( input->getData ( ) );
	}

	virtual std::shared_ptr < OperationAbstraction > getXmlFileWriterFromResult ( const std::string & ) const override {
		throw exception::CommonException ( "Void algorithm does not have a result." );
	}

	virtual std::shared_ptr < OperationAbstraction > getValuePrinterFromResult ( ) const override {
		throw exception::CommonException ( "Void algorithm does not have a result." );
	}

	virtual std::shared_ptr < OperationAbstraction > getCastFromResult ( const std::string & ) const override {
		throw exception::CommonException ( "Void algorithm does not have a result." );
	}

	virtual std::shared_ptr < OperationAbstraction > getNormalizeResult ( ) const override {
		throw exception::CommonException ( "Void algorithm does not have a result." );
	}

	virtual std::shared_ptr < OperationAbstraction > getDowncastResult ( ) const override {
		throw exception::CommonException ( "Void algorithm does not have a result." );
	}

	virtual std::type_index type ( ) const override {
		return std::type_index ( typeid ( void ) );
	}

	virtual std::type_index runtime_type ( ) const override {
		return std::type_index ( typeid ( void ) );
	}

	virtual bool cached ( ) const override {
		return false;
	}
};

} /* namespace abstraction */

#endif /* _VOID_OPERATION_ABSTRACTION_HPP_ */
