/*
 * XmlParserRegistry.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _XML_PARSER_REGISTRY_HPP_
#define _XML_PARSER_REGISTRY_HPP_

#include <memory>
#include <string>

#include <exception/CommonException.h>
#include <abstraction/OperationAbstraction.hpp>

namespace abstraction {

class XmlParserRegistry {
	class Entry {
	public:
		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( std::deque < sax::Token > data ) const = 0;

	};

	template < class Return >
	class EntryImpl : public Entry {
	public:
		EntryImpl ( ) {
		}

		virtual std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( std::deque < sax::Token > data ) const override;
	};

	static std::map < std::string, std::unique_ptr < Entry > > & getEntries ( ) {
		static std::map < std::string, std::unique_ptr < Entry > > parsers;
		return parsers;
	}

public:
	template < class ReturnType >
	static void registerXmlParser ( ) {
		std::string ret = alib::xmlApi < ReturnType >::xmlTagName ( );

		getEntries ( ).insert ( std::make_pair ( ret, std::unique_ptr < Entry > ( new EntryImpl < ReturnType > ( ) ) ) );
	}

	static std::shared_ptr < abstraction::OperationAbstraction > getAbstraction ( std::string typeName, std::deque < sax::Token > tokens ) {
		auto type = getEntries ( ).find ( typeName );
		if ( type == getEntries ( ).end ( ) )
			throw exception::CommonException ( "Entry " + typeName + " not available." );

		return type->second->getAbstraction ( std::move ( tokens ) );
	}
};

} /* namespace abstraction */

#include <abstraction/XmlParserAbstraction.hpp>

namespace abstraction {

template < class Return >
std::shared_ptr < abstraction::OperationAbstraction > XmlParserRegistry::EntryImpl < Return >::getAbstraction ( std::deque < sax::Token > data ) const {
	return std::make_shared < abstraction::XmlParserAbstraction < Return > > ( std::move ( data ) );
}

} /* namespace abstraction */

#endif /* _XML_PARSER_REGISTRY_HPP_ */
