/*
 * Double.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#include "Double.h"
#include <sax/FromXMLParserHelper.h>
#include "Primitive.h"
#include <object/Object.h>

#include <registration/TypeRegistration.hpp>

namespace primitive {

Double::Double(double data) : m_data(data) {

}

PrimitiveBase* Double::clone() const {
	return new Double(*this);
}

PrimitiveBase* Double::plunder() && {
	return new Double(std::move(*this));
}

double Double::getData() const {
	return m_data;
}

Double::operator double ( ) const {
	return m_data;
}

void Double::setData(double data) {
	this->m_data = data;
}

int Double::compare(const Double& other) const {
	if ( m_data < other.m_data )
		return -1;
	else if ( m_data > other.m_data )
		return 1;
	else
		return 0;
}

void Double::operator>>(std::ostream& out) const {
	out << "(Double " << m_data << ")";
}

Double::operator std::string() const {
	return ext::to_string(m_data);
}

Double Double::parse(std::deque<sax::Token>::iterator& input) {
	return Double(parseRaw(input));
}

double Double::parseRaw(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, Double::getXmlTagName());
	double data = ext::from_string < double > (sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, Double::getXmlTagName());
	return data;
}

void Double::compose(std::deque<sax::Token>& out) const {
	compose(out, this->getData());
}

void Double::compose(std::deque<sax::Token>& out, double primitive) {
	out.emplace_back(Double::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(ext::to_string(primitive), sax::Token::TokenType::CHARACTER);
	out.emplace_back(Double::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

PrimitiveBase * Double::inc() && {
	m_data += 1; // TODO fix this
	return this;
}

} /* namespace primitive */

namespace alib {

int xmlApi<double>::parse(std::deque<sax::Token>::iterator& input) {
	return primitive::Double::parseRaw(input);
}

bool xmlApi<double>::first(const std::deque<sax::Token>::const_iterator& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, xmlTagName());
}

std::string xmlApi<double>::xmlTagName() {
	return primitive::Double::getXmlTagName();
}

void xmlApi<double>::compose(std::deque<sax::Token>& output, double data) {
	primitive::Double::compose(output, data);
}

auto doubleType = registration::TypeRegister < primitive::Primitive, primitive::Double > ();
auto doubleType2 = registration::TypeRegister < alib::Object, primitive::Double > ();

} /* namespace alib */

