/*
 * Double.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_DOUBLE_H_
#define PRIMITIVE_DOUBLE_H_

#include "PrimitiveBase.h"
#include <core/xmlApi.hpp>

namespace primitive {

/**
 * Represents symbol in an alphabet.
 */
class Double final : public PrimitiveBase {
protected:
	int m_data;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit Double ( double data );

	virtual PrimitiveBase * clone ( ) const;

	virtual PrimitiveBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	double getData ( ) const;

	explicit operator double ( ) const;

	/**
	 * sets new content of primitive
	 */
	void setData ( double data );

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const Double & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "Double";

		return xmlTagName;
	}

	static Double parse ( std::deque < sax::Token >::iterator & input );
	static double parseRaw ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
	static void compose ( std::deque < sax::Token > & out, double primitive );

	virtual PrimitiveBase * inc ( ) &&;

	typedef Double normalized_type;

	virtual ObjectBase * normalize ( ) && {
		return this;
	}
};

} /* namespace primitive */

namespace alib {

template < >
struct xmlApi < double > {
	static int parse ( std::deque < sax::Token >::iterator & input );
	static bool first ( const std::deque < sax::Token >::const_iterator & input );
	static std::string xmlTagName ( );
	static void compose ( std::deque < sax::Token > & output, double data );
};

} /* namespace alib */

#endif /* PRIMITIVE_DOUBLE_H_ */
