/*
 * String.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "String.h"
#include <sax/FromXMLParserHelper.h>
#include "Primitive.h"
#include <object/Object.h>

#include <registration/TypeRegistration.hpp>

namespace primitive {

String::String(std::string data) : m_data(std::move(data)) {

}

String::String(const char* data) : String((std::string) data) {

}

PrimitiveBase* String::clone() const {
	return new String(*this);
}

PrimitiveBase* String::plunder() && {
	return new String(std::move(*this));
}

const std::string& String::getData() const {
	return m_data;
}

std::string& String::getData() {
	return m_data;
}

int String::compare(const String& other) const {
	return m_data.compare(other.m_data);
}

void String::operator>>(std::ostream& out) const {
	out << "(String " << m_data << ")";
}

String::operator std::string() const {
	return m_data;
}

String String::parse(std::deque<sax::Token>::iterator& input) {
	return String(parseRaw(input));
}

std::string String::parseRaw(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, primitive::String::getXmlTagName());
	std::string data = "";
	if( sax::FromXMLParserHelper::isTokenType(input, sax::Token::TokenType::CHARACTER))
		data = sax::FromXMLParserHelper::popTokenData(input, sax::Token::TokenType::CHARACTER);
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, primitive::String::getXmlTagName());
	return data;
}


void String::compose(std::deque<sax::Token>& out) const {
	compose(out, this->getData());
}

void String::compose(std::deque<sax::Token>& out, std::string primitive) {
	out.emplace_back(primitive::String::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	out.emplace_back(primitive, sax::Token::TokenType::CHARACTER);
	out.emplace_back(primitive::String::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

PrimitiveBase * String::inc() && {
	m_data.push_back('\'');
	return NULL;
}

} /* namespace primitive */

namespace alib {

std::string xmlApi<std::string>::parse(std::deque<sax::Token>::iterator& input) {
	return primitive::String::parseRaw(input);
}

bool xmlApi<std::string>::first(const std::deque<sax::Token>::const_iterator& input) {
	return sax::FromXMLParserHelper::isToken(input, sax::Token::TokenType::START_ELEMENT, xmlTagName());
}

std::string xmlApi<std::string>::xmlTagName() {
	return primitive::String::getXmlTagName();
}

void xmlApi<std::string>::compose(std::deque<sax::Token>& output, std::string data) {
	primitive::String::compose(output, data);
}

auto stringType = registration::TypeRegister < primitive::Primitive, primitive::String > ();
auto stringType2 = registration::TypeRegister < alib::Object, primitive::String > ();

} /* namespace alib */

