#include "DispatchTest.h"
#include <registration/AlgoRegistration.hpp>
#include "base/CommonBase.hpp"
#include <set>
#include <string>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( DispatchTest, "std" );
CPPUNIT_TEST_SUITE_REGISTRATION ( DispatchTest );

void DispatchTest::setUp ( ) {
}

void DispatchTest::tearDown ( ) {
}

namespace dispatch {



class TmpBase : public alib::CommonBase < TmpBase > {
};

class Tmp1 : public TmpBase {
	int m_data;

public:
	int m_moves;
	int m_copies;

	Tmp1 ( int data ) : m_data ( data ), m_moves ( 0 ), m_copies ( 0 ) {
	}

	Tmp1 ( const Tmp1 & other ) : m_data ( other.m_data ), m_moves ( other.m_moves ), m_copies ( other.m_copies + 1 ) {
	}

	Tmp1 ( Tmp1 && other ) noexcept : m_data ( other.m_data ), m_moves ( other.m_moves + 1 ), m_copies ( other.m_copies ) {
	}

	~Tmp1 ( ) noexcept {
	}

	TmpBase * clone ( ) const {
		return new Tmp1 ( * this );
	}

	TmpBase * plunder ( ) && {
		return new Tmp1 ( * this );
	}

	virtual int compare ( const TmpBase & other ) const {
		return -other.compare ( * this );
	}

	virtual int compare ( const Tmp1 & other ) const {
		return this->m_data - other.m_data;
	}

	virtual void operator >>( std::ostream & os ) const {
		os << "Tmp1(" << m_data << ")";
	}

	virtual operator std::string ( ) const {
		return "Tmp1(" + ext::to_string ( m_data ) + ")";
	}

	int getData ( ) const {
		return m_data;
	}

	virtual TmpBase * normalize ( ) && {
		return this;
	}

};

class Tmp2 : public TmpBase {
	double m_data;

public:
	int m_moves;
	int m_copies;

	Tmp2 ( double data ) : m_data ( data ), m_moves ( 0 ), m_copies ( 0 )  {
	}

	Tmp2 ( const Tmp1 & other ) : m_data ( other.getData ( ) ) {
	}

	Tmp2 ( const Tmp2 & other ) : m_data ( other.m_data ), m_moves ( other.m_moves ), m_copies ( other.m_copies + 1 ) {
	}

	Tmp2 ( Tmp2 && other ) noexcept : m_data ( other.m_data ), m_moves ( other.m_moves + 1 ), m_copies ( other.m_copies ) {
	}

	~Tmp2 ( ) noexcept {
	}

	TmpBase * clone ( ) const {
		return new Tmp2 ( * this );
	}

	TmpBase * plunder ( ) && {
		return new Tmp2 ( * this );
	}

	virtual int compare ( const TmpBase & other ) const {
		return -other.compare ( * this );
	}

	virtual int compare ( const Tmp2 & other ) const {
		return this->m_data - other.m_data;
	}

	virtual void operator >>( std::ostream & os ) const {
		os << "Tmp2(" << m_data << ")";
	}

	virtual operator std::string ( ) const {
		return "Tmp2(" + ext::to_string ( m_data ) + ")";
	}

	double getData ( ) const {
		return m_data;
	}

	virtual TmpBase * normalize ( ) && {
		return this;
	}

};

class Tmp3 : public TmpBase {
	std::string m_data;

public:
	int m_moves;
	int m_copies;

	Tmp3 ( const std::string & data ) : m_data ( data ), m_moves ( 0 ), m_copies ( 0 )  {
	}

	Tmp3 ( const Tmp1 & other ) : m_data ( ext::to_string ( other.getData ( ) ) ), m_moves ( 0 ), m_copies ( 0 )  {
	}

	Tmp3 ( const Tmp2 & other ) : m_data ( ext::to_string ( other.getData ( ) ) ), m_moves ( 0 ), m_copies ( 0 )  {
	}

	Tmp3 ( const Tmp3 & other ) : m_data ( other.m_data ), m_moves ( other.m_moves ), m_copies ( other.m_copies + 1 ) {
	}

	Tmp3 ( Tmp3 && other ) noexcept : m_data ( other.m_data ), m_moves ( other.m_moves + 1 ), m_copies ( other.m_copies ) {
	}

	~Tmp3 ( ) noexcept {
	}

	TmpBase * clone ( ) const {
		return new Tmp3 ( * this );
	}

	TmpBase * plunder ( ) && {
		return new Tmp3 ( * this );
	}

	virtual int compare ( const TmpBase & other ) const {
		return -other.compare ( * this );
	}

	virtual int compare ( const Tmp3 & other ) const {
		return this->m_data.compare ( other.m_data );
	}

	virtual void operator >>( std::ostream & os ) const {
		os << "Tmp3(" << m_data << ")";
	}

	virtual operator std::string ( ) const {
		return "Tmp3(" + m_data + ")";
	}

	const std::string & getData ( ) const {
		return m_data;
	}

	virtual TmpBase * normalize ( ) && {
		return this;
	}

};

// -------------------------------------------------------------------------------------------------------------------------------------------------------

class TmpVisitor : public alib::MultipleDispatch < TmpVisitor, int, std::tuple < >, std::tuple < const TmpBase & >, std::tuple < > > {
public:
	static int eval ( const Tmp2 & first ) {
		std::cout << first << std::endl;

		CPPUNIT_ASSERT ( first.m_moves == 0);
		CPPUNIT_ASSERT ( first.m_copies == 0);

		return 2;
	}

	static int eval ( const Tmp3 & first ) {
		std::cout << first << std::endl;

		CPPUNIT_ASSERT ( first.m_moves == 0);
		CPPUNIT_ASSERT ( first.m_copies == 0);

		return 3;
	}

	static int eval ( const TmpBase & first ) {
		return dispatch ( first );
	}

};

int TmpVisitorExtensionTmp1 ( const Tmp1 & first ) {
	std::cout << first << std::endl;

	return 1;
}

auto TmpVisitorTmp1 = registration::OverloadRegister < TmpVisitor, int, Tmp1 > ( TmpVisitorExtensionTmp1 );
auto TmpVisitorTmp2 = registration::OverloadRegister < TmpVisitor, int, Tmp2 > ( TmpVisitor::eval );
auto TmpVisitorTmp3 = registration::OverloadRegister < TmpVisitor, int, Tmp3 > ( TmpVisitor::eval );

} /* namespace dispatch */

void DispatchTest::testDispatch ( ) {
	dispatch::Tmp1 tmp1 ( 2 );

	int a = dispatch::TmpVisitor::eval ( tmp1 );

	CPPUNIT_ASSERT ( a == 1 );

	a = dispatch::TmpVisitor::eval ( ( dispatch::TmpBase & ) tmp1 );
	CPPUNIT_ASSERT ( a == 1 );

	dispatch::Tmp2 tmp2 ( 3.3 );

	a = dispatch::TmpVisitor::eval ( tmp2 );
	CPPUNIT_ASSERT ( a == 2 );

	a = dispatch::TmpVisitor::eval ( ( dispatch::TmpBase & ) tmp2 );
	CPPUNIT_ASSERT ( a == 2 );
}

// -------------------------------------------------------------------------------------------------------------------------------------------------------

namespace dispatch {

class TmpVisitor2 : public alib::SingleDispatchFirstStaticParam < TmpVisitor2, void, int &, const TmpBase & > {
public:
	static void eval ( int & res, const Tmp2 & first ) {
		std::cout << first << std::endl;

		CPPUNIT_ASSERT ( first.m_moves == 0);
		CPPUNIT_ASSERT ( first.m_copies == 0);

		res = 2;
	}

	static void eval ( int & res, const Tmp3 & first ) {
		std::cout << first << std::endl;

		CPPUNIT_ASSERT ( first.m_moves == 0);
		CPPUNIT_ASSERT ( first.m_copies == 0);

		res = 3;
	}

	static void eval ( int & res, const TmpBase & first ) {
		dispatch ( res, first );
	}

};

void TmpVisitor2ExtensionTmp1 ( int & res, const Tmp1 & first ) {
	std::cout << first << std::endl;

	CPPUNIT_ASSERT ( first.m_moves == 0);
	CPPUNIT_ASSERT ( first.m_copies == 0);

	res = 1;
}

auto TmpVisitor2Tmp1 = registration::OverloadRegister < TmpVisitor2, void, Tmp1 > ( TmpVisitor2ExtensionTmp1 );
auto TmpVisitor2Tmp2 = registration::OverloadRegister < TmpVisitor2, void, Tmp2 > ( TmpVisitor2::eval );
auto TmpVisitor2Tmp3 = registration::OverloadRegister < TmpVisitor2, void, Tmp3 > ( TmpVisitor2::eval );

} /* namespace dispatch */

void DispatchTest::testDispatch2 ( ) {
	dispatch::Tmp1 tmp1 ( 2 );

	int a;
	dispatch::TmpVisitor2::eval ( a, tmp1 );

	CPPUNIT_ASSERT ( a == 1 );

	dispatch::TmpVisitor2::eval ( a, ( dispatch::TmpBase & ) tmp1 );
	CPPUNIT_ASSERT ( a == 1 );

	dispatch::Tmp2 tmp2 ( 3.3 );

	dispatch::TmpVisitor2::eval ( a, tmp2 );
	CPPUNIT_ASSERT ( a == 2 );

	dispatch::TmpVisitor2::eval ( a, ( dispatch::TmpBase & ) tmp2 );
	CPPUNIT_ASSERT ( a == 2 );
}

// -------------------------------------------------------------------------------------------------------------------------------------------------------

namespace dispatch {

class TmpVisitor3 : public alib::SingleDispatch < TmpVisitor3, int, TmpBase && > {
public:
	static int eval ( Tmp2 && first ) {
		std::cout << first << std::endl;

		CPPUNIT_ASSERT ( first.m_moves == 0);
		CPPUNIT_ASSERT ( first.m_copies == 0);

		return 2;
	}

	static int eval ( Tmp3 && first ) {
		std::cout << first << std::endl;

		CPPUNIT_ASSERT ( first.m_moves == 0);
		CPPUNIT_ASSERT ( first.m_copies == 0);

		return 3;
	}

	static int eval ( TmpBase && first ) {
		return dispatch ( std::move ( first ) );
	}

};

int TmpVisitor3ExtensionTmp1 ( Tmp1 && first ) {
	std::cout << first << std::endl;

	CPPUNIT_ASSERT ( first.m_moves == 0);
	CPPUNIT_ASSERT ( first.m_copies == 0);

	return 1;
}

auto TmpVisitor3Tmp1 = registration::OverloadRegister < TmpVisitor3, int, Tmp1 > ( TmpVisitor3ExtensionTmp1 );
auto TmpVisitor3Tmp2 = registration::OverloadRegister < TmpVisitor3, int, Tmp2 > ( TmpVisitor3::eval );
auto TmpVisitor3Tmp3 = registration::OverloadRegister < TmpVisitor3, int, Tmp3 > ( TmpVisitor3::eval );

} /* namespace dispatch */

void DispatchTest::testDispatch3 ( ) {
	dispatch::Tmp1 tmp1 ( 2 );

	int a = dispatch::TmpVisitor3::eval ( std::move ( tmp1 ) );

	CPPUNIT_ASSERT ( a == 1 );

	a = dispatch::TmpVisitor3::eval ( std::move ( ( dispatch::TmpBase & ) tmp1 ) );
	CPPUNIT_ASSERT ( a == 1 );

	dispatch::Tmp2 tmp2 ( 3.3 );

	a = dispatch::TmpVisitor3::eval ( std::move ( tmp2 ) );
	CPPUNIT_ASSERT ( a == 2 );

	a = dispatch::TmpVisitor3::eval ( std::move ( ( dispatch::TmpBase & ) tmp2 ) );
	CPPUNIT_ASSERT ( a == 2 );
}
