/*
 * BottomOfTheStackSymbol.h
 *
 *  Created on: Jun 19, 2014
 *      Author: Jan Travnicek
 */

#ifndef BOTTOM_OF_THE_STACK_SYMBOL_H_
#define BOTTOM_OF_THE_STACK_SYMBOL_H_

#include "SymbolBase.h"
#include "Symbol.h"
#include <object/Object.h>

namespace alphabet {

/**
 * Represents blank symbol in an alphabet.
 */
class BottomOfTheStackSymbol : public SymbolBase {
public:
	/**
	 * Creates a blank symbol.
	 * @param symbol name of the symbol
	 */
	explicit BottomOfTheStackSymbol ( );

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const BottomOfTheStackSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	template < typename Base >
	static inline typename std::enable_if <   std::is_integral < Base >::value, Base >::type instance ( );
	template < typename Base >
	static inline typename std::enable_if < ! std::is_integral < Base >::value, Base >::type instance ( );

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "BottomOfTheStackSymbol";

		return xmlTagName;
	}

	static BottomOfTheStackSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual SymbolBase * inc ( ) &&;

	typedef BottomOfTheStackSymbol normalized_type;

	virtual SymbolBase * normalize ( ) && {
		return this;
	}
};

template < typename Base >
inline typename std::enable_if < std::is_integral < Base >::value, Base >::type BottomOfTheStackSymbol::instance ( ) {
	return '_';
}

template < >
inline std::string BottomOfTheStackSymbol::instance < std::string > ( ) {
	return std::string ( 1, BottomOfTheStackSymbol::instance < char > ( ) );
}

template < >
inline BottomOfTheStackSymbol BottomOfTheStackSymbol::instance < BottomOfTheStackSymbol > ( ) {
	return BottomOfTheStackSymbol ( );
}

template < >
inline Symbol BottomOfTheStackSymbol::instance < Symbol > ( ) {
	return Symbol ( BottomOfTheStackSymbol ( ) );
}

template < >
inline alib::Object BottomOfTheStackSymbol::instance < alib::Object > ( ) {
	return alib::Object ( BottomOfTheStackSymbol ( ) );
}

} /* namespace alphabet */

#endif /* BOTTOM_OF_THE_STACK_SYMBOL_H_ */
