/*
 * RankedSymbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Stepan Plachy
 */

#ifndef RANKED_SYMBOL_H_
#define RANKED_SYMBOL_H_

#include <sax/FromXMLParserHelper.h>

#include "SymbolBase.h"
#include "Symbol.h"
#include "SymbolFeatures.h"
#include <common/ranked_symbol.hpp>

#include <alphabet/common/SymbolNormalize.h>

namespace alphabet {

/**
 * Represents symbol in an alphabet.
 */
template < class SymbolType, class RankType >
class RankedSymbol : public SymbolBase, public common::ranked_symbol < SymbolType, RankType > {
public:
	using common::ranked_symbol < SymbolType, RankType >::ranked_symbol;

	explicit RankedSymbol ( common::ranked_symbol < SymbolType, RankType > symbol ) : common::ranked_symbol < SymbolType, RankType > ( std::move ( symbol ) ) {
	}

	virtual SymbolBase * clone ( ) const;
	virtual SymbolBase * plunder ( ) &&;

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const RankedSymbol & other ) const;

	virtual void operator >>( std::ostream & out ) const;

	virtual explicit operator std::string ( ) const;

	using SymbolBase::operator ==;
	using SymbolBase::operator !=;
	using SymbolBase::operator <;
	using SymbolBase::operator <=;
	using SymbolBase::operator >;
	using SymbolBase::operator >=;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "RankedSymbol";

		return xmlTagName;
	}

	static RankedSymbol parse ( std::deque < sax::Token >::iterator & input );
	static common::ranked_symbol < SymbolType, RankType > parseRaw ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;
	static void compose ( std::deque < sax::Token > & out, const common::ranked_symbol < SymbolType, RankType > & input );

	virtual SymbolBase * inc ( ) &&;

	common::ranked_symbol < DefaultSymbolType, DefaultRankType > normalizeRaw ( common::ranked_symbol < SymbolType, RankType > && source ) {
		return common::ranked_symbol < DefaultSymbolType, DefaultRankType > ( SymbolNormalize::normalizeSymbol ( std::move ( source.getSymbol ( ) ) ), DefaultRankType ( std::move ( source.getRank ( ) ) ) );
	}

	typedef RankedSymbol < > normalized_type;

	virtual SymbolBase * normalize ( ) && {
		if ( typeid ( RankedSymbol < > ) == typeid ( RankedSymbol < SymbolType, RankType > ) )
			return this;

		return new RankedSymbol < > ( normalizeRaw ( std::move ( *this ) ) );
	}
};

template < class SymbolType, class RankType >
SymbolBase* RankedSymbol < SymbolType, RankType >::clone() const {
	return new RankedSymbol(*this);
}

template < class SymbolType, class RankType >
SymbolBase* RankedSymbol < SymbolType, RankType >::plunder() && {
	return new RankedSymbol(std::move(*this));
}

template < class SymbolType, class RankType >
int RankedSymbol < SymbolType, RankType >::compare(const RankedSymbol& other) const {
	return static_cast < const common::ranked_symbol < SymbolType, RankType > & > ( * this ).compare ( static_cast < const common::ranked_symbol < SymbolType, RankType > & > ( other ) );
}

template < class SymbolType, class RankType >
void RankedSymbol < SymbolType, RankType >::operator>>(std::ostream& out) const {
	out << static_cast < const common::ranked_symbol < SymbolType, RankType > & > ( * this );
}

template < class SymbolType, class RankType >
RankedSymbol < SymbolType, RankType >::operator std::string () const {
	return (std::string) static_cast < const common::ranked_symbol < SymbolType, RankType > & > ( * this );
}

template < class SymbolType, class RankType >
RankedSymbol < SymbolType, RankType > RankedSymbol < SymbolType, RankType >::parse(std::deque<sax::Token>::iterator& input) {
	return RankedSymbol ( parseRaw ( input ) );
}

template < class SymbolType, class RankType >
common::ranked_symbol < SymbolType, RankType > RankedSymbol < SymbolType, RankType >::parseRaw(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, RankedSymbol::getXmlTagName());
	SymbolType data = alib::xmlApi<SymbolType>::parse(input);
	RankType rank = alib::xmlApi<RankType>::parse(input);
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, RankedSymbol::getXmlTagName());
	return common::ranked_symbol < SymbolType, RankType >(std::move(data), std::move(rank));
}

template < class SymbolType, class RankType >
void RankedSymbol < SymbolType, RankType >::compose(std::deque<sax::Token>& out) const {
	compose ( out, * this );
}

template < class SymbolType, class RankType >
void RankedSymbol < SymbolType, RankType >::compose( std::deque<sax::Token>& out, const common::ranked_symbol < SymbolType, RankType > & symbol ) {
	out.emplace_back(RankedSymbol::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<SymbolType>::compose(out, symbol.getSymbol ( ) );
	alib::xmlApi<RankType>::compose(out, symbol.getRank ( ) );
	out.emplace_back(RankedSymbol::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

template < class SymbolType, class RankType >
SymbolBase* RankedSymbol < SymbolType, RankType >::inc() && {
	this->m_symbol.inc();
	return NULL;
}

} /* namespace alphabet */

namespace alib {

template < typename T, typename R >
struct xmlApi < common::ranked_symbol < T, R > > {
	static common::ranked_symbol < T, R > parse ( std::deque < sax::Token >::iterator & input );
	static bool first ( const std::deque < sax::Token >::const_iterator & input );
	static void compose ( std::deque < sax::Token > & output, const common::ranked_symbol < T, R > & data );
};

template < typename T, typename R >
common::ranked_symbol < T, R > xmlApi < common::ranked_symbol < T, R > >::parse ( std::deque < sax::Token >::iterator & input ) {
	return alphabet::RankedSymbol < T, R >::parseRaw ( input );
}

template < typename T, typename R >
bool xmlApi < common::ranked_symbol < T, R > >::first ( const std::deque < sax::Token >::const_iterator & input ) {
	return sax::FromXMLParserHelper::isToken ( input, sax::Token::TokenType::START_ELEMENT, alphabet::RankedSymbol < T, R >::getXmlTagName() );
}

template < typename T, typename R >
void xmlApi < common::ranked_symbol < T, R > >::compose ( std::deque < sax::Token > & output, const common::ranked_symbol < T, R > & input ) {
	return alphabet::RankedSymbol < T, R >::compose( output, input );
}

} /* namespace alib */

#endif /* RANKED_SYMBOL_H_ */
