/*
 * UniqueSymbol.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_ID_SYMBOL_H_
#define SYMBOL_ID_SYMBOL_H_

#include "Symbol.h"
#include <primitive/Integer.h>

namespace alphabet {

/**
 * Represents symbol in an alphabet.
 */
class UniqueSymbol : public SymbolBase {
protected:
	Symbol m_symbol;
	primitive::Integer m_id;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit UniqueSymbol ( Symbol symbol, primitive::Integer id );

	virtual SymbolBase * clone ( ) const;

	virtual SymbolBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	const Symbol & getSymbol ( ) const;

	const primitive::Integer & getId ( ) const;

	primitive::Integer & getId ( );

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const UniqueSymbol & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "UniqueSymbol";

		return xmlTagName;
	}

	static UniqueSymbol parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual SymbolBase * inc ( ) &&;

	typedef UniqueSymbol normalized_type;

	virtual SymbolBase * normalize ( ) && {
		return this;
	}
};

} /* namespace alphabet */

#endif /* SYMBOL_ID_SYMBOL_H_ */
