/*
 * SymbolNormalize.h
 *
 *  Created on: Apr 8, 2017
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_NORMALIZE_H_
#define SYMBOL_NORMALIZE_H_

#include <vector>
#include <set>

#include <common/ranked_symbol.hpp>

#include <object/AnyObject.h>

namespace alphabet {

/**
 * This class contains methods to print XML representation of automata to the output stream.
 */
class SymbolNormalize {
	template < class SymbolType >
	static DefaultSymbolType normalizeSymbolInternal ( typename std::enable_if < std::is_constructible < DefaultSymbolType, SymbolType >::value, SymbolType && >::type symbol );

	template < class SymbolType >
	static DefaultSymbolType normalizeSymbolInternal ( typename std::enable_if < ! std::is_constructible < DefaultSymbolType, SymbolType >::value, SymbolType && >::type symbol );

public:
	template < class SymbolType >
	static std::set < DefaultSymbolType > normalizeAlphabet ( std::set < SymbolType > && symbols );

	template < class SymbolType, class RankType >
	static std::set < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > normalizeRankedAlphabet ( std::set < common::ranked_symbol < SymbolType, RankType > > && symbols );

	template < class SymbolType >
	static DefaultSymbolType normalizeSymbol ( SymbolType && symbol );

	template < class SymbolType, class RankType >
	static common::ranked_symbol < DefaultSymbolType, DefaultRankType > normalizeRankedSymbol ( common::ranked_symbol < SymbolType, RankType > && symbol );

	template < class SymbolType >
	static std::vector < DefaultSymbolType > normalizeSymbols ( std::vector < SymbolType > && symbols );

	template < class SymbolType, class RankType >
	static std::vector < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > normalizeRankedSymbols ( std::vector < common::ranked_symbol < SymbolType, RankType > > && symbols );
};

template < class SymbolType >
std::set < DefaultSymbolType > SymbolNormalize::normalizeAlphabet ( std::set < SymbolType > && symbols ) {
	std::set < DefaultSymbolType > res;
	for ( SymbolType && symbol : std::make_moveable_set ( symbols ) ) {
		res.insert ( normalizeSymbol ( std::move ( symbol ) ) );
	}
	return res;
}

template < class SymbolType, class RankType >
std::set < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > SymbolNormalize::normalizeRankedAlphabet ( std::set < common::ranked_symbol < SymbolType, RankType > > && symbols ) {
	std::set < common::ranked_symbol < DefaultSymbolType > > res;
	for ( common::ranked_symbol < SymbolType, RankType > && symbol : std::make_moveable_set ( symbols ) ) {
		res.insert ( normalizeRankedSymbol ( std::move ( symbol ) ) );
	}
	return res;
}

template < class SymbolType >
DefaultSymbolType SymbolNormalize::normalizeSymbolInternal ( typename std::enable_if < std::is_constructible < DefaultSymbolType, SymbolType >::value, SymbolType && >::type symbol ) {
	return DefaultSymbolType ( std::move ( symbol ) );
}

template < class SymbolType >
DefaultSymbolType SymbolNormalize::normalizeSymbolInternal ( typename std::enable_if < ! std::is_constructible < DefaultSymbolType, SymbolType >::value, SymbolType && >::type symbol ) {
	return DefaultSymbolType ( alib::AnyObject < SymbolType > ( std::move ( symbol ) ) );
}

template < class SymbolType >
DefaultSymbolType SymbolNormalize::normalizeSymbol ( SymbolType && symbol ) {
	return SymbolNormalize::normalizeSymbolInternal < SymbolType > ( std::move ( symbol ) );
}

template < class SymbolType, class RankType >
common::ranked_symbol < DefaultSymbolType, DefaultRankType > SymbolNormalize::normalizeRankedSymbol ( common::ranked_symbol < SymbolType, RankType > && symbol ) {
	return common::ranked_symbol < DefaultSymbolType, DefaultRankType > ( normalizeSymbol ( std::move ( symbol.getSymbol ( ) ) ), DefaultRankType ( std::move ( symbol.getRank ( ) ) ) );
}

template < class SymbolType >
std::vector < DefaultSymbolType > SymbolNormalize::normalizeSymbols ( std::vector < SymbolType > && symbols ) {
	std::vector < DefaultSymbolType > res;
	for ( SymbolType & symbol : symbols ) {
		res.push_back ( normalizeSymbol ( std::move ( symbol ) ) );
	}
	return res;
}

template < class SymbolType, class RankType >
std::vector < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > SymbolNormalize::normalizeRankedSymbols ( std::vector < common::ranked_symbol < SymbolType, RankType > > && symbols ) {
	std::vector < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > res;
	for ( common::ranked_symbol < SymbolType, RankType > & symbol : symbols ) {
		res.push_back ( normalizeRankedSymbol ( std::move ( symbol ) ) );
	}
	return res;
}

} /* namespace alphabet */

#endif /* SYMBOL_NORMALIZE_H_ */
