/*
 * IndexesNormalize.h
 *
 *  Created on: Apr 7, 2017
 *      Author: Jan Travnicek
 */

#ifndef INDEXES_NORMALIZE_H_
#define INDEXES_NORMALIZE_H_

#include <vector>
#include <tuple>
#include <set>
#include <variant>
#include <trie>

#include <alphabet/common/SymbolNormalize.h>

namespace indexes {

/**
 * This class contains methods to print XML representation of automata to the output stream.
 */
class IndexesNormalize {
	template < class SymbolType, class ValueType >
	static ext::trie < DefaultSymbolType, ValueType > normalizeTrieInner ( ext::trie < SymbolType, ValueType > && node ) {
		std::map < DefaultSymbolType, ext::trie < DefaultSymbolType, ValueType > > children;

		for ( std::pair < SymbolType, ext::trie < SymbolType, ValueType > > && child : std::make_moveable_map ( node.getChildren ( ) ) ) {
			children.insert ( std::make_pair ( alphabet::SymbolNormalize::normalizeSymbol ( std::move ( child.first ) ), normalizeTrieInner ( std::move ( child.second ) ) ) );
		}

		return ext::trie < DefaultSymbolType, ValueType > ( std::move ( node.getData ( ) ), std::move ( children ) );
	}

public:
	template < class SymbolType >
	static ext::trie < DefaultSymbolType, unsigned > normalizeTrie ( ext::trie < SymbolType, unsigned > && trie );

	template < class SymbolType >
	static ext::trie < DefaultSymbolType, ext::variant < void, unsigned > > normalizeTrie ( ext::trie < SymbolType, ext::variant < void, unsigned > > && trie );
};

template < class SymbolType >
ext::trie < DefaultSymbolType, unsigned > IndexesNormalize::normalizeTrie ( ext::trie < SymbolType, unsigned > && trie ) {
	return  normalizeTrieInner ( std::move ( trie ) ) ;
}

template < class SymbolType >
ext::trie < DefaultSymbolType, ext::variant < void, unsigned > > IndexesNormalize::normalizeTrie ( ext::trie < SymbolType, ext::variant < void, unsigned > > && trie ) {
	return  normalizeTrieInner ( std::move ( trie ) ) ;
}

} /* namespace indexes */

#endif /* INDEXES_NORMALIZE_H_ */
