/*
 * ObjectLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author:Jan Travnicek 
 */

#include "ObjectLabel.h"
#include <sax/FromXMLParserHelper.h>
#include "Label.h"
#include <object/Object.h>
#include <registration/TypeRegistration.hpp>
#include "UniqueLabel.h"

namespace label {

ObjectLabel::ObjectLabel(alib::Object label) : m_label(std::move(label)) {

}

LabelBase* ObjectLabel::clone() const {
	return new ObjectLabel(*this);
}

LabelBase* ObjectLabel::plunder() && {
	return new ObjectLabel(std::move(*this));
}

const alib::Object& ObjectLabel::getData() const {
	return m_label;
}

int ObjectLabel::compare(const ObjectLabel& other) const {
	return m_label.getData().compare(other.m_label.getData());
}

void ObjectLabel::operator>>(std::ostream& out) const {
	out << "(ObjectLabel " << m_label << ")";
}

ObjectLabel::operator std::string() const {
	return (std::string) m_label;
}

ObjectLabel ObjectLabel::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, ObjectLabel::getXmlTagName());
	ObjectLabel data(alib::xmlApi<alib::Object>::parse(input));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, ObjectLabel::getXmlTagName());
	return data;
}

void ObjectLabel::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(ObjectLabel::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<alib::Object>::compose(out, m_label);
	out.emplace_back(ObjectLabel::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

LabelBase* ObjectLabel::inc() && {
	return new UniqueLabel(Label(std::move(*this)), primitive::Integer(0));
}

} /* namespace label */

namespace alib {

auto objectLabelType = registration::TypeRegister < label::Label, label::ObjectLabel > ( );
auto objectLabelType2 = registration::TypeRegister < alib::Object, label::ObjectLabel > ( );

} /* namespace alib */
