/*
 * PrimitiveLabel.cpp
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travicek
 */

#include "PrimitiveLabel.h"
#include <sax/FromXMLParserHelper.h>
#include "Label.h"
#include <object/Object.h>
#include <registration/TypeRegistration.hpp>
#include "UniqueLabel.h"

namespace label {

PrimitiveLabel::PrimitiveLabel(primitive::Primitive primitive) : m_primitive(std::move(primitive)) {

}

LabelBase* PrimitiveLabel::clone() const {
	return new PrimitiveLabel(*this);
}

LabelBase* PrimitiveLabel::plunder() && {
	return new PrimitiveLabel(std::move(*this));
}

const primitive::Primitive& PrimitiveLabel::getData() const {
	return m_primitive;
}

primitive::Primitive& PrimitiveLabel::getData() {
	return m_primitive;
}

int PrimitiveLabel::compare(const PrimitiveLabel& other) const {
	return m_primitive.getData().compare(other.m_primitive.getData());
}

void PrimitiveLabel::operator>>(std::ostream& out) const {
	out << "(PrimitiveLabel " << m_primitive << ")";
}

PrimitiveLabel::operator std::string() const {
	return (std::string) m_primitive;
}

PrimitiveLabel PrimitiveLabel::parse(std::deque<sax::Token>::iterator& input) {
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::START_ELEMENT, PrimitiveLabel::getXmlTagName());
	PrimitiveLabel data(alib::xmlApi<primitive::Primitive>::parse(input));
	sax::FromXMLParserHelper::popToken(input, sax::Token::TokenType::END_ELEMENT, PrimitiveLabel::getXmlTagName());
	return data;
}

void PrimitiveLabel::compose(std::deque<sax::Token>& out) const {
	out.emplace_back(PrimitiveLabel::getXmlTagName(), sax::Token::TokenType::START_ELEMENT);
	alib::xmlApi<primitive::Primitive>::compose(out, m_primitive);
	out.emplace_back(PrimitiveLabel::getXmlTagName(), sax::Token::TokenType::END_ELEMENT);
}

LabelBase* PrimitiveLabel::inc() && {
	m_primitive.inc();
	return NULL;
}

} /* namespace label */

namespace alib {

auto primitiveLabelType = registration::TypeRegister < label::Label, label::PrimitiveLabel > ( );
auto primitiveLabelType2 = registration::TypeRegister < alib::Object, label::PrimitiveLabel > ( );

} /* namespace alib */
