/*
 * PrimitiveLabel.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef PRIMITIVE_LABEL_H_
#define PRIMITIVE_LABEL_H_

#include <primitive/Primitive.h>
#include "LabelBase.h"
#include "Label.h"

namespace label {

/**
 * Represents symbol in an alphabet.
 */
class PrimitiveLabel final : public LabelBase {
protected:
	primitive::Primitive m_primitive;

public:
	/**
	 * Creates new symbol with given name.
	 * @param symbol name of the symbol
	 */
	explicit PrimitiveLabel ( primitive::Primitive primitive );

	virtual LabelBase * clone ( ) const;

	virtual LabelBase * plunder ( ) &&;

	/**
	 * @return name of the symbol
	 */
	const primitive::Primitive & getData ( ) const;

	primitive::Primitive & getData ( );

	virtual int compare ( const ObjectBase & other ) const {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	virtual int compare ( const PrimitiveLabel & other ) const;

	virtual void operator >>( std::ostream & ) const;

	virtual explicit operator std::string ( ) const;

	static const std::string & getXmlTagName() {
		static std::string xmlTagName = "PrimitiveLabel";

		return xmlTagName;
	}

	static PrimitiveLabel parse ( std::deque < sax::Token >::iterator & input );

	void compose ( std::deque < sax::Token > & out ) const;

	virtual LabelBase * inc ( ) &&;

	typedef PrimitiveLabel normalized_type;

	virtual LabelBase * normalize ( ) && {
		return this;
	}
};

} /* namespace label */

#endif /* PRIMITIVE_LABEL_H_ */
