/*
 * UnboundedRegExpEmpty.h
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#ifndef UNBOUNDED_REG_EXP_EMPTY_H_
#define UNBOUNDED_REG_EXP_EMPTY_H_

#include <sstream>

#include "UnboundedRegExpElement.h"

namespace regexp {

/**
 * Represents empty regular expression in the regular expression.
 */
template < class SymbolType >
class UnboundedRegExpEmpty : public UnboundedRegExpElement < SymbolType >, public ext::NullaryNode < std::smart_ptr < UnboundedRegExpElement < SymbolType > >, std::smart_ptr < const UnboundedRegExpElement < SymbolType > >, UnboundedRegExpSymbol < SymbolType > > {
public:
	void accept ( typename UnboundedRegExpElement < SymbolType >::Visitor & visitor ) const override {
		visitor.visit ( * this );
	}

	explicit UnboundedRegExpEmpty ( );

	/**
	 * @copydoc UnboundedRegExpElement::clone() const
	 */
	UnboundedRegExpEmpty < SymbolType > * clone ( ) const override;

	/**
	 * @copydoc UnboundedRegExpElement::plunder() const
	 */
	UnboundedRegExpEmpty < SymbolType > * plunder ( ) && override;

	/**
	 * @copydoc UnboundedRegExpElement::cloneAsFormal() const
	 */
	std::smart_ptr < FormalRegExpElement < SymbolType > > asFormal ( ) const override;

	/**
	 * @copydoc UnboundedRegExpElement::testSymbol() const
	 */
	bool testSymbol ( const SymbolType & symbol ) const override;

	/**
	 * @copydoc RegExpElement::computeMinimalAlphabet()
	 */
	void computeMinimalAlphabet ( std::set < SymbolType > & alphabet ) const override;

	/**
	 * @copydoc UnboundedRegExpElement::checkAlphabet()
	 */
	bool checkAlphabet ( const std::set < SymbolType > & alphabet ) const override;

	int compare ( const UnboundedRegExpElement < SymbolType > & other ) const override {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	int compare ( const UnboundedRegExpEmpty & ) const;

	/**
	 * @copydoc UnboundedRegExpElement::operator>>() const
	 */
	void operator >>( std::ostream & out ) const override;

	explicit operator std::string ( ) const override;

	virtual std::smart_ptr < UnboundedRegExpElement < DefaultSymbolType > > normalize ( ) && override {
		return std::smart_ptr < UnboundedRegExpElement < DefaultSymbolType > > ( new UnboundedRegExpEmpty < DefaultSymbolType > ( ) );
	}
};

} /* namespace regexp */

#include "../formal/FormalRegExpEmpty.h"

namespace regexp {

template < class SymbolType >
UnboundedRegExpEmpty < SymbolType >::UnboundedRegExpEmpty ( ) {
	// so that default constructor is available
}

template < class SymbolType >
UnboundedRegExpEmpty < SymbolType > * UnboundedRegExpEmpty < SymbolType >::clone ( ) const {
	return new UnboundedRegExpEmpty ( * this );
}

template < class SymbolType >
UnboundedRegExpEmpty < SymbolType > * UnboundedRegExpEmpty < SymbolType >::plunder ( ) && {
	return new UnboundedRegExpEmpty ( std::move ( * this ) );
}

template < class SymbolType >
std::smart_ptr < FormalRegExpElement < SymbolType > > UnboundedRegExpEmpty < SymbolType >::asFormal ( ) const {
	return std::smart_ptr < FormalRegExpElement < SymbolType > > ( new FormalRegExpEmpty < SymbolType > ( ) );
}

template < class SymbolType >
int UnboundedRegExpEmpty < SymbolType >::compare ( const UnboundedRegExpEmpty & ) const {
	return 0;
}

template < class SymbolType >
void UnboundedRegExpEmpty < SymbolType >::operator >>( std::ostream & out ) const {
	out << "(UnboundedRegExpEmpty)";
}

template < class SymbolType >
bool UnboundedRegExpEmpty < SymbolType >::testSymbol ( const SymbolType & ) const {
	return false;
}

template < class SymbolType >
bool UnboundedRegExpEmpty < SymbolType >::checkAlphabet ( const std::set < SymbolType > & ) const {
	return true;
}

template < class SymbolType >
void UnboundedRegExpEmpty < SymbolType >::computeMinimalAlphabet ( std::set < SymbolType > & ) const {
}

template < class SymbolType >
UnboundedRegExpEmpty < SymbolType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace regexp */

#endif /* UNBOUNDED_REG_EXP_EMPTY_H_ */
