/*
 * UnboundedRegExpEpsilon.h
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#ifndef UNBOUNDED_REG_EXP_EPSILON_H_
#define UNBOUNDED_REG_EXP_EPSILON_H_

#include <sstream>

#include "UnboundedRegExpElement.h"

namespace regexp {

/**
 * Represents epsilon in the regular expression.
 */
template < class SymbolType >
class UnboundedRegExpEpsilon : public UnboundedRegExpElement < SymbolType >, public ext::NullaryNode < std::smart_ptr < UnboundedRegExpElement < SymbolType > >, std::smart_ptr < const UnboundedRegExpElement < SymbolType > >, UnboundedRegExpSymbol < SymbolType > > {
public:
	void accept ( typename UnboundedRegExpElement < SymbolType >::Visitor & visitor ) const override {
		visitor.visit ( * this );
	}

	explicit UnboundedRegExpEpsilon ( );

	/**
	 * @copydoc UnboundedRegExpElement::clone() const
	 */
	UnboundedRegExpEpsilon < SymbolType > * clone ( ) const override;

	/**
	 * @copydoc UnboundedRegExpElement::plunder() const
	 */
	UnboundedRegExpEpsilon < SymbolType > * plunder ( ) && override;

	/**
	 * @copydoc UnboundedRegExpElement::cloneAsFormal() const
	 */
	std::smart_ptr < FormalRegExpElement < SymbolType > > asFormal ( ) const override;

	/**
	 * @copydoc UnboundedRegExpElement::testSymbol() const
	 */
	bool testSymbol ( const SymbolType & symbol ) const override;

	/**
	 * @copydoc UnboundedRegExpElement::computeMinimalAlphabet()
	 */
	void computeMinimalAlphabet ( std::set < SymbolType > & alphabet ) const override;

	/**
	 * @copydoc UnboundedRegExpElement::checkAlphabet()
	 */
	bool checkAlphabet ( const std::set < SymbolType > & alphabet ) const override;

	int compare ( const UnboundedRegExpElement < SymbolType > & other ) const override {
		if ( std::type_index ( typeid ( * this ) ) == std::type_index ( typeid ( other ) ) ) return this->compare ( ( decltype ( * this ) )other );

		return std::type_index ( typeid ( * this ) ) - std::type_index ( typeid ( other ) );
	}

	int compare ( const UnboundedRegExpEpsilon & ) const;

	/**
	 * @copydoc UnboundedRegExpElement::operator>>() const
	 */
	void operator >>( std::ostream & out ) const override;

	explicit operator std::string ( ) const override;

	virtual std::smart_ptr < UnboundedRegExpElement < DefaultSymbolType > > normalize ( ) && override {
		return std::smart_ptr < UnboundedRegExpElement < DefaultSymbolType > > ( new UnboundedRegExpEpsilon < DefaultSymbolType > ( ) );
	}
};

} /* namespace regexp */

#include "../formal/FormalRegExpEpsilon.h"

namespace regexp {

template < class SymbolType >
UnboundedRegExpEpsilon < SymbolType >::UnboundedRegExpEpsilon ( ) {
	// so that default constructor is available
}

template < class SymbolType >
UnboundedRegExpEpsilon < SymbolType > * UnboundedRegExpEpsilon < SymbolType >::clone ( ) const {
	return new UnboundedRegExpEpsilon ( * this );
}

template < class SymbolType >
UnboundedRegExpEpsilon < SymbolType > * UnboundedRegExpEpsilon < SymbolType >::plunder ( ) && {
	return new UnboundedRegExpEpsilon ( std::move ( * this ) );
}

template < class SymbolType >
std::smart_ptr < FormalRegExpElement < SymbolType > > UnboundedRegExpEpsilon < SymbolType >::asFormal ( ) const {
	return std::smart_ptr < FormalRegExpElement < SymbolType > > ( new FormalRegExpEpsilon < SymbolType > ( ) );
}

template < class SymbolType >
int UnboundedRegExpEpsilon < SymbolType >::compare ( const UnboundedRegExpEpsilon & ) const {
	return 0;
}

template < class SymbolType >
void UnboundedRegExpEpsilon < SymbolType >::operator >>( std::ostream & out ) const {
	out << "(UnboundedRegExpEpsilon)";
}

template < class SymbolType >
bool UnboundedRegExpEpsilon < SymbolType >::testSymbol ( const SymbolType & ) const {
	return false;
}

template < class SymbolType >
bool UnboundedRegExpEpsilon < SymbolType >::checkAlphabet ( const std::set < SymbolType > & ) const {
	return true;
}

template < class SymbolType >
void UnboundedRegExpEpsilon < SymbolType >::computeMinimalAlphabet ( std::set < SymbolType > & ) const {
}

template < class SymbolType >
UnboundedRegExpEpsilon < SymbolType >::operator std::string ( ) const {
	std::stringstream ss;
	ss << * this;
	return ss.str ( );
}

} /* namespace regexp */

#endif /* UNBOUNDED_REG_EXP_EPSILON_H_ */
