/*
 * TreeToXMLComposer.h
 *
 *  Created on: Nov 16, 2014
 *      Author: Stepan Plachy
 */

#ifndef TREE_TO_XML_COMPOSER_H_
#define TREE_TO_XML_COMPOSER_H_

#include <deque>
#include <set>
#include <vector>
#include <tree>
#include <sax/Token.h>
#include <core/xmlApi.hpp>

namespace tree {

/**
 * This class contains methods to print XML representation of tree to the output stream.
 */
class TreeToXMLComposer {
public:
	template < class SymbolType >
	static void composeAlphabet ( std::deque < sax::Token > & out, const std::set < SymbolType > & symbols );

	template < class SymbolType >
	static void composeBar ( std::deque < sax::Token > & out, const SymbolType & symbol );
	template < class SymbolType >
	static void composeBars ( std::deque < sax::Token > & out, const std::set < SymbolType > & symbols );
	template < class SymbolType >
	static void composeVariablesBar ( std::deque < sax::Token > & out, const SymbolType & symbol );

	template < class SymbolType >
	static void composeSubtreeWildcard ( std::deque < sax::Token > & out, const SymbolType & symbol );

	template < class SymbolType >
	static void composeContent ( std::deque < sax::Token > & out, const std::vector < SymbolType > & content );
	template < class SymbolType >
	static void composeContent ( std::deque < sax::Token > & out, const ext::tree < SymbolType > & content );

	template < class SymbolType >
	static void composeNonlinearVariables ( std::deque < sax::Token > & out, const std::set < SymbolType > & symbols );
};

template < class SymbolType >
void TreeToXMLComposer::composeBar ( std::deque < sax::Token > & out, const SymbolType & symbol ) {
	out.emplace_back ( sax::Token ( "bar", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < SymbolType >::compose ( out, symbol );
	out.emplace_back ( sax::Token ( "bar", sax::Token::TokenType::END_ELEMENT ) );
}

template < class SymbolType >
void TreeToXMLComposer::composeBars ( std::deque < sax::Token > & out, const std::set < SymbolType > & symbols ) {
	out.emplace_back ( sax::Token ( "bars", sax::Token::TokenType::START_ELEMENT ) );

	for ( const auto & symbol : symbols )
		alib::xmlApi < SymbolType >::compose ( out, symbol );

	out.emplace_back ( sax::Token ( "bars", sax::Token::TokenType::END_ELEMENT ) );
}

template < class SymbolType >
void TreeToXMLComposer::composeVariablesBar ( std::deque < sax::Token > & out, const SymbolType & symbol ) {
	out.emplace_back ( sax::Token ( "variablesBar", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < SymbolType >::compose ( out, symbol );
	out.emplace_back ( sax::Token ( "variablesBar", sax::Token::TokenType::END_ELEMENT ) );
}

template < class SymbolType >
void TreeToXMLComposer::composeSubtreeWildcard ( std::deque < sax::Token > & out, const SymbolType & symbol ) {
	out.emplace_back ( sax::Token ( "subtreeWildcard", sax::Token::TokenType::START_ELEMENT ) );
	alib::xmlApi < SymbolType >::compose ( out, symbol );
	out.emplace_back ( sax::Token ( "subtreeWildcard", sax::Token::TokenType::END_ELEMENT ) );
}

template < class SymbolType >
void TreeToXMLComposer::composeAlphabet ( std::deque < sax::Token > & out, const std::set < SymbolType > & symbols ) {
	out.emplace_back ( sax::Token ( "alphabet", sax::Token::TokenType::START_ELEMENT ) );

	for ( const auto & symbol : symbols )
		alib::xmlApi < SymbolType >::compose ( out, symbol );

	out.emplace_back ( sax::Token ( "alphabet", sax::Token::TokenType::END_ELEMENT ) );
}

template < class SymbolType >
void TreeToXMLComposer::composeContent ( std::deque < sax::Token > & out, const std::vector < SymbolType > & content ) {
	out.emplace_back ( "content", sax::Token::TokenType::START_ELEMENT );

	for ( const auto & symbol : content )
		alib::xmlApi < SymbolType >::compose ( out, symbol );

	out.emplace_back ( "content", sax::Token::TokenType::END_ELEMENT );
}

template < class SymbolType >
void TreeToXMLComposer::composeContent ( std::deque < sax::Token > & out, const ext::tree < SymbolType > & content ) {
	out.emplace_back ( "content", sax::Token::TokenType::START_ELEMENT );

	unsigned level = 0;

	for ( typename ext::tree < SymbolType >::const_prefix_iterator iter = content.prefix_begin ( ); iter != content.prefix_end ( ); ) {
		while ( iter.getLevel ( ) > level ) {
			out.emplace_back ( "Children", sax::Token::TokenType::START_ELEMENT );
			++level;
		}

		alib::xmlApi < SymbolType >::compose ( out, * iter );
		++iter;

		while ( iter.getLevel ( ) < level ) {
			out.emplace_back ( "Children", sax::Token::TokenType::END_ELEMENT );
			--level;
		}
	}

	out.emplace_back ( "content", sax::Token::TokenType::END_ELEMENT );
}

template < class SymbolType >
void TreeToXMLComposer::composeNonlinearVariables ( std::deque < sax::Token > & out, const std::set < SymbolType > & symbols ) {
	out.emplace_back ( sax::Token ( "nonlinearVariables", sax::Token::TokenType::START_ELEMENT ) );

	for ( const auto & symbol : symbols )
		alib::xmlApi < SymbolType >::compose ( out, symbol );

	out.emplace_back ( sax::Token ( "nonlinearVariables", sax::Token::TokenType::END_ELEMENT ) );
}

} /* namespace tree */

#endif /* TREE_TO_XML_COMPOSER_H_ */
