/*
 * Author: Radovan Cerveny
 */

#include "MeasurementProcessorViews.hpp"

namespace measurements {

std::string MeasurementProcessorViews::viewTimeDataFrame ( const MeasurementFrame & frame, const std::set < std::string > & outputEngineAttr ) {
	 // make this default behavior
	 // if(outputEngineAttr.count(TIME_DURATION_ATTR))
	if ( outputEngineAttr.count ( INFRAME_ATTR ) )
		return ext::to_string ( frame.time.inFrameDuration.count ( ) );
	else
		return ext::to_string ( frame.time.duration.count ( ) );
}

std::string MeasurementProcessorViews::viewMemoryDataFrame ( const MeasurementFrame & frame, const std::set < std::string > & outputEngineAttr ) {
	if ( outputEngineAttr.count ( MEMORY_SHU_ATTR ) )
		return ext::to_string ( frame.memory.startHeapUsage );

	if ( outputEngineAttr.count ( MEMORY_EHU_ATTR ) )
		return ext::to_string ( frame.memory.endHeapUsage );

	if ( outputEngineAttr.count ( MEMORY_HW_ATTR ) ) {
		if ( outputEngineAttr.count ( INFRAME_ATTR ) )
			return ext::to_string ( frame.memory.inFrameHighWatermark );
		else
			return ext::to_string ( frame.memory.highWatermark );
	}

	 // make this default behavior
	 // if(outputEngineAttr.count(MEMORY_DELTA_ATTR))
	if ( outputEngineAttr.count ( INFRAME_ATTR ) )
		return ext::to_string ( frame.memory.inFrameHighWatermark - frame.memory.startHeapUsage );
	else
		return ext::to_string ( frame.memory.highWatermark - frame.memory.startHeapUsage );
}

std::string MeasurementProcessorViews::viewCounterDataFrame ( const MeasurementFrame & frame, const std::set < std::string > & outputEngineAttr ) {
	if ( outputEngineAttr.count ( INFRAME_ATTR ) ) {
		for ( const std::string & attr : outputEngineAttr ) {
			if ( attr == INFRAME_ATTR ) continue;

			auto it = frame.counter.counters.find ( measurements::stealthStringFromString ( attr ) );

			if ( it != frame.counter.counters.end ( ) )
				return ext::to_string ( it->second );
		}
	} else {
		auto it = frame.counter.inFrameCounters.find ( measurements::stealthStringFromString ( * outputEngineAttr.begin ( ) ) );

		if ( it != frame.counter.inFrameCounters.end ( ) )
			return ext::to_string ( it->second );
	}

	 // we default to default constructed value_type
	return ext::to_string ( measurements::CounterHint::value_type { } );
}

std::string MeasurementProcessorViews::INFRAME_ATTR = "inFrame";

std::string MeasurementProcessorViews::TIME_DURATION_ATTR = "duration";

std::string MeasurementProcessorViews::MEMORY_DELTA_ATTR = "delta";
std::string MeasurementProcessorViews::MEMORY_SHU_ATTR = "startHeapUsage";
std::string MeasurementProcessorViews::MEMORY_EHU_ATTR = "endHeapUsage";
std::string MeasurementProcessorViews::MEMORY_HW_ATTR  = "highWatermark";
}
