/*
 * deque.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef __DEQUE_HPP_
#define __DEQUE_HPP_

#include <deque>
#include <ostream>
#include <string>
#include <sstream>

#include "compare.hpp"

namespace std {

template < class T, class ... Ts >
std::ostream & operator <<( std::ostream & out, const std::deque < T, Ts ... > & list ) {
	out << "[";

	bool first = true;

	for ( const T & item : list ) {
		if ( !first ) out << ", ";

		first = false;
		out << item;
	}

	out << "]";
	return out;
}

} /* namespace std */

namespace ext {

template < class T, class ... Ts >
struct compare < std::deque < T, Ts ... > > {
	int operator ()( const std::deque < T, Ts ... > & first, const std::deque < T, Ts ... > & second ) const {
		if ( first.size ( ) < second.size ( ) ) return -1;

		if ( first.size ( ) > second.size ( ) ) return 1;

		static compare < typename std::decay < T >::type > comp;

		for ( auto iterF = first.begin ( ), iterS = second.begin ( ); iterF != first.end ( ); ++iterF, ++iterS ) {
			int res = comp ( * iterF, * iterS );

			if ( res != 0 ) return res;
		}

		return 0;
	}

};

template < class T, class ... Ts >
std::string to_string ( const std::deque < T, Ts ... > & value ) {
	std::stringstream ss;
	ss << value;
	return ss.str();
}

} /* namespace ext */

#endif /* __DEQUE_HPP_ */
