/*
 * pair.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef __PAIR_HPP_
#define __PAIR_HPP_

#include <string>
#include <sstream>

#include "type_traits.hpp"
#include "utility.hpp"
#include "compare.hpp"

namespace std {

template< class T, class R >
std::ostream& operator<<(std::ostream& out, const std::pair<T, R>& pair) {
	out << "(" << pair.first << ", " << pair.second << ")";
	return out;
}

} /* namespace std */

namespace ext {

template < class T, class R >
struct compare < std::pair < T, R > > {
	int operator ( ) ( const std::pair < T, R > & first, const std::pair < T, R > & second ) const {
		static compare < typename std::decay < T >::type > compT;
		static compare < typename std::decay < R >::type > compR;

		int res = compT(first.first, second.first);
		if(res == 0) res = compR(first.second, second.second);
		return res;
	}
};

template < class T, class R >
std::string to_string ( const std::pair < T, R > & value ) {
	std::stringstream ss;
	ss << value;
	return ss.str();
}

} /* namespace ext */

#endif /* __PAIR_HPP_ */
