/*
 * tuple.hpp
 *
 * Created on: Apr 1, 2013
 * Author: Jan Travnicek
 */

#ifndef __TUPLE_HPP_
#define __TUPLE_HPP_

#include <tuple>
#include <ostream>
#include <string>
#include <sstream>

#include "compare.hpp"

namespace std {

template < class Result, unsigned I, class Tuple, class Callable >
struct call_on_nth_helper;

template < class Result, unsigned I, class Tuple, class Callable >
struct call_on_nth_helper {
	static Result call_on_nth_fn ( Tuple & t, unsigned index, Callable callback ) {
		if ( index == 0 )
			return callback ( std::get < std::tuple_size < Tuple >::value - I > ( t ) );
		else
			return call_on_nth_helper < Result, I - 1, Tuple, Callable >::call_on_nth_fn ( t, index - 1, callback );
	}
};

template < class Result, class Tuple, class Callable >
struct call_on_nth_helper < Result, 0, Tuple, Callable > {
	static Result call_on_nth_fn ( Tuple &, unsigned, Callable ) {
		throw std::out_of_range ( "Not enough elements in tuple." );
	}
};

template < class Result, class Tuple, class Callable >
Result call_on_nth ( Tuple & t, unsigned index, Callable callback ) {
	return call_on_nth_helper < Result, std::tuple_size < Tuple >::value, Tuple, Callable >::call_on_nth_fn ( t, index, callback );
}



template < unsigned I, class Tuple, class Callable >
struct foreach_helper;

template < unsigned I, class Tuple, class Callable >
struct foreach_helper {
	static void foreach_fn ( Tuple & t, Callable callback ) {
		callback ( std::get < I - 1 > ( t ) );
		foreach_helper < I - 1, Tuple, Callable >::foreach_fn ( t, callback );
	}
};

template < class Tuple, class Callable >
struct foreach_helper < 0, Tuple, Callable > {
	static void foreach_fn ( Tuple &, Callable ) {
	}
};

template < class Tuple, class Callable >
void foreach ( Tuple & t, Callable callback ) {
	return foreach_helper < std::tuple_size < Tuple >::value, Tuple, Callable >::foreach_fn ( t, callback );
}



template < class Tuple, class Callable >
bool all_of ( const Tuple & t, Callable callback ) {
	bool res = true;

	auto aggregateCallback = [ & ] ( auto & arg0 ) {
		res &= callback ( arg0 );
	};

	std::foreach ( t, aggregateCallback  );
	return res;
}



template<int I, class Tuple>
struct operator_shift_left_impl;

template<int I, class Tuple>
struct operator_shift_left_impl {
	static void operator_shift_left ( std::ostream& out, const Tuple& t) {
		operator_shift_left_impl < I - 1, Tuple>::operator_shift_left ( out, t );
		out << ", " << std::get < I > ( t );
	}
};

template<class Tuple>
struct operator_shift_left_impl<0, Tuple> {
	static void operator_shift_left ( std::ostream& out, const Tuple& t) {
		out << std::get < 0 > ( t );
	}
};

template< class... Ts>
std::ostream& operator<<(std::ostream& out, const std::tuple<Ts...>& tuple) {
	out << "(";
	std::operator_shift_left_impl < std::tuple_size < std::tuple < Ts ... > >::value - 1, std::tuple < Ts ... > >::operator_shift_left ( out, tuple );
	out << ")";
	return out;
}

} /* namespace std */

namespace ext {

template < int I, typename Tuple >
struct compareTupleHelper;

template < int I, typename Tuple >
struct compareTupleHelper {
	static int compHelp ( const Tuple & t1, const Tuple & t2 ) {
		int res = compareTupleHelper < I - 1, Tuple >::compHelp ( t1, t2 );

		if ( res != 0 ) return res;

		static compare < typename std::decay < typename std::tuple_element < I, Tuple >::type >::type > comp;
		return comp ( std::get < I > ( t1 ), std::get < I > ( t2 ) );
	}

};

template < class Tuple >
struct compareTupleHelper < 0, Tuple > {
	static int compHelp ( const Tuple & t1, const Tuple & t2 ) {
		static compare < typename std::decay < typename std::tuple_element < 0, Tuple >::type >::type > comp;
		return comp ( std::get < 0 > ( t1 ), std::get < 0 > ( t2 ) );
	}

};

template < typename ... Ts >
struct compare < std::tuple < Ts ... > > {
	int operator ()( const std::tuple < Ts ... > & first, const std::tuple < Ts ... > & second ) const {
		return compareTupleHelper < std::tuple_size < std::tuple < Ts ... > >::value - 1, std::tuple < Ts ... > >::compHelp ( first, second );
	}

};

template < class ... Ts >
std::string to_string ( const std::tuple < Ts ... > & value ) {
	std::stringstream ss;
	ss << value;
	return ss.str();
}

} /* namespace ext */

namespace std {

template < class Type, int size, class ... Types >
struct TupleBuilder;

template < class Type, class ... Types >
struct TupleBuilder < Type, 0, Types ... > {
	typedef std::tuple < Types ... > type;
};

template < class Type, int n, class ... Types >
struct TupleBuilder : public TupleBuilder < Type, n - 1, Type, Types ... > {
};

} /* namespace std */

#endif /* __TUPLE_HPP_ */
