#include "VectorTest.h"
#include <vector>
#include <type_traits>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( VectorTest, "bits" );
CPPUNIT_TEST_SUITE_REGISTRATION( VectorTest );

void VectorTest::setUp() {
}

void VectorTest::tearDown() {
}

void VectorTest::testProperties() {
	std::vector<int> data = {1, 2, 3, 4};

	const std::vector<int>& constData = data;

	CPPUNIT_ASSERT((std::is_same<decltype(constData[0]), const int&>::value));
}

void VectorTest::testVectorBool() {
	std::vector < bool > A;
	A.push_back ( true );
	A.push_back ( false );
	A.push_back ( true );
	A.push_back ( false );

	std::vector < bool > B;
	B.push_back ( true );
	B.push_back ( true );
	B.push_back ( false );
	B.push_back ( false );

	{
		std::vector < bool > C = A | B;
		std::vector < bool > D = A;
		D |= B;

		std::vector < bool > X;
		X.push_back ( true );
		X.push_back ( true );
		X.push_back ( true );
		X.push_back ( false );

		std::cout << X << std::endl;
		std::cout << C << std::endl;

		CPPUNIT_ASSERT ( C == X );
		CPPUNIT_ASSERT ( D == X );
	}

	{
		std::vector < bool > C = A & B;
		std::vector < bool > D = A;
		D &= B;

		std::vector < bool > X;
		X.push_back ( true );
		X.push_back ( false );
		X.push_back ( false );
		X.push_back ( false );

		CPPUNIT_ASSERT ( C == X );
		CPPUNIT_ASSERT ( D == X );
	}

	{
		std::vector < bool > C = A ^ B;
		std::vector < bool > D = A;
		D ^= B;

		std::vector < bool > X;
		X.push_back ( false );
		X.push_back ( true );
		X.push_back ( true );
		X.push_back ( false );

		CPPUNIT_ASSERT ( C == X );
		CPPUNIT_ASSERT ( D == X );
	}
}

void testOffset ( long offset, long shift ) {
	unsigned long long shadow = 0x2A76B147D6521C87ULL;
	std::vector < bool > data;

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 16 + offset; ++ i ) {
		data.push_back ( true );
	}

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	data >>= sizeof ( unsigned long long ) * 6 + shift;

	std::vector < bool > ref;
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 10 + offset - shift; ++ i ) {
		ref.push_back ( true );
	}

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		ref.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	ref.resize ( 192 + offset );

/*	std::cout << "offset = " << offset << std::endl;
	std::cout << "shift = " << shift << std::endl;
	std::cout << "data = " << data << std::endl;
	std::cout << "ref  = " << ref << std::endl;*/

	CPPUNIT_ASSERT ( data == ref );
}

void VectorTest::testVectorBool2() {
	unsigned long long shadow = 0x2A76B147D6521C87ULL;
	std::vector < bool > data;

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	shadow >>= 10;
	data >>= 10;

	std::vector < bool > ref;
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		ref.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	std::cout << "data = " << data << std::endl;
	std::cout << "ref  = " << ref << std::endl;

	CPPUNIT_ASSERT ( data == ref );

	for ( long i = - ( long ) sizeof ( unsigned long long ) / 2 ; i < ( long ) sizeof ( unsigned long long ) * 8; ++ i )
		for ( long j = - ( long ) sizeof ( unsigned long long ) / 2 ; j < ( long ) sizeof ( unsigned long long ) * 8; ++ j )
			testOffset ( i, j );
}

void VectorTest::testVectorBool3() {
	unsigned long long shadow = 0x2A76B147D6521C87ULL;
	std::vector < bool > data;

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	std::vector < bool > data2 = data;
	data2 >>= 64;

	data >>= 32;
	data >>= 16;
	data >>= 8;
	data >>= 4;
	data >>= 2;
	data >>= 1;
	data >>= 1;

	std::cout << "data  = " << data  << std::endl;
	std::cout << "data2 = " << data2 << std::endl;

	data2 >>= 1;
	data2 >>= 2;
	data2 >>= 4;
	data2 >>= 8;
	data2 >>= 16;
	data2 >>= 32;
	data2 >>= 1;

	data >>= 27;
	data >>= 17;
	data >>= 13;
	data >>= 7;

	std::cout << "data  = " << data  << std::endl;
	std::cout << "data2 = " << data2 << std::endl;

	CPPUNIT_ASSERT ( data2 == data );
}

void testOffset2 ( long offset, long shift ) {
	unsigned long long shadow = 0x2A76B147D6521C87ULL;
	std::vector < bool > data;

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8 + offset; ++ i ) {
		data.push_back ( true );
	}

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	data <<= sizeof ( unsigned long long ) * 2 + shift;

	std::vector < bool > ref;
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 2 + shift; ++ i ) {
		ref.push_back ( false );
	}

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8 + offset; ++ i ) {
		ref.push_back ( true );
	}

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		ref.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	ref.resize ( 128 + offset );

/*	std::cout << "offset = " << offset << std::endl;
	std::cout << "shift = " << shift << std::endl;
	std::cout << "data = " << data << std::endl;
	std::cout << "ref  = " << ref << std::endl;*/

	CPPUNIT_ASSERT ( data == ref );
}

void VectorTest::testVectorBool4() {
	unsigned long long shadow = 0x2A76B147D6521C87ULL;
	std::vector < bool > data;

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	shadow <<= 10;
	data <<= 10;

	std::vector < bool > ref;
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		ref.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	std::cout << "data = " << data << std::endl;
	std::cout << "ref  = " << ref << std::endl;

	CPPUNIT_ASSERT ( data == ref );

	for ( long i = - ( long ) sizeof ( unsigned long long ) / 2 ; i < ( long ) sizeof ( unsigned long long ) * 8; ++ i )
		for ( long j = - ( long ) sizeof ( unsigned long long ) / 2 ; j < ( long ) sizeof ( unsigned long long ) * 8; ++ j )
			testOffset2 ( i, j );
}

void VectorTest::testVectorBool5() {
	unsigned long long shadow = 0x2A76B147D6521C87ULL;
	std::vector < bool > data;

	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}
	for ( unsigned i = 0; i < sizeof ( unsigned long long ) * 8; ++ i ) {
		data.push_back ( ( bool ) ( shadow & ( 1ULL << i ) ) );
	}

	std::vector < bool > data2 = data;
	data2 <<= 64;

	data <<= 32;
	data <<= 16;
	data <<= 8;
	data <<= 4;
	data <<= 2;
	data <<= 1;
	data <<= 1;

	std::cout << "data  = " << data  << std::endl;
	std::cout << "data2 = " << data2 << std::endl;

	data2 <<= 1;
	data2 <<= 2;
	data2 <<= 4;
	data2 <<= 8;
	data2 <<= 16;
	data2 <<= 32;
	data2 <<= 1;

	data <<= 27;
	data <<= 17;
	data <<= 13;
	data <<= 7;

	std::cout << "data  = " << data  << std::endl;
	std::cout << "data2 = " << data2 << std::endl;

	CPPUNIT_ASSERT ( data2 == data );
}

