/*
 * ObjectFromStringLexer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "ObjectFromStringLexer.h"

namespace alib {

ObjectFromStringLexer::Token ObjectFromStringLexer::next(std::istream& in) {
	ObjectFromStringLexer::Token token;
	token.type = TokenType::ERROR;
	token.value = "";
	token.raw = "";
	char character;

L0:
	character = in.get();
	if(in.eof()) {
		token.type = TokenType::TEOF;
		return token;
	} else if(character == ' ' || character == '\n' || character == '\t') {
		token.raw += character;
		goto L0;
	} else if(character == '[') {
		token.type = TokenType::SET_BEGIN;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == ']') {
		token.type = TokenType::SET_END;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '<') {
		token.type = TokenType::PAIR_BEGIN;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == '>') {
		token.type = TokenType::PAIR_END;
		token.value += character;
		token.raw += character;
		return token;
	} else if(character == ',') {
		token.type = TokenType::COMMA;
		token.value += character;
		token.raw += character;
		return token;
	} else if((character >= 'a' && character <= 'z') || (character >= 'A' && character <= 'Z')) {
		token.type = TokenType::STRING;
		token.value += character;
		token.raw += character;
		goto L1;
	} else if(character >= '0' && character <= '9') {
		token.type = TokenType::INTEGER;
		token.value += character;
		token.raw += character;
		goto L2;
	} else {
		in.putback(character);
		putback(in, std::move(token));
		token.type = TokenType::ERROR;
		return token;
	}
L1:
	character = in.get();
	if(in.eof()) {
		return token;
	} else if((character >= 'a' && character <= 'z') || (character >= 'A' && character <= 'Z') ) {
		token.value += character;
		token.raw += character;
		goto L1;
	} else {
		in.unget();
		return token;
	}
L2:
	character = in.get();
	if(in.eof()) {
		return token;
	} else if(character >= '0' && character <= '9') {
		token.value += character;
		token.raw += character;
		goto L1;
	} else {
		in.unget();
		return token;
	}
}

void ObjectFromStringLexer::putback(std::istream& in, ObjectFromStringLexer::Token token) {
	while(!token.raw.empty()) {
		in.putback(token.raw.back());
		token.raw.pop_back();
	}
	in.clear();
}

} /* namespace alib */
