/*
 * ObjectFromStringParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include <exception/CommonException.h>

#include <primitive/String.h>
#include <primitive/Integer.h>

#include <container/ObjectsSet.h>
#include <container/ObjectsPair.h>

#include "ObjectFromStringParser.h"

namespace alib {

Object ObjectFromStringParser::parseObject(std::istream& input) {
	ObjectFromStringLexer::Token token = ObjectFromStringLexer::next(input);
	switch(token.type) {
	case ObjectFromStringLexer::TokenType::TEOF:
		throw exception::CommonException("Unexpected end of file");
	case ObjectFromStringLexer::TokenType::SET_BEGIN: {
		token = ObjectFromStringLexer::next(input);

		std::set<Object> objectsSet;
		if(token.type != ObjectFromStringLexer::TokenType::SET_END) while(true) {
			ObjectFromStringLexer::putback(input, std::move(token));
			Object innerObject = parseObject(input);
			objectsSet.insert(innerObject);

			token = ObjectFromStringLexer::next(input);
			if(token.type != ObjectFromStringLexer::TokenType::COMMA) 
				break;

			token = ObjectFromStringLexer::next(input);
		}

		if(token.type != ObjectFromStringLexer::TokenType::SET_END)
			throw exception::CommonException("Expected SET_END token.");
		return Object(container::ObjectsSet < > (objectsSet));
	}
	case ObjectFromStringLexer::TokenType::PAIR_BEGIN: {
		Object firstObject = parseObject(input);

		token = ObjectFromStringLexer::next(input);
		if(token.type != ObjectFromStringLexer::TokenType::COMMA)
			throw exception::CommonException("Excepted COMMA token.");

		Object secondObject = parseObject(input);

		token = ObjectFromStringLexer::next(input);
		if(token.type != ObjectFromStringLexer::TokenType::PAIR_END)
			throw exception::CommonException("Expected PAIR_END token.");

		return Object(container::ObjectsPair < > (std::make_pair(firstObject, secondObject)));
	}
	case ObjectFromStringLexer::TokenType::SET_END:
		throw exception::CommonException("Unexpected start of Object SET_END.");
	case ObjectFromStringLexer::TokenType::PAIR_END:
		throw exception::CommonException("Unexpected start of Object PAIR_END.");
	case ObjectFromStringLexer::TokenType::COMMA:
		throw exception::CommonException("Unexpected start of Object COMMA.");
	case ObjectFromStringLexer::TokenType::STRING:
		return Object(primitive::String(token.value));
	case ObjectFromStringLexer::TokenType::INTEGER:
		return Object(primitive::Integer(ext::from_string < int > (token.value)));
	case ObjectFromStringLexer::TokenType::ERROR:
		break;
	}
	throw exception::CommonException("Unrecognised input \"" + token.value + "...\"");
}

} /* namespace alib */
