/*
 * TreeToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "TreeToStringComposer.h"
#include <tree/ranked/RankedTree.h>
#include <tree/ranked/RankedPattern.h>
#include <tree/unranked/UnrankedTree.h>
#include <tree/unranked/UnrankedPattern.h>

#include <registration/AlgoRegistration.hpp>

namespace tree {

template < class SymbolType, class RankType >
void TreeToStringComposer::compose ( std::ostream & out, const RankedTree < SymbolType, RankType > & tree ) {
	compose ( out, tree.getContent ( ) );
}

auto StringToStringComposerRankedTree = registration::OverloadRegister < TreeToStringComposer, void, RankedTree < > > ( TreeToStringComposer::compose );
auto ObjectToStringComposerRankedTree = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, RankedTree < > >(TreeToStringComposer::compose);

template < class SymbolType, class RankType >
void TreeToStringComposer::compose ( std::ostream & out, const RankedPattern < SymbolType, RankType > & tree ) {
	compose ( out, tree.getSubtreeWildcard ( ), tree.getContent ( ) );
}

auto StringToStringComposerRankedPattern = registration::OverloadRegister < TreeToStringComposer, void, RankedPattern < > > ( TreeToStringComposer::compose );
auto ObjectToStringComposerRankedPattern = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, RankedPattern < > >(TreeToStringComposer::compose);

template < class SymbolType >
void TreeToStringComposer::compose ( std::ostream & out, const UnrankedTree < SymbolType > & tree ) {
	compose ( out, tree.getContent ( ) );
}

auto StringToStringComposerUnrankedTree = registration::OverloadRegister < TreeToStringComposer, void, UnrankedTree < > > ( TreeToStringComposer::compose );
auto ObjectToStringComposerUnrankedTree = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, UnrankedTree < > >(TreeToStringComposer::compose);

template < class SymbolType >
void TreeToStringComposer::compose ( std::ostream & out, const UnrankedPattern < SymbolType > & tree ) {
	compose ( out, tree.getSubtreeWildcard ( ), tree.getContent ( ) );
}

auto StringToStringComposerUnrankedPattern = registration::OverloadRegister < TreeToStringComposer, void, UnrankedPattern < > > ( TreeToStringComposer::compose );
auto ObjectToStringComposerUnrankedPattern = registration::OverloadRegister < alib::ObjectBaseDispatcher, void, UnrankedPattern < > >(TreeToStringComposer::compose);

void TreeToStringComposer::compose ( std::ostream & out, const Tree & tree ) {
	dispatch ( out, tree.getData ( ) );
}

auto ObjectToStringComposerTree = registration::OverloadRegister < alib::WrapperBaseDispatcher, void, Tree>(TreeToStringComposer::compose);

} /* namespace tree */
