#include "AutomatonTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "automaton/FSM/DFA.h"
#include "automaton/FSM/ExtendedNFA.h"
#include "automaton/PDA/SinglePopDPDA.h"
#include "automaton/PDA/DPDA.h"
#include "automaton/PDA/NPDA.h"
#include "automaton/PDA/RealTimeHeightDeterministicNPDA.h"
#include "automaton/TA/NFTA.h"
#include <automaton/Automaton.h>

#include "automaton/AutomatonException.h"

#include "factory/XmlDataFactory.hpp"
#include "factory/StringDataFactory.hpp"

#include "alphabet/LabeledSymbol.h"
#include "alphabet/BottomOfTheStackSymbol.h"

#include "regexp/RegExp.h"
#include "regexp/unbounded/UnboundedRegExp.h"
#include "regexp/unbounded/UnboundedRegExpElements.h"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( AutomatonTest, "automaton" );
CPPUNIT_TEST_SUITE_REGISTRATION( AutomatonTest );

void AutomatonTest::setUp() {
}

void AutomatonTest::tearDown() {
}

void AutomatonTest::FSMStringParserTest() {
	{
		std::string input = 	"ENFA a b c d #E\n"
					">0 3|4 5 1|3|4 - 2\n"
					"1 2 - - - -\n"
					"2 3 - - - -\n"
					"3 - - 4 - -\n"
					"4 - 5 - - 5\n"
					"<5 - - - - 3\n";
		automaton::Automaton automaton = alib::StringDataFactory::fromString (input);

		std::string output = alib::StringDataFactory::toString(automaton);

		CPPUNIT_ASSERT( input == output );

		automaton::Automaton automaton2 = alib::StringDataFactory::fromString (output);

		CPPUNIT_ASSERT( automaton == automaton2 );
	}
	{
		std::string input = 	"MISNFA a b c d\n"
					">0 3|4 5 1|3|4 -\n"
					"1 2 - - -\n"
					"2 3 - - -\n"
					">3 - - 4 -\n"
					"4 - 5 - -\n"
					"<5 - - - 3\n";
		automaton::Automaton automaton = alib::StringDataFactory::fromString (input);

		std::string output = alib::StringDataFactory::toString(automaton);

		CPPUNIT_ASSERT( input == output );

		automaton::Automaton automaton2 = alib::StringDataFactory::fromString (output);

		CPPUNIT_ASSERT( automaton == automaton2 );
	}
	{
		std::string input = 	"NFA a b c d\n"
					">0 3|4 5 1|3|4 -\n"
					"1 2 - - -\n"
					"2 3 - - -\n"
					"3 - - 4 -\n"
					"4 - 5 - -\n"
					"<5 - - - 3\n";
		automaton::Automaton automaton = alib::StringDataFactory::fromString (input);

		std::string output = alib::StringDataFactory::toString(automaton);

		CPPUNIT_ASSERT( input == output );

		automaton::Automaton automaton2 = alib::StringDataFactory::fromString (output);

		CPPUNIT_ASSERT( automaton == automaton2 );
	}
	{
		std::string input = 	"DFA a b c d\n"
					">0 3 5 1 -\n"
					"1 2 - - -\n"
					"2 3 - - -\n"
					"3 - - 4 -\n"
					"4 - 5 - -\n"
					"<5 - - - 3\n";
		automaton::Automaton automaton = alib::StringDataFactory::fromString (input);

		std::string output = alib::StringDataFactory::toString(automaton);

		CPPUNIT_ASSERT( input == output );

		automaton::Automaton automaton2 = alib::StringDataFactory::fromString (output);

		CPPUNIT_ASSERT( automaton == automaton2 );
	}
}

