/*
 * GraphFromStringParser.h
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 */

#ifndef GRAPH_FROM_STRING_PARSER_H_
#define GRAPH_FROM_STRING_PARSER_H_

#include "graph/GraphClasses.h"
#include "GraphFromStringLexer.h"

namespace graph {

class GraphFromStringParser
{
public:
	static Graph parseGraph(std::istream &input);
	template<typename GraphType>
	static GraphType parseDirectedGraph(std::istream &input);
	template<typename GraphType>
	static GraphType parseUndirectedGraph(std::istream &input);

	static REPRESENTATION parseRepresentation(std::istream &input);
	static Node parseNode(std::istream &input);
	static DirectedEdge parseDirectedEdge(std::istream &input);
	static UndirectedEdge parseUndirectedEdge(std::istream &input);

	template<typename T>
	static void parseNodes(std::istream &input, T &graph);
	static void parseDirectedEdges(std::istream &input, DirectedGraph &graph);
	static void parseUndirectedEdges(std::istream &input, UndirectedGraph &graph);
	static void parseDelimiter(std::istream &input);
	static int parseValue(std::istream &input);

	template<typename T>
	static void parseNodeValues(std::istream &input, T &graph);
	static void parseDirectedEdgeValues(std::istream &input, DirectedGraph &graph);
	static void parseUndirectedEdgeValues(std::istream &input, UndirectedGraph &graph);
};

} // namespace graph

#endif // GRAPH_FROM_STRING_PARSER_H_
