/*
 * GraphTest.cpp
 *
 *   Created on: Jun 19, 2014
 *       Author: David Rosca
 *  Modified by: Jan Broz
 */

#include "GraphTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "factory/StringDataFactory.hpp"


CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GraphTest, "graph" );
CPPUNIT_TEST_SUITE_REGISTRATION( GraphTest );


template< typename UndirectedGraphImpl, typename DirectedGraphImpl >
void testStringParser_impl()
{
	// Common
	graph::Node n1("n1");
	graph::Node n2("n2");
	graph::Node n3("n3");

	// Directed
	DirectedGraphImpl dg;
	dg.addNode(n1);
	dg.addNode(n2);
	dg.addNode(n3);
	dg.addEdge(graph::DirectedEdge(n1, n2));
	dg.addEdge(graph::DirectedEdge(n1, n3));

	graph::Graph dg1(dg);
	std::string tmp = alib::StringDataFactory::toString(dg1);
	graph::Graph dg2 = alib::StringDataFactory::fromString (tmp);
	std::string tmp2 = alib::StringDataFactory::toString(dg2);
	CPPUNIT_ASSERT(dg1 == dg2);

	dg.setEdgeValue(graph::DirectedEdge(n1, n2), 1);
	dg.setEdgeValue(graph::DirectedEdge(n1, n3), 2);

	graph::Graph dg1_2(dg);
	tmp = alib::StringDataFactory::toString(dg1_2);
	graph::Graph dg2_2 = alib::StringDataFactory::fromString (tmp);
	CPPUNIT_ASSERT(dg1_2 == dg2_2);

	// Undirected
	UndirectedGraphImpl ug;
	ug.addNode(n1);
	ug.addNode(n2);
	ug.addNode(n3);
	ug.addEdge(graph::UndirectedEdge(n1, n2));
	ug.addEdge(graph::UndirectedEdge(n1, n3));

	graph::Graph ug1(ug);
	tmp = alib::StringDataFactory::toString(ug1);
	graph::Graph ug2 = alib::StringDataFactory::fromString (tmp);
	CPPUNIT_ASSERT(ug1 == ug2);

	ug.setEdgeValue(graph::UndirectedEdge(n1, n2), 1);
	ug.setEdgeValue(graph::UndirectedEdge(n1, n3), 2);

	graph::Graph ug1_2(ug);
	tmp = alib::StringDataFactory::toString(ug1_2);
	graph::Graph ug2_2 = alib::StringDataFactory::fromString (tmp);
	CPPUNIT_ASSERT(ug1_2 == ug2_2);
}


#define TEST_REPRESENTATIONS(t) {\
	std::cout << "ADJACENCY_LIST" << std::endl;\
	t< graph::AdjacencyListUndirectedGraph, graph::AdjacencyListDirectedGraph >();\
	std::cout << "ADJACENCY_MATRIX" << std::endl;\
	t< graph::AdjacencyMatrixUndirectedGraph, graph::AdjacencyMatrixDirectedGraph >();\
}

void GraphTest::testStringParser()
{
	TEST_REPRESENTATIONS(testStringParser_impl);
}
