/*
 * aminimize.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <sax/FromXMLParserHelper.h>

#include <exception/CommonException.h>
#include <factory/XmlDataFactory.hpp>

#include "automaton/simplify/Minimize.h"
#include "automaton/simplify/MinimizeBrzozowski.h"

int main(int argc, char** argv) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd("Automaton minimize binary", ' ', "0.01");

		TCLAP::ValueArg<std::string> input(	"i",	"input",	"Automaton to minimize",	false,	"-",		"file");
		cmd.add( input );

		std::vector<std::string> algorithms {"hopcroft", "brzozowski" };
		TCLAP::ValuesConstraint<std::string> algorithmVals( algorithms );
		TCLAP::ValueArg<std::string> algorithm(	"a",	"algorithm",	"Use specific algorithm. Hopcroft default.",	false,	"hopcroft",	&algorithmVals);
		cmd.add( algorithm );

		TCLAP::SwitchArg measure(		"m",	"measure",	"Measure times",		false);
		cmd.add( measure );

		TCLAP::SwitchArg verbose(		"v",	"verbose",	"Be verbose",			false);
		cmd.add( verbose );

		cmd.parse(argc, argv);

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		measurements::start("Overal", measurements::Type::OVERALL);
		measurements::start("Input read", measurements::Type::AUXILIARY);

		automaton::Automaton automaton = alib::XmlDataFactory::fromTokens (sax::FromXMLParserHelper::parseInput(input));

		measurements::end();
		measurements::start("Algorithm", measurements::Type::MAIN);

		if(algorithm.getValue() == "hopcroft") {
			automaton::Automaton res = automaton::simplify::Minimize::minimize(automaton);

			measurements::end();
			measurements::start("Output write", measurements::Type::AUXILIARY);

			alib::XmlDataFactory::toStdout(res);
		} else if(algorithm.getValue() == "brzozowski") {
			automaton::Automaton res = automaton::simplify::MinimizeBrzozowski::minimize(automaton);

			measurements::end();
			measurements::start("Output write", measurements::Type::AUXILIARY);

			alib::XmlDataFactory::toStdout(res);
		} else {
			throw exception::CommonException("Unknown algorithm selected.");
		}

		measurements::end();
		measurements::end();

		if(measure.getValue()) ext::cmeasure << measurements::results() << std::endl;

		return 0;
	} catch (const exception::CommonException& exception) {
		alib::XmlDataFactory::toStdout(exception);
		return 1;
	} catch (const TCLAP::ArgException& exception) {
		std::cout << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch (...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
