/*
 * aquery.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Jan Travnicek
 */

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <measure>
#include <vector>
#include <sax/FromXMLParserHelper.h>

#include <factory/XmlDataFactory.hpp>
#include <exception/CommonException.h>
#include <string/String.h>

#include <stringology/query/SuffixTrieFactors.h>
#include <stringology/query/SuffixArrayFactors.h>
#include <stringology/query/PositionHeapFactors.h>
#include <stringology/query/BitParallelismFactors.h>
#include <stringology/query/CompressedBitParallelismFactors.h>
#include <arbology/query/CompressedBitParallelismPatterns.h>
#include <arbology/query/FullAndLinearIndexPatterns.h>
#include <arbology/query/NonlinearFullAndLinearIndexPatterns.h>

int main ( int argc, char * argv[] ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "Stringology algorithm access binary", ' ', "0.01" );

		std::vector < std::string > allowed;
		allowed.push_back ( "suffixTrieFactors" );
		allowed.push_back ( "suffixArrayFactors" );
		allowed.push_back ( "positionHeapFactors" );
		allowed.push_back ( "bitParallelismFactors" );
		allowed.push_back ( "compressedBitParallelismFactors" );
		allowed.push_back ( "compressedBitParallelismPatterns" );
		allowed.push_back ( "fullAndLinearIndexPatterns" );
		allowed.push_back ( "nonlinearFullAndLinearIndexPatterns" );
		TCLAP::ValuesConstraint < std::string > allowedVals ( allowed );

		TCLAP::ValueArg < std::string > query ( "q", "query", "Query index", false, "exactFactorMatch", & allowedVals );
		cmd.add ( query );

		TCLAP::ValueArg <std::string > indexInput ( "i", "index", "Index to query", false, "-", "file");
		cmd.add( indexInput );

		TCLAP::MultiArg < std::string > patternInput ( "p", "pattern", "Pattern object from file", false, "file" );
		cmd.add ( patternInput );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if(verbose.isSet())
			common::GlobalData::verbose = true;
		if(measure.isSet())
			common::GlobalData::measure = true;

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );

		if ( query.getValue ( ) == "suffixTrieFactors" ) {
			indexes::stringology::SuffixTrie < DefaultSymbolType > suffixTrie = alib::XmlDataFactory::fromTokens ( sax::FromXMLParserHelper::parseInput ( indexInput ) );
			string::String pattern = alib::XmlDataFactory::fromTokens ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = stringology::query::SuffixTrieFactors::query ( suffixTrie, pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( query.getValue ( ) == "suffixArrayFactors" ) {
			indexes::stringology::SuffixArray < > suffixArray = alib::XmlDataFactory::fromTokens ( sax::FromXMLParserHelper::parseInput ( indexInput ) );
			string::String pattern = alib::XmlDataFactory::fromTokens ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = stringology::query::SuffixArrayFactors::query ( suffixArray, pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( query.getValue ( ) == "positionHeapFactors" ) {
			indexes::stringology::PositionHeap < > positionHeap = alib::XmlDataFactory::fromTokens ( sax::FromXMLParserHelper::parseInput ( indexInput ) );
			string::String pattern = alib::XmlDataFactory::fromTokens ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = stringology::query::PositionHeapFactors::query ( positionHeap, pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( query.getValue ( ) == "bitParallelismFactors" ) {
			indexes::stringology::BitParallelIndex < > bitParallelIndex = alib::XmlDataFactory::fromTokens ( sax::FromXMLParserHelper::parseInput ( indexInput ) );
			string::String pattern = alib::XmlDataFactory::fromTokens ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = stringology::query::BitParallelismFactors::query ( bitParallelIndex, pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( query.getValue ( ) == "compressedBitParallelismFactors" ) {
			indexes::stringology::CompressedBitParallelIndex < > compressedBitParallelIndex = alib::XmlDataFactory::fromTokens ( sax::FromXMLParserHelper::parseInput ( indexInput ) );
			string::String pattern = alib::XmlDataFactory::fromTokens ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = stringology::query::CompressedBitParallelismFactors::query ( compressedBitParallelIndex, pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( query.getValue ( ) == "compressedBitParallelismPatterns" ) {
			indexes::arbology::CompressedBitParallelTreeIndex < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > compressedBitParallelTreeIndex = alib::XmlDataFactory::fromTokens ( sax::FromXMLParserHelper::parseInput ( indexInput ) );
			tree::RankedTreeWrapper pattern = alib::XmlDataFactory::fromTokens ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = arbology::query::CompressedBitParallelismPatterns::query ( compressedBitParallelTreeIndex, pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( query.getValue ( ) == "fullAndLinearIndexPatterns" ) {
			indexes::arbology::FullAndLinearIndex < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > fullAndLinearIndex = alib::XmlDataFactory::fromTokens ( sax::FromXMLParserHelper::parseInput ( indexInput ) );
			tree::RankedTreeWrapper pattern = alib::XmlDataFactory::fromTokens ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = arbology::query::FullAndLinearIndexPatterns::query ( fullAndLinearIndex, pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else if ( query.getValue ( ) == "nonlinearFullAndLinearIndexPatterns" ) {
			indexes::arbology::NonlinearFullAndLinearIndex < common::ranked_symbol < DefaultSymbolType, DefaultRankType > > nonlinearFullAndLinearIndex = alib::XmlDataFactory::fromTokens ( sax::FromXMLParserHelper::parseInput ( indexInput ) );
			tree::RankedTreeWrapper pattern = alib::XmlDataFactory::fromTokens ( std::move ( sax::FromXMLParserHelper::parseInput(true, patternInput).front ( ) ) );

			measurements::end ( );
			measurements::start ( "Algorithm", measurements::Type::MAIN );

			std::set < unsigned > res = arbology::query::NonlinearFullAndLinearIndexPatterns::query ( nonlinearFullAndLinearIndex, pattern );

			measurements::end ( );
			measurements::start ( "Output write", measurements::Type::AUXILIARY );

			alib::XmlDataFactory::toStdout ( res );
		} else {
			throw exception::CommonException ( "Invalid algorithm" );
		}

		measurements::end ( );
		measurements::end ( );

		if ( measure.getValue ( ) ) ext::cmeasure << measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::CommonException & exception ) {
		alib::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		std::cout << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		std::cerr << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
